@@ -1,11 +1,11 @@
 /* Bluefish HTML Editor
  * bftextview2_langmgr.c
  *
- * Copyright (C) 2008,2009 Olivier Sessink
+ * Copyright (C) 2008,2009,2010,2011 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <libxml/xmlreader.h>
@@ -37,16 +36,18 @@ typedef struct {
 	GHashTable *contexts;
 	/*GHashTable *setoptions;*/
 	Tscantable *st; /* while not finished */
-	GList *comments; /* while not finished */
-	Tcomment *line; /* while not finished */
-	Tcomment *block;/* while not finished */
+	GHashTable *commentid_table; /* a hash table with a comment ID string as key and an integer position in the GArray as value */
+/*	Tcomment *line; / * while not finished * /
+	Tcomment *block;/ * while not finished */
 	gchar *smartindentchars;
+	gchar *smartoutdentchars;
 	Tbflang *bflang;
 	gboolean load_completion;
 	gboolean load_reference;
 	gboolean autoclose_tags;
 #ifdef HAVE_LIBENCHANT
 	gboolean default_spellcheck;
+	gboolean spell_decode_entities;
 #endif
 } Tbflangparsing;
 
@@ -61,6 +62,7 @@ typedef struct {
 	GHashTable *bflang_options; /* key: a NULL terminated char **array with first value the language name,
 											second value the option name, third NULL
 											and as value a gchar * with the value of the option */
+	GtkTextTag **highlight_tags; /* text used for highlighting, so not _needscanning_ or _needspellcheck_ or _folded_ and such */
 #ifdef HAVE_LIBENCHANT
 	GtkTextTag **need_spellcheck_tags;
 	GtkTextTag **no_spellcheck_tags;
@@ -173,7 +175,7 @@ static void langmgr_load_default_styles(void) {
 		{"special-function","#990000","","","","0",NULL},
 		{"keyword","#000000","","1","0","0",NULL},
 		{"special-keyword","#005050","","1","","0",NULL},
-		{"value","#0000FF","","0","0","0",NULL},
+		{"value","#0000FF","","0","0","1",NULL},
 		{"special-value","#0000FF","","1","","0",NULL},
 		{"variable","#990000","","1","0","0",NULL},
 		{"operator","#C86400","","0","0","0",NULL},
@@ -194,9 +196,20 @@ static void langmgr_load_default_styles(void) {
 	}
 }
 
+static GtkTextTag **texttag_array_from_list(GList *thelist) {
+	gint i=0;
+	GList *tmplist;
+	GtkTextTag **retval = (GtkTextTag **)g_new0(gpointer, g_list_length(thelist)+1);
+	for (tmplist = g_list_first(thelist);tmplist;tmplist=tmplist->next) {
+		retval[i] = tmplist->data;
+		i++;
+	}
+	g_list_free(thelist);
+	return retval;
+}
+
 void langmgr_reload_user_styles(void) {
-	GList *tmplist, *needlist=NULL,*noscanlist=NULL;
-	gint i;
+	GList *tmplist, *needlist=NULL,*noscanlist=NULL,*highlightlist=NULL;
 
 	if (main_v->props.textstyles == NULL) {
 		langmgr_load_default_styles();
@@ -207,8 +220,9 @@ void langmgr_reload_user_styles(void) {
 	for (tmplist = g_list_last(main_v->props.textstyles);tmplist;tmplist=tmplist->prev) {
 		GtkTextTag *tag;
 		gchar **arr = (gchar **)tmplist->data;
-		if (count_array(arr)==6) {
+		if (g_strv_length(arr)==6) {
 			tag = langmrg_create_style(arr[0], arr[1], arr[2], arr[3], arr[4]);
+			highlightlist = g_list_prepend(highlightlist, tag);
 #ifdef HAVE_LIBENCHANT
 			if (arr[5][0]=='1')
 				needlist = g_list_prepend(needlist, tag);
@@ -217,25 +231,10 @@ void langmgr_reload_user_styles(void) {
 #endif
 		}
 	}
+	langmgr.highlight_tags = texttag_array_from_list(highlightlist);
 #ifdef HAVE_LIBENCHANT
-	langmgr.need_spellcheck_tags = (GtkTextTag **) g_new0(gpointer, g_list_length(needlist)+1);
-	i=0;
-	for (tmplist = g_list_first(needlist);tmplist;tmplist=tmplist->next) {
-		/*DBG_SPELL("tag %p in need_spellcheck_tags[%d]\n",tmplist->data,i);*/
-		langmgr.need_spellcheck_tags[i] = tmplist->data;
-		i++;
-	}
-	DBG_SPELL("have %d items in need_spellcheck_tags %p\n",i,langmgr.need_spellcheck_tags);
-	g_list_free(needlist);
-	langmgr.no_spellcheck_tags = (GtkTextTag **) g_new0(gpointer, g_list_length(noscanlist)+1);
-	i=0;
-	for (tmplist = g_list_first(noscanlist);tmplist;tmplist=tmplist->next) {
-		/*DBG_SPELL("tag %p in no_spellcheck_tags[%d]\n",tmplist->data,i);*/
-		langmgr.no_spellcheck_tags[i] = tmplist->data;
-		i++;
-	}
-	DBG_SPELL("have %d items in no_spellcheck_tags %p\n",i,langmgr.no_spellcheck_tags);
-	g_list_free(noscanlist);
+	langmgr.need_spellcheck_tags = texttag_array_from_list(needlist);
+	langmgr.no_spellcheck_tags = texttag_array_from_list(noscanlist);
 #endif
 }
 
@@ -309,6 +308,7 @@ GtkTextTag *langmrg_lookup_tag_highlight(const gchar *lang, const gchar *highlig
 
 static void foreachdoc_lcb(Tdocument *doc, gpointer data) {
 	if (BLUEFISH_TEXT_VIEW(doc->view)->bflang == data) {
+/*		g_print("calling bluefish_text_view_rescan for %p\n",doc);*/
 		bluefish_text_view_rescan(BLUEFISH_TEXT_VIEW(doc->view));
 	}
 }
@@ -319,16 +319,17 @@ static gboolean build_lang_finished_lcb(gpointer data)
 	Tbflangparsing *bfparser=data;
 	if (bfparser->st) {
 		bfparser->bflang->st = bfparser->st;
-		bfparser->bflang->comments = bfparser->comments;
-		bfparser->bflang->line = bfparser->line;
-		bfparser->bflang->block = bfparser->block;
+		/*bfparser->bflang->line = bfparser->line;
+		bfparser->bflang->block = bfparser->block;*/
 /*		g_print("build_lang_finished_lcb, bflang %p, line=%p, block=%p\n",bfparser->bflang, bfparser->bflang->line, bfparser->bflang->block);*/
 	} else {
 		bfparser->bflang->no_st = TRUE;
 	}
 	bfparser->bflang->smartindentchars = bfparser->smartindentchars;
+	bfparser->bflang->smartoutdentchars = bfparser->smartoutdentchars;
 #ifdef HAVE_LIBENCHANT
 	bfparser->bflang->default_spellcheck = bfparser->default_spellcheck;
+	bfparser->bflang->spell_decode_entities = bfparser->spell_decode_entities;
 #endif
 	bfparser->bflang->parsing=FALSE;
 	DBG_PARSING("build_lang_finished_lcb..\n");
@@ -339,6 +340,7 @@ static gboolean build_lang_finished_lcb(gpointer data)
 	/* cleanup the parsing structure */
 	g_hash_table_destroy(bfparser->patterns);
 	g_hash_table_destroy(bfparser->contexts);
+	g_hash_table_destroy(bfparser->commentid_table);
 	/*g_hash_table_destroy(bfparser->setoptions);*/
 	g_slice_free(Tbflangparsing,bfparser);
 	return FALSE;
@@ -426,13 +428,13 @@ static void process_header(xmlTextReaderPtr reader, Tbflang *bflang) {
 						/* the user-set style does not exist, create an empty user-set style */
 						gchar *arr[] = {use_textstyle,"","","","",NULL};
 						g_warning("textstyle %s is set by the user but does not exist\n",use_textstyle);
-						main_v->props.textstyles = g_list_prepend(main_v->props.textstyles, duplicate_stringarray(arr));
+						main_v->props.textstyles = g_list_prepend(main_v->props.textstyles, g_strdupv(arr));
 					}
 				} else if (style) { /* no textstyle was configured, set the provided style */
 					if (!gtk_text_tag_table_lookup(langmgr.tagtable,style)) {
 						gchar *arr[] = {style,"","","","",NULL};
 						g_warning("textstyle %s is set in the language file but does not exist\n",style);
-						main_v->props.textstyles = g_list_prepend(main_v->props.textstyles, duplicate_stringarray(arr));
+						main_v->props.textstyles = g_list_prepend(main_v->props.textstyles, g_strdupv(arr));
 					}
 					g_hash_table_insert(langmgr.configured_styles,array_from_arglist(bflang->name,name,NULL),g_strdup(style));
 					main_v->props.highlight_styles = g_list_prepend(main_v->props.highlight_styles, array_from_arglist(bflang->name,name,style,NULL));
@@ -462,6 +464,7 @@ static guint16 process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing
 	gchar *pattern=NULL, *idref=NULL, *highlight=NULL, *blockstartelement=NULL, *blockhighlight=NULL, *class=NULL, *notclass=NULL, *autocomplete_append=NULL, *autocomplete_string=NULL, *id=NULL;
 	gboolean case_insens=FALSE, is_regex=FALSE, starts_block=FALSE, ends_block=FALSE, is_empty, autocomplete=FALSE, tagclose_from_blockstack=FALSE;
 	gint ends_context=0,autocomplete_backup_cursor=0;
+	gint identifier_mode;
 	is_empty = xmlTextReaderIsEmptyElement(reader);
 
 
@@ -487,6 +490,7 @@ static guint16 process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing
 			set_string_if_attribute_name(reader,aname,(xmlChar *)"autocomplete_append",&autocomplete_append);
 			set_integer_if_attribute_name(reader, aname, (xmlChar *)"autocomplete_backup_cursor", &autocomplete_backup_cursor);
 		}
+		set_integer_if_attribute_name(reader, aname, (xmlChar *)"identifier_mode", &identifier_mode);
 		xmlFree(aname);
 	}
 	tmp = lookup_user_option(bfparser->bflang->name,class);
@@ -518,7 +522,10 @@ static guint16 process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing
 			}
 			matchnum = add_keyword_to_scanning_table(bfparser->st, pattern, bfparser->bflang->name, highlight?highlight:ih_highlight, blockhighlight,is_regex?is_regex:ih_is_regex,case_insens?case_insens:ih_case_insens, context, nextcontext
 					, starts_block, ends_block, blockstartelementum
-					, tagclose_from_blockstack /* the very special case for the generix xml tag close pattern */);
+					, tagclose_from_blockstack /* the very special case for the generix xml tag close pattern */
+					, identifier_mode);
+			/* BUG: if ih_autocomplete is enabled, but this element explicitly disables autocomplete 
+			the next code will still enable autocompletion */
 			if (autocomplete?autocomplete:ih_autocomplete) {
 				match_add_autocomp_item(bfparser->st, matchnum, autocomplete_string
 										, autocomplete_append?autocomplete_append:ih_autocomplete_append
@@ -666,13 +673,13 @@ static guint16 process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing *bfp
 				gchar *tmp2 = g_strconcat("></",tag,">",NULL);
 				matchnum = add_keyword_to_scanning_table(bfparser->st, tmp,bfparser->bflang->name
 							,highlight?highlight:ih_highlight,NULL, FALSE, case_insens, context
-							, contexttag, TRUE, FALSE, 0, 0);
+							, contexttag, TRUE, FALSE, 0, TRUE, 0);
 				match_add_autocomp_item(bfparser->st, matchnum, NULL, tmp2, strlen(tag)+3);
 				g_free(tmp2);
 			} else {
 				matchnum = add_keyword_to_scanning_table(bfparser->st, tmp,bfparser->bflang->name
 							,highlight?highlight:ih_highlight,NULL, FALSE, case_insens, context
-							, contexttag, TRUE, FALSE, 0, 0);
+							, contexttag, TRUE, FALSE, 0, TRUE, 0);
 				match_add_autocomp_item(bfparser->st, matchnum, NULL, autocomplete_append?autocomplete_append:ih_autocomplete_append, autocomplete_backup_cursor?autocomplete_backup_cursor:ih_autocomplete_backup_cursor);
 			}
 			DBG_PARSING("insert tag %s into hash table with matchnum %d\n",id?id:tmp,matchnum);
@@ -691,7 +698,7 @@ static guint16 process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing *bfp
 					while (*tmp2) {
 						guint16 attrmatch = add_keyword_to_scanning_table(bfparser->st, *tmp2,bfparser->bflang->name
 									,attribhighlight?attribhighlight:ih_attribhighlight
-									,NULL, FALSE, TRUE, contexttag, 0, FALSE, FALSE, 0, 0);
+									,NULL, FALSE, TRUE, contexttag, 0, FALSE, FALSE, 0, 0, 0);
 						match_add_autocomp_item(bfparser->st, attrmatch, NULL, attrib_autocomplete_append?attrib_autocomplete_append:ih_attrib_autocomplete_append, attrib_autocomplete_backup_cursor?attrib_autocomplete_backup_cursor:ih_attrib_autocomplete_backup_cursor);
 						match_autocomplete_reference(bfparser->st,attrmatch,contexttag);
 						tmp2++;
@@ -710,7 +717,7 @@ static guint16 process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing *bfp
 					compile_existing_match(bfparser->st,matchstring, contexttag);
 				} else {
 					/*g_print("adding new tag string %s to context %d and highlight %s\n",internal_tag_string_d,contexttag,stringhighlight);*/
-					matchstring = add_keyword_to_scanning_table(bfparser->st, "\"[^\"]*\"", bfparser->bflang->name, stringhighlight,NULL,TRUE, FALSE, contexttag, 0, FALSE, FALSE, 0, 0);
+					matchstring = add_keyword_to_scanning_table(bfparser->st, "\"[^\"]*\"", bfparser->bflang->name, stringhighlight,NULL,TRUE, FALSE, contexttag, 0, FALSE, FALSE, 0, 0, 0);
 					g_hash_table_insert(bfparser->patterns, g_strdup(internal_tag_string_d), GINT_TO_POINTER((gint)matchstring));
 				}
 				matchstring = GPOINTER_TO_INT(g_hash_table_lookup(bfparser->patterns, internal_tag_string_s));
@@ -719,19 +726,19 @@ static guint16 process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing *bfp
 					compile_existing_match(bfparser->st,matchstring, contexttag);
 				} else {
 					/*g_print("adding new tag string %s to context %d and highlight %s\n",internal_tag_string_s,contexttag,stringhighlight);*/
-					matchstring = add_keyword_to_scanning_table(bfparser->st, "'[^']*'", bfparser->bflang->name, stringhighlight,NULL,TRUE, FALSE, contexttag, 0, FALSE, FALSE, 0, 0);
+					matchstring = add_keyword_to_scanning_table(bfparser->st, "'[^']*'", bfparser->bflang->name, stringhighlight,NULL,TRUE, FALSE, contexttag, 0, FALSE, FALSE, 0, 0, 0);
 					g_hash_table_insert(bfparser->patterns, g_strdup(internal_tag_string_s), GINT_TO_POINTER((gint)matchstring));
 				}
 
 				if (!sgml_shorttag) {
 					guint16 tmpnum;
-					tmpnum = add_keyword_to_scanning_table(bfparser->st, "/>", bfparser->bflang->name, highlight?highlight:ih_highlight, NULL, FALSE, FALSE, contexttag, -1, FALSE, TRUE, -1, 0);
+					tmpnum = add_keyword_to_scanning_table(bfparser->st, "/>", bfparser->bflang->name, highlight?highlight:ih_highlight, NULL, FALSE, FALSE, contexttag, -1, FALSE, TRUE, -1, 0, 0);
 					if (bfparser->autoclose_tags)
 						match_add_autocomp_item(bfparser->st, tmpnum, NULL,NULL, 0);
 					match_autocomplete_reference(bfparser->st,tmpnum,contexttag);
 				}
 				tmp = no_close ? NULL : g_strconcat("</",tag,">",NULL);
-				starttagmatch = add_keyword_to_scanning_table(bfparser->st, ">", bfparser->bflang->name, highlight?highlight:ih_highlight, NULL, FALSE, FALSE, contexttag, -1, FALSE, FALSE, 0, 0);
+				starttagmatch = add_keyword_to_scanning_table(bfparser->st, ">", bfparser->bflang->name, highlight?highlight:ih_highlight, NULL, FALSE, FALSE, contexttag, -1, FALSE, FALSE, 0, 0, 0);
 				if (bfparser->autoclose_tags&&!no_close)
 					match_add_autocomp_item(bfparser->st, starttagmatch, NULL,tmp,tmp?strlen(tmp):0);
 				if (tmp) g_free(tmp);
@@ -782,7 +789,7 @@ static guint16 process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing *bfp
 			match_autocomplete_reference(bfparser->st,matchnum,context);
 			if (!no_close) {
 				tmp = g_strconcat("</",tag,">",NULL);
-				endtagmatch = add_keyword_to_scanning_table(bfparser->st, tmp, bfparser->bflang->name,highlight?highlight:ih_highlight,NULL, FALSE, case_insens, innercontext, (innercontext==context)?0:-2, FALSE, TRUE, matchnum, 0);
+				endtagmatch = add_keyword_to_scanning_table(bfparser->st, tmp, bfparser->bflang->name,highlight?highlight:ih_highlight,NULL, FALSE, case_insens, innercontext, (innercontext==context)?0:-2, FALSE, TRUE, matchnum, 0, 0);
 				match_add_autocomp_item(bfparser->st, endtagmatch, NULL, NULL, 0);
 				match_autocomplete_reference(bfparser->st,endtagmatch,innercontext);
 				g_hash_table_insert(bfparser->patterns, g_strdup(tmp), GINT_TO_POINTER((gint)endtagmatch));
@@ -838,7 +845,6 @@ static void process_scanning_group(xmlTextReaderPtr reader, Tbflangparsing *bfpa
 	tmp2 = lookup_user_option(bfparser->bflang->name,notclass);
 	if ((class && tmp && tmp[0]!='1')||(notclass && tmp2 && tmp2[0]=='1')){
 		DBG_PARSING("group disabled, class=%s, notclass=%s, skip to end of group, my depth=%d\n",class,notclass,depth);
-		printf("group disabled, class=%s, notclass=%s, skip to end of group, my depth=%d\n",class,notclass,depth);
 		skip_to_end_tag(reader, depth);
 	} else {
 		while (xmlTextReaderRead(reader)==1) {
@@ -894,8 +900,36 @@ static void process_scanning_group(xmlTextReaderPtr reader, Tbflangparsing *bfpa
 	/*g_free(attribhighlight);*/
 }
 
+static guint8 get_first_comment_of_type(Tbflangparsing *bfparser, Tcomment_type type) {
+	gint i=0;
+	while (g_array_index(bfparser->st->comments, Tcomment, i).so) {
+		if (g_array_index(bfparser->st->comments, Tcomment, i).type == type)
+			return i;
+		i++;
+	}
+	return COMMENT_INDEX_NONE;
+}
+
+static void set_commentid(Tbflangparsing *bfparser, gboolean topevel_context, guint8 *toset, const gchar *string, Tcomment_type type) {
+	*toset = COMMENT_INDEX_INHERIT;
+	if (string) {
+		if (g_strcmp0(string, "none")==0) {
+			*toset = COMMENT_INDEX_NONE;
+		} else {
+			gint tmp = GPOINTER_TO_INT(g_hash_table_lookup(bfparser->commentid_table, string));
+			if (tmp != 0)
+				*toset = (guint8)(tmp-1);
+		}
+	}
+	if (topevel_context && *toset == COMMENT_INDEX_INHERIT) {
+		/* set the first block comment as default for the toplevel context */
+		*toset = get_first_comment_of_type(bfparser, type);
+	}
+}
+
+
 static gint16 process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing *bfparser, GQueue *contextstack) {
-	gchar *symbols=NULL, *highlight=NULL, *id=NULL, *idref=NULL;
+	gchar *symbols=NULL, *highlight=NULL, *id=NULL, *idref=NULL, *commentid_block=NULL, *commentid_line=NULL;
 	gboolean autocomplete_case_insens=FALSE,is_empty;
 #ifdef HAVE_LIBENCHANT_OLD
 	gboolean spellcheck=FALSE;
@@ -908,6 +942,8 @@ static gint16 process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing *
 		set_string_if_attribute_name(reader,aname,(xmlChar *)"idref",&idref);
 		set_string_if_attribute_name(reader,aname,(xmlChar *)"symbols",&symbols);
 		set_string_if_attribute_name(reader,aname,(xmlChar *)"highlight",&highlight);
+		set_string_if_attribute_name(reader,aname,(xmlChar *)"commentid_block",&commentid_block);
+		set_string_if_attribute_name(reader,aname,(xmlChar *)"commentid_line",&commentid_line);
 #ifdef HAVE_LIBENCHANT_OLD
 		set_boolean_if_attribute_name(reader,aname,(xmlChar *)"spellcheck",&spellcheck);
 #endif
@@ -933,6 +969,17 @@ static gint16 process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing *
 		DBG_PARSING("insert context %s into hash table as %d\n",id,context);
 		g_hash_table_insert(bfparser->contexts, g_strdup(id), GINT_TO_POINTER(context));
 	}
+	set_commentid(bfparser
+					, (g_queue_get_length(contextstack)==1)
+					, &g_array_index(bfparser->st->contexts, Tcontext, context).comment_block
+					, commentid_block
+					, comment_type_block);
+	set_commentid(bfparser
+					, (g_queue_get_length(contextstack)==1)
+					, &g_array_index(bfparser->st->contexts, Tcontext, context).comment_line
+					, commentid_line
+					, comment_type_line);
+
 	g_free(id);
 	g_free(symbols);
 	/*g_free(highlight); stored in the structure */
@@ -978,7 +1025,8 @@ static gpointer build_lang_thread(gpointer data)
 		g_hash_table_insert(bfparser->setoptions,g_strdup(tmplist->data),GINT_TO_POINTER(1));
 	}*/
 	bfparser->st = scantable_new(bflang->size_table,bflang->size_matches,bflang->size_contexts);
-
+	bfparser->commentid_table = g_hash_table_new_full(g_str_hash, g_str_equal,xmlFree,NULL);
+	
 	tmp = lookup_user_option(bflang->name, "load_reference");
 	bfparser->load_reference = !(tmp && tmp[0]=='0');
 	tmp = lookup_user_option(bflang->name, "load_completion");
@@ -1035,40 +1083,45 @@ static gpointer build_lang_thread(gpointer data)
 			while (xmlTextReaderRead(reader)==1) {
 				xmlChar *name2 = xmlTextReaderName(reader);
 				if (xmlStrEqual(name2,(xmlChar *)"comment")) {
-					gchar *type=NULL;
-					Tcomment *com = g_slice_new0(Tcomment);
+					gchar *typestr=NULL, *id=NULL;
+					Tcomment com = {NULL, NULL, 0};
 					while (xmlTextReaderMoveToNextAttribute(reader)) {
 						xmlChar *aname = xmlTextReaderName(reader);
-						set_string_if_attribute_name(reader,aname,(xmlChar *)"start", &com->so);
-						set_string_if_attribute_name(reader,aname,(xmlChar *)"end", &com->eo);
-						set_string_if_attribute_name(reader,aname,(xmlChar *)"type", &type);
+						set_string_if_attribute_name(reader,aname,(xmlChar *)"start", &com.so);
+						set_string_if_attribute_name(reader,aname,(xmlChar *)"end", &com.eo);
+						set_string_if_attribute_name(reader,aname,(xmlChar *)"type", &typestr);
+						set_string_if_attribute_name(reader,aname,(xmlChar *)"id", &id);
 						xmlFree(aname);
 					}
-					if (com && com->so && type && (strcmp(type, "line")==0 || com->eo)) {
-						com->type=strcmp(type, "line")==0 ? comment_type_line : comment_type_block;
-						bfparser->comments = g_list_prepend(bfparser->comments, com);
-						if (com->type==comment_type_line && bfparser->line==NULL)
-							bfparser->line=com;
-						else if (com->type==comment_type_block && bfparser->block==NULL)
-							bfparser->block=com;
-						/*g_print("adding comment %s, line=%p, block=%p\n",com->so,bfparser->line,bfparser->block);*/
+					com.type= g_strcmp0(typestr, "block")==0 ? comment_type_block : comment_type_line; 
+					if (com.so && (com.type==comment_type_line || com.eo)) {
+						g_array_append_val(bfparser->st->comments, com);
+						if (id)
+							g_hash_table_insert(bfparser->commentid_table, id, GINT_TO_POINTER(bfparser->st->comments->len));
 					} else {
-						if (com->so) xmlFree(com->so);
-						if (com->eo) xmlFree(com->eo);
-						g_slice_free(Tcomment, com);
+						xmlFree(com.so);
+						xmlFree(com.eo);
+						xmlFree(id);
 					}
-					xmlFree(type);
+					xmlFree(typestr);
 				} else if (xmlStrEqual(name2,(xmlChar *)"smartindent")) {
 					while (xmlTextReaderMoveToNextAttribute(reader)) {
 						xmlChar *aname = xmlTextReaderName(reader);
 						set_string_if_attribute_name(reader,aname,(xmlChar *)"characters", &bfparser->smartindentchars);
 						xmlFree(aname);
 					}
+				} else if (xmlStrEqual(name2,(xmlChar *)"smartoutdent")) {
+					while (xmlTextReaderMoveToNextAttribute(reader)) {
+						xmlChar *aname = xmlTextReaderName(reader);
+						set_string_if_attribute_name(reader,aname,(xmlChar *)"characters", &bfparser->smartoutdentchars);
+						xmlFree(aname);
+					}
 #ifdef HAVE_LIBENCHANT
 				} else if (xmlStrEqual(name2,(xmlChar *)"default_spellcheck")) {
 					while (xmlTextReaderMoveToNextAttribute(reader)) {
 						xmlChar *aname = xmlTextReaderName(reader);
 						set_boolean_if_attribute_name(reader,aname,(xmlChar *)"enabled", &bfparser->default_spellcheck);
+						set_boolean_if_attribute_name(reader,aname,(xmlChar *)"spell_decode_entities", &bfparser->spell_decode_entities);
 						xmlFree(aname);
 					}
 #endif
@@ -1096,8 +1149,8 @@ static gpointer build_lang_thread(gpointer data)
 		bflang->tags = bftextview2_scantable_rematch_highlights(bfparser->st, bflang->name);
 	}
 	DBG_PARSING("build_lang_thread finished bflang=%p\n",bflang);
-	/* when done call mainloop */
-	g_idle_add_full(G_PRIORITY_LOW,build_lang_finished_lcb, bfparser,NULL);
+	/* when done call mainloop, see the top of bftextview2.c why we use priority 122 here */
+	g_idle_add_full(122,build_lang_finished_lcb, bfparser,NULL);
 	return bflang;
 }
 
@@ -1174,6 +1227,17 @@ GList *langmgr_get_languages_mimetypes(void) {
 	}
 	return retlist;
 }
+
+gboolean langmgr_in_highlight_tags(GtkTextTag *tag) {
+	gint i=0;
+	while (langmgr.highlight_tags[i]) {
+		if (langmgr.highlight_tags[i] == tag)
+			return TRUE;
+		i++;
+	}
+	return FALSE;
+}
+
 #ifdef HAVE_LIBENCHANT
 GtkTextTag **langmgr_no_spellcheck_tags(void) {
 	return langmgr.no_spellcheck_tags;
@@ -1181,7 +1245,6 @@ GtkTextTag **langmgr_no_spellcheck_tags(void) {
 GtkTextTag **langmgr_need_spellcheck_tags(void) {
 	return langmgr.need_spellcheck_tags;
 }
-
 #endif
 
 GList *langmgr_get_languages(void) {
@@ -1237,7 +1300,7 @@ static void bflang_cleanup(Tbflang *bflang) {
 	free_stringlist(bflang->mimetypes);
 	g_slice_free(Tbflang,bflang);
 }
-
+#ifdef MEMORY_LEAK_DEBUG
 void langmgr_cleanup(void) {
 	GList *tmplist = g_list_first(langmgr.bflang_list);
 	while (tmplist) {
@@ -1246,6 +1309,7 @@ void langmgr_cleanup(void) {
 	}
 	g_list_free(langmgr.bflang_list);
 }
+#endif
 
 static void register_bflanguage(Tbflang *bflang) {
 	if (bflang) {
@@ -1273,6 +1337,7 @@ static void scan_bflang2files(const gchar *dir) {
 	GError *error = NULL;
 	GPatternSpec *ps = g_pattern_spec_new("*.bflang2");
 	GDir *gd = g_dir_open(dir, 0, &error);
+	DEBUG_PATH("load language files from %s\n",dir);
 	if (!error) {
 		filename = g_dir_read_name(gd);
 		while (filename) {
