@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 /*#define SPELL_PROFILING*/
 
@@ -35,6 +34,7 @@
 #include "bftextview2_langmgr.h"
 #include "bftextview2_spell.h"
 #include "document.h"
+#include "xml_entity.h"
 
 /*#undef DBG_SPELL
 #define DBG_SPELL g_print*/
@@ -45,6 +45,8 @@ static EnchantBroker *eb;
 static guint loops_per_timer=1000;
 
 static gboolean bftextview2_find_region2spellcheck(BluefishTextView * btv, GtkTextBuffer *buffer, GtkTextIter *start, GtkTextIter *end) {
+	GtkTextTag *misspelled;
+	
 	/* first find a region that needs a spellcheck */
 	gtk_text_buffer_get_start_iter(buffer, start);
 	if (!gtk_text_iter_begins_tag(start,btv->needspellcheck) ) {
@@ -64,6 +66,16 @@ static gboolean bftextview2_find_region2spellcheck(BluefishTextView * btv, GtkTe
 			return FALSE;
 		}
 	}
+	/* if the region is within a misspelled word, enlarge it to the total misspelled word
+	(this fixes the situation where you add a space in the middle of "forgotthespace" and only 
+	the space is scanned again) */
+	misspelled = gtk_text_tag_table_lookup(langmgr_get_tagtable(), "_spellerror_");
+	if (gtk_text_iter_has_tag(start, misspelled) && !gtk_text_iter_begins_tag(start,misspelled)) {
+		gtk_text_iter_backward_to_tag_toggle(start, misspelled);
+	}
+	if (gtk_text_iter_has_tag(end, misspelled) && !gtk_text_iter_ends_tag(start,misspelled)) {
+		gtk_text_iter_forward_to_tag_toggle(end, misspelled);
+	}
 	return TRUE;
 }
 
@@ -105,13 +117,15 @@ void unload_spell_dictionary(Tbfwin *bfwin) {
 }
 
 static gboolean load_dictionary(Tbfwin *bfwin) {
-	unload_spell_dictionary(bfwin);
+	DBG_SPELL("load_dictionary for bfwin %p, bfwin->ed=%p\n", bfwin, bfwin->ed);
+	unload_spell_dictionary(bfwin);	
 	if (bfwin->session->spell_lang && bfwin->session->spell_lang[0]!='\0' && enchant_broker_dict_exists(eb,bfwin->session->spell_lang)) {
-	    DBG_SPELL("load_dictionary called for bfwin %p which has session->spell_lang=%s\n",bfwin, bfwin->session->spell_lang);
+	   DBG_SPELL("load_dictionary called for bfwin %p which has session->spell_lang=%s\n",bfwin, bfwin->session->spell_lang);
 		bfwin->ed = (void *)enchant_broker_request_dict(eb, bfwin->session->spell_lang);
 		DBG_SPELL("loaded dictionary %s at %p\n", bfwin->session->spell_lang, bfwin->ed);
 		return (bfwin->ed != NULL);
 	} else {
+		DBG_SPELL("load_dictionary, no setting, load first available\n");
 		bfwin->ed = NULL;
 		/* load the first existing enchant dict */
 		enchant_broker_list_dicts(eb,dicts_load_first_lcb,bfwin);
@@ -123,32 +137,31 @@ static void spellcheck_word(BluefishTextView * btv, GtkTextBuffer *buffer, GtkTe
 	gchar *tocheck;
 	
 	tocheck = gtk_text_buffer_get_text(buffer, start,end, FALSE);
+	if (!tocheck)
+		return;
+	
 	DBG_SPELL("spellcheck_word, check word %s in dictionary %p\n",tocheck,BFWIN(DOCUMENT(btv->doc)->bfwin)->ed);
 	if (enchant_dict_check((EnchantDict *)BFWIN(DOCUMENT(btv->doc)->bfwin)->ed, tocheck, strlen(tocheck)) != 0) {
 		DBG_SPELL("'%s' *not* spelled correctly!\n", tocheck);
-		gtk_text_buffer_apply_tag_by_name(buffer, "_spellerror_", start, end);
+		if (g_utf8_strchr(tocheck, -1, '&')) {
+			gchar *tocheck_conv = xmlentities2utf8(tocheck);
+			/* check for entities */
+			if (tocheck_conv && enchant_dict_check((EnchantDict *)BFWIN(DOCUMENT(btv->doc)->bfwin)->ed, tocheck_conv, strlen(tocheck_conv)) != 0) {
+				DBG_SPELL("'%s' after entity conversion (%s) *not* spelled correctly!\n", tocheck, tocheck_conv);
+				gtk_text_buffer_apply_tag_by_name(buffer, "_spellerror_", start, end);
+			} else {
+				DBG_SPELL("'%s' after entity conversion (%s) spelled correctly!\n", tocheck, tocheck_conv);
+			}
+			g_free(tocheck_conv);
+		} else {
+			gtk_text_buffer_apply_tag_by_name(buffer, "_spellerror_", start, end);
+		}
 	} else {
 		DBG_SPELL("'%s' spelled correctly!\n", tocheck);
 	}
 	g_free(tocheck);
 }
 
-/* handle apostrophe in word gracefully */ 
-static inline gboolean text_iter_forward_real_word_end(GtkTextIter *it) {
-	GtkTextIter iter;
-	if (!gtk_text_iter_forward_word_end(it))
-		return FALSE;
-	if (gtk_text_iter_get_char(it) != '\'')
-		return TRUE;
-	iter = *it;
-	if (gtk_text_iter_forward_char(&iter)) {
-		if (g_unichar_isalpha(gtk_text_iter_get_char(&iter))) {
-			return (gtk_text_iter_forward_word_end(it));
-		}
-	}
-	return TRUE;
-}
-
 #ifdef HAVE_LIBENCHANT_OLD
 static gint OLD_foundstack_needspellcheck(BluefishTextView * btv, Tfoundstack *fstack) {
 	guint16 contextnum;
@@ -354,7 +367,7 @@ static gboolean get_next_region(BluefishTextView * btv, GtkTextIter *so, GtkText
 			}
 			g_slist_free(tmpslist);
 		}
-		if (feo) {
+		if (feo || btv->bflang->default_spellcheck) {
 			*eo = iter;
 			return TRUE;
 		}
@@ -363,6 +376,86 @@ static gboolean get_next_region(BluefishTextView * btv, GtkTextIter *so, GtkText
 	return FALSE;
 }
 
+static inline gboolean forward_to_end_of_entity(GtkTextIter *iter) {
+	gint i=0;
+	GtkTextIter tmpiter=*iter;
+	while(gtk_text_iter_get_char(&tmpiter) != ';') {
+		if (!gtk_text_iter_forward_char(&tmpiter))
+			return FALSE;
+		i++;
+		if (i > 8)
+			return FALSE;
+	}
+	/* forward one more char */
+	gtk_text_iter_forward_char(&tmpiter);
+	*iter = tmpiter;
+	return TRUE;
+}
+
+/* handle apostrophe and entities in word gracefully */ 
+static inline gboolean text_iter_next_word_bounds(GtkTextIter *soword, GtkTextIter *eoword, gboolean enable_entities) {
+	gunichar uc;
+	gboolean handled_starting_entity=FALSE;
+	
+	if (!gtk_text_iter_forward_word_end(eoword))
+		return FALSE;
+	*soword = *eoword;
+	gtk_text_iter_backward_word_start(soword);
+	
+	uc = gtk_text_iter_get_char(eoword);
+	DBG_SPELL("text_iter_next_word_bounds, uc=%c\n",uc);
+	while (uc == '\'' || (enable_entities && (uc == ';' || uc == '&'))) {
+		GtkTextIter iter = *eoword;
+		DBG_SPELL("text_iter_next_word_bounds, in loop, uc=%c\n",uc);
+		if (uc == '\'' && gtk_text_iter_forward_char(&iter)) {
+			if (g_unichar_isalpha(gtk_text_iter_get_char(&iter))) {
+				gtk_text_iter_forward_word_end(eoword);
+			} else {
+				return TRUE;
+			}
+		} else if (enable_entities && uc == '&') {
+			if (!forward_to_end_of_entity(&iter))
+				return TRUE; /* no entity, return previous word end */
+			*eoword = iter;
+			if (g_unichar_isalpha(gtk_text_iter_get_char(&iter))) {
+				/* continue in the loop */
+				gtk_text_iter_forward_word_end(eoword);
+			} else {
+				/* after the entity the word stops, return TRUE */
+				return TRUE;
+			}
+			
+		} else if (enable_entities && uc == ';') {
+			GtkTextIter tmp = *soword;
+			if (!handled_starting_entity && gtk_text_iter_backward_char(&tmp) && gtk_text_iter_get_char(&tmp) == '&') {
+				/* the word probably starts with an entity */
+				*soword = tmp;
+				DBG_SPELL("text_iter_next_word_bounds, word starts with an enity, forward eoword one position\n");
+				gtk_text_iter_forward_char(eoword);
+				if (g_unichar_isalpha(gtk_text_iter_get_char(eoword))) {
+					gtk_text_iter_forward_word_end(eoword);
+				}
+				handled_starting_entity=TRUE;
+			} else {
+				tmp = *eoword;
+				gtk_text_iter_forward_char(&tmp);
+				if (g_unichar_isalpha(gtk_text_iter_get_char(&tmp))) {
+					gtk_text_iter_forward_word_end(eoword);
+				} else {
+					DBG_SPELL("text_iter_next_word_bounds, tmp uc=%c\n",gtk_text_iter_get_char(&tmp));
+					return TRUE;
+				}
+			}
+		} else {
+			/* is it possible to get here? */
+			return TRUE;
+		}
+		uc = gtk_text_iter_get_char(eoword);
+		DBG_SPELL("text_iter_next_word_bounds, new uc=%c\n",uc);
+	}
+	return TRUE;
+}
+
 gboolean bftextview2_run_spellcheck(BluefishTextView * btv) {
 	GtkTextIter so,eo,iter;
 	GtkTextBuffer *buffer;
@@ -400,16 +493,17 @@ gboolean bftextview2_run_spellcheck(BluefishTextView * btv) {
 		} else { /* no scantable */
 			eo2=eo;
 		}
+		gtk_text_buffer_remove_tag_by_name(gtk_text_view_get_buffer(GTK_TEXT_VIEW(btv)),
+				"_spellerror_", &iter, &eo2);
 		DBG_SPELL("bftextview2_run_spellcheck, loop2 from %d to %d\n",gtk_text_iter_get_offset(&iter),gtk_text_iter_get_offset(&eo2));
 		while (cont2 && (loop%loops_per_timer!=0 || g_timer_elapsed(timer,NULL)<MAX_CONTINUOUS_SPELLCHECK_INTERVAL)) { /* loop from iter to eo2 */
+			GtkTextIter wordstart=iter;
 			loop++;
 			DBG_SPELL("iter at %d, now forward to word end\n",gtk_text_iter_get_offset(&iter));
-			if (gtk_text_iter_forward_word_end(&iter) && gtk_text_iter_compare(&iter, &eo2) <= 0) {
-				GtkTextIter wordstart=iter;
-				gtk_text_iter_backward_word_start(&wordstart);
+			if (text_iter_next_word_bounds(&wordstart, &iter, btv->bflang->spell_decode_entities) && gtk_text_iter_compare(&iter, &eo2) <= 0) {
 				DBG_SPELL("iter at %d, backward wordstart at %d\n",gtk_text_iter_get_offset(&iter),gtk_text_iter_get_offset(&wordstart));
 				
-				/* handle apostrophe gracefully */
+/*
 				if (gtk_text_iter_get_char(&iter) == '\'') {
 					DBG_SPELL("handle apostrophe, forward to next word end\n");
 					if (gtk_text_iter_forward_char(&iter)) {
@@ -417,7 +511,14 @@ gboolean bftextview2_run_spellcheck(BluefishTextView * btv) {
 							gtk_text_iter_forward_word_end(&iter);
 						}
 					}
-				}
+				} else if (gtk_text_iter_get_char(&iter) == '&' && forward_to_end_of_entity(&iter)) { / * handle entities gracefully * /
+					DBG_SPELL("handle entity, forward to next word end\n");
+					if (gtk_text_iter_forward_char(&iter)) {
+						if (g_unichar_isalpha(gtk_text_iter_get_char(&iter))) {
+							gtk_text_iter_forward_word_end(&iter);
+						}
+					}
+				}*/
 				/* check word */
 #ifdef SPELL_PROFILING
 				profile_words++;
@@ -470,11 +571,28 @@ void bftextview2_spell_cleanup(void) {
 	eb=NULL;
 }
 
+static void recheck_document(Tdocument *doc) {
+	GtkTextIter start,end;
+	gtk_text_buffer_get_bounds(doc->buffer,&start,&end);
+	gtk_text_buffer_apply_tag(doc->buffer, BLUEFISH_TEXT_VIEW(doc->view)->needspellcheck, &start, &end);
+}
+
+static void recheck_bfwin(Tbfwin *bfwin) {
+	GList *tmplist;
+	for (tmplist=g_list_first(bfwin->documentlist);tmplist;tmplist=g_list_next(tmplist)) {
+		recheck_document(DOCUMENT(tmplist->data));
+	}
+	if (bfwin->current_document)
+		bluefish_text_view_rescan(BLUEFISH_TEXT_VIEW(bfwin->current_document->view));
+}
+
+
 static gboolean get_misspelled_word_at_bevent(BluefishTextView *btv, GtkTextIter *wordstart, GtkTextIter *wordend) {
 	GtkTextTag *misspelled;
 	
 	misspelled = gtk_text_tag_table_lookup(langmgr_get_tagtable(), "_spellerror_");
-	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_VIEW(btv)->buffer,wordstart,main_v->bevent_charoffset);
+	gtk_text_buffer_get_iter_at_offset(gtk_text_view_get_buffer(GTK_TEXT_VIEW(btv)),
+			wordstart, main_v->bevent_charoffset);
 	if (gtk_text_iter_has_tag(wordstart, misspelled)) {
 		*wordend=*wordstart;
 		if (gtk_text_iter_backward_to_tag_toggle(wordstart,misspelled) && gtk_text_iter_forward_to_tag_toggle(wordend,misspelled))
@@ -483,42 +601,87 @@ static gboolean get_misspelled_word_at_bevent(BluefishTextView *btv, GtkTextIter
 	return FALSE;
 }
 
+static void bftextview2_add_word_backend(BluefishTextView *btv, Tbfwin *bfwin, gboolean to_dict) {
+	GtkTextIter so,eo;
+	gchar *word;
+	if (!get_misspelled_word_at_bevent(btv, &so,&eo)) 
+		return;
+	
+	word = gtk_text_buffer_get_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(btv)), &so, &eo, FALSE);
+	if (to_dict) {
+#ifdef HAVE_LIBENCHANT_1_4
+		enchant_dict_add((EnchantDict *)bfwin->ed, word,strlen(word));
+#else
+		enchant_dict_add_to_pwl((EnchantDict *)bfwin->ed, word,strlen(word));
+#endif
+	} else {
+		enchant_dict_add_to_session((EnchantDict *)bfwin->ed, word,strlen(word));
+	}
+	recheck_bfwin(bfwin);
+}
+
+static void bftexview2_add_word_to_dict(GtkWidget *widget, gpointer data) {
+	Tdocument*doc = data;
+	bftextview2_add_word_backend(BLUEFISH_TEXT_VIEW(doc->view), BFWIN(doc->bfwin), TRUE);
+}
+
+static void bftexview2_add_word_to_ses(GtkWidget *widget, gpointer data) {
+	Tdocument*doc = data;
+	bftextview2_add_word_backend(BLUEFISH_TEXT_VIEW(doc->view), BFWIN(doc->bfwin), FALSE);
+}
+
 static void bftextview2_suggestion_menu_lcb(GtkWidget *widget, gpointer data) {
 	Tdocument *doc=data;
 	GtkTextIter wordstart,wordend;
 	if (main_v->bevent_doc != doc)
 		return;
-	DBG_SPELL("chosen %s\n",gtk_label_get_text(GTK_LABEL(GTK_BIN(widget)->child)));
+	DBG_SPELL("chosen %s\n",gtk_label_get_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(widget)))));
 	if (get_misspelled_word_at_bevent(BLUEFISH_TEXT_VIEW(doc->view), &wordstart, &wordend)) {
 		gint start,end;
 		/* no need to remove the tag because the text with this tag is deleted by the replace
 		gtk_text_buffer_remove_tag_by_name(doc->buffer, "_spellerror_", &wordstart, &wordend);*/
 		start = gtk_text_iter_get_offset(&wordstart);
 		end = gtk_text_iter_get_offset(&wordend);
-		doc_replace_text(doc, gtk_label_get_text(GTK_LABEL(GTK_BIN(widget)->child)), start, end);
+		if (BFWIN(DOCUMENT(data)->bfwin)->session->spell_insert_entities) {
+			gchar *word;
+			word = utf82xmlentities(gtk_label_get_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(widget)))), TRUE, TRUE, TRUE, TRUE, TRUE, FALSE);
+			doc_replace_text(doc, word, start, end);
+			g_free(word);
+		} else {
+			doc_replace_text(doc, gtk_label_get_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(widget)))), start, end);
+		}
 	}
 }
 
+static void mark_all_docs_needspelling(Tbfwin *bfwin) {
+	GList *tmplist;			
+	/* now mark all documents in this window with 'need_spellcheck'*/
+	for (tmplist=g_list_first(bfwin->documentlist);tmplist;tmplist=g_list_next(tmplist)) {
+		Tdocument *doc = DOCUMENT(tmplist->data);
+		GtkTextIter start,end;
+		gtk_text_buffer_get_bounds(doc->buffer,&start,&end);
+		gtk_text_buffer_apply_tag(doc->buffer, BLUEFISH_TEXT_VIEW(doc->view)->needspellcheck, &start, &end);
+	}
+	if (bfwin->current_document)
+		bluefish_text_view_rescan(BLUEFISH_TEXT_VIEW(bfwin->current_document->view));
+}
+
 static void bftextview2_preferences_menu_lcb(GtkWidget *widget, gpointer data) {
 	Tbfwin *bfwin=data;
-	if (GTK_CHECK_MENU_ITEM(widget)->active) {
+	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) {
 		if (bfwin->session->spell_lang)
 			g_free(bfwin->session->spell_lang);
-		bfwin->session->spell_lang = g_strdup(gtk_label_get_text(GTK_LABEL(GTK_BIN(widget)->child)));
+		bfwin->session->spell_lang = g_strdup(gtk_label_get_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(widget)))));
 		if (load_dictionary(bfwin)) {
-			GList *tmplist;			
-			/* now mark all documents in this window with 'need_spellcheck'*/
-			for (tmplist=g_list_first(bfwin->documentlist);tmplist;tmplist=g_list_next(tmplist)) {
-				Tdocument *doc = DOCUMENT(tmplist->data);
-				GtkTextIter start,end;
-				gtk_text_buffer_get_bounds(doc->buffer,&start,&end);
-				gtk_text_buffer_apply_tag(doc->buffer, BLUEFISH_TEXT_VIEW(doc->view)->needspellcheck, &start, &end);
-			}
-			bluefish_text_view_rescan(BLUEFISH_TEXT_VIEW(bfwin->current_document->view));
+			mark_all_docs_needspelling(bfwin);
 		}
 	}
 }
 
+static void bftexview2_spell_insert_entities(GtkWidget *widget, gpointer data) {
+	BFWIN(DOCUMENT(data)->bfwin)->session->spell_insert_entities = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
+}
+
 typedef struct {
 	Tbfwin *bfwin;
 	GtkWidget *menu;
@@ -539,6 +702,18 @@ static void list_dicts_lcb(const char * const lang_tag,const char * const provid
 	gtk_menu_shell_prepend(GTK_MENU_SHELL(dl->menu), GTK_WIDGET(menuitem));
 }
 
+static gboolean pure_ascii(const gchar *string) {
+	gunichar uchar;
+	const gchar *tmp = string;
+	do {
+		uchar = g_utf8_get_char(tmp);
+		if (uchar > 127)
+			return FALSE;
+		tmp = g_utf8_next_char(tmp);
+	} while (*tmp!='\0');
+	return TRUE;
+}
+
 void bftextview2_populate_suggestions_popup(GtkMenu *menu, Tdocument *doc) {
 	GtkTextIter wordstart,wordend;
 	Tdictlist dl;
@@ -563,16 +738,50 @@ void bftextview2_populate_suggestions_popup(GtkMenu *menu, Tdocument *doc) {
 	dl.menu = submenu;
 	dl.group=NULL;
 	enchant_broker_list_dicts(eb, list_dicts_lcb, &dl);
-
+	
 	if (!BFWIN(doc->bfwin)->ed)
 		return;
 	
 	if (get_misspelled_word_at_bevent(BLUEFISH_TEXT_VIEW(doc->view), &wordstart, &wordend)) {
 		gchar *word, **suggestions;
 		size_t n_suggs;
-		word = gtk_text_buffer_get_text(GTK_TEXT_VIEW(doc->view)->buffer, &wordstart,&wordend,FALSE);
+		gboolean have_non_ascii = FALSE;
+
+		word = gtk_text_buffer_get_text(doc->buffer, &wordstart,&wordend,FALSE);
+		
+		if (g_utf8_strchr(word, -1, '&') && g_utf8_strchr(word, -1, ';')) {
+			gchar *tmp = xmlentities2utf8(word);
+			g_free(word);
+			word=tmp;
+		}
+		
 		DBG_SPELL("list alternatives for %s\n", word);
 		suggestions = enchant_dict_suggest((EnchantDict *)BFWIN(doc->bfwin)->ed, word,strlen(word), &n_suggs);
+
+		menuitem = gtk_image_menu_item_new_with_label(_("Add to dictionary"));
+		gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), GTK_WIDGET(menuitem));
+		g_signal_connect(menuitem, "activate", G_CALLBACK(bftexview2_add_word_to_dict), doc);
+		menuitem = gtk_image_menu_item_new_with_label(_("Ignore spelling"));
+		gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), GTK_WIDGET(menuitem));
+		g_signal_connect(menuitem, "activate", G_CALLBACK(bftexview2_add_word_to_ses), doc);
+
+		if (suggestions) {
+			gint i;
+			for (i=0; i<n_suggs; i++) {
+				if (!pure_ascii(suggestions[i])) {
+					have_non_ascii = TRUE;
+					break;
+				}
+			}
+		}
+		
+		if (have_non_ascii) {
+			menuitem = gtk_check_menu_item_new_with_label(_("Insert special characters as entities"));
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), BFWIN(doc->bfwin)->session->spell_insert_entities);
+			gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), GTK_WIDGET(menuitem));
+			g_signal_connect(menuitem, "activate", G_CALLBACK(bftexview2_spell_insert_entities), doc);
+		}
+
 		if (suggestions) {
 			GtkWidget *menuitem;
 			gint i;
@@ -590,10 +799,17 @@ void bftextview2_populate_suggestions_popup(GtkMenu *menu, Tdocument *doc) {
 	}
 }
 
+void reload_spell_dictionary(Tbfwin *bfwin) {
+	if (load_dictionary(bfwin)) {
+		mark_all_docs_needspelling(bfwin);
+	}
+}
+
+
 /*
 static void bftextview2_preferences_menu_enable_lcb(GtkWidget *widget, gpointer data) {
 	Tbfwin *bfwin=data;
-	bfwin->session->spell_enable = GTK_CHECK_MENU_ITEM(widget)->active;
+	bfwin->session->spell_enable = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bfwin->toolbar_spell),bfwin->session->spell_enable);
 }
 
@@ -610,7 +826,7 @@ void bftextview2_populate_preferences_popup(GtkMenu *menu, Tdocument *doc) {
 
 /*void bftextview2_gui_toggle_spell_check(GtkWidget *widget, gpointer data) {
 	Tbfwin *bfwin=data;
-	bfwin->session->spell_enable = GTK_TOGGLE_BUTTON(widget)->active;
+	bfwin->session->spell_enable = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 	if (bfwin->current_document && bfwin->current_document->view) {
 		/ * the signal is also emitted when the toggle button gets it's initial value during the building of the window * /
 		bluefish_text_view_rescan(BLUEFISH_TEXT_VIEW(bfwin->current_document->view));
