@@ -6,7 +6,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,11 +15,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 /* indented with indent -ts4 -kr -l110   */
 
+/*#define IDENTSTORING*/
+
 /* #define HL_PROFILING */
 /* if you define DEBUG here you will get debug output from all Bluefish parts */
 /* #define DEBUG */
@@ -27,6 +28,9 @@
 #ifndef __BLUEFISH_H_
 #define __BLUEFISH_H_
 
+/*#define MEMORY_LEAK_DEBUG*/
+/*#define DEBUG_PATHS*/
+
 #define ENABLEPLUGINS
 
 #include "config.h"
@@ -57,14 +61,14 @@
 #define DEBUG_MSG_E g_error
 #define DEBUG_MSG_W g_warning
 #else							/* not DEBUG */
-#if defined(__GNUC__) || defined(__SUNPRO_C)
+#if defined(__GNUC__) || defined(__SUNPRO_C) && (__SUNPRO_C > 0x580)
 #define DEBUG_MSG(args...)
 #define DEBUG_MSG_C(args...)
 #define DEBUG_MSG_E(args...)
 #define DEBUG_MSG_W(args...)
  /**/
 #else							/* notdef __GNUC__ || __SUNPRO_C */
-extern void g_none(gchar * first, ...);
+extern void g_none(char * first, ...);
 #define DEBUG_MSG g_none
 #define DEBUG_MSG_C g_none
 #define DEBUG_MSG_E g_none
@@ -72,11 +76,26 @@ extern void g_none(gchar * first, ...);
 #endif							/* __GNUC__ || __SUNPRO_C */
 #endif							/* DEBUG */
 
+#ifdef DEBUG_PATHS
+#define DEBUG_PATH g_print
+#else
+#if defined(__GNUC__) || defined(__SUNPRO_C)
+#define DEBUG_PATH(args...)
+ /**/
+#else							/* notdef __GNUC__ || __SUNPRO_C */
+extern void g_none(gchar * first, ...);
+#endif							/* __GNUC__ || __SUNPRO_C */
+#endif							/* DEBUG */
+
 #ifdef ENABLE_NLS
 #include <glib/gi18n.h>
 #else							/* ENABLE_NLS */
 #define _(String)(String)
 #define N_(String)(String)
+#define ngettext(Msgid1, Msgid2, N) \
+	((N) == 1 \
+	? ((void) (Msgid2), (const char *) (Msgid1)) \
+	: ((void) (Msgid1), (const char *) (Msgid2)))
 #endif							/* ENABLE_NLS */
 
 
@@ -93,8 +112,6 @@ extern void g_none(gchar * first, ...);
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <pcre.h>
-
 #include <gio/gio.h>
 
 #define BF_FILEINFO "standard::name,standard::display-name,standard::size,standard::type,unix::mode,unix::uid,unix::gid,time::modified,time::modified-usec,etag::value,standard::fast-content-type"
@@ -178,10 +195,11 @@ typedef struct {
 	gint is_symlink;			/* file is a symbolic link */
 	gulong del_txt_id;			/* text delete signal */
 	gulong ins_txt_id;			/* text insert signal */
-	/*gulong ins_aft_txt_id; *//* text insert-after signal, for auto-indenting */
 	guint newdoc_autodetect_lang_id;	/* a timer function that runs for new documents to detect their mime type  */
 	unre_t unre;
 	GtkWidget *view;
+	GtkWidget *slave; /* used in split view for the bottom view */
+	GtkWidget *vsplit; /* used for split view */
 	GtkWidget *tab_label;
 	GtkWidget *tab_eventbox;
 	GtkWidget *tab_menu;
@@ -191,9 +209,7 @@ typedef struct {
 	gint need_highlighting;		/* if you open 10+ documents you don't need immediate highlighting, just set this var, and notebook_switch() will trigger the actual highlighting when needed */
 	gboolean highlightstate;	/* does this document use highlighting ? */
 	gboolean wrapstate;			/* does this document use wrap? */
-	gboolean symstate;			/* does this document show symbols? */
-	gboolean overwrite_mode;	/* is document in overwrite mode */
-	gboolean autoclosingtag;	/* does the document use autoclosing of tags */
+	gboolean overwrite_mode;	/* is keyboard in overwrite mode */
 	gpointer floatingview;		/* a 2nd textview widget that has its own window */
 	gpointer bfwin;
 	GtkTreeIter *bmark_parent;	/* if NULL this document doesn't have bookmarks, if
@@ -204,18 +220,19 @@ typedef struct {
 	gint do_periodic_check;
 	gchar *editor_font_string;	/* editor font */
 	gint editor_smart_cursor;
-	gint editor_indent_wspaces;	/* indent with spaces, not tabs */
+	gint editor_tab_indent_sel; /* tab key will indent a selected block */
+	gint use_system_tab_font;
 	gchar *tab_font_string;		/* notebook tabs font */
 	/*  gchar *tab_color_normal; *//* notebook tabs text color normal.  This is just NULL! */
 	gchar *tab_color_modified;	/* tab text color when doc is modified and unsaved */
 	gchar *tab_color_loading;	/* tab text color when doc is loading */
 	gchar *tab_color_error;		/* tab text color when doc has errors */
 	gint visible_ws_mode;
+	gint right_margin_pos;
 	/* new replacements: */
 	GList *external_command;	/* array: name,command,is_default_browser */
 	GList *external_filter;		/* array: name,command */
 	GList *external_outputbox;	/* array:name,command,....... */
-
 	/*gint defaulthighlight; *//* highlight documents by default */
 	gint transient_htdialogs;	/* set html dialogs transient ro the main window */
 	gint leave_to_window_manager;	/* don't set any dimensions, leave all to window manager */
@@ -263,13 +280,14 @@ typedef struct {
 	/* key conversion */
 	gint open_in_running_bluefish;	/* open commandline documents in already running process */
 	gint open_in_new_window;	/* open commandline files in a new window as opposed to an existing window */
-	gint show_splash_screen;
+	gint register_recent_mode; /* 0=none,1=all,2=project only*/
 	GList *plugin_config;		/* array, 0=filename, 1=enabled, 2=name */
 	gchar *btv_color_str[BTV_COLOR_COUNT];	/* editor colors */
 	GList *textstyles;			/* tet styles: name,foreground,background,weight,style */
 	gint block_folding_mode;
 	GList *highlight_styles;
 	GList *bflang_options;		/* array with: lang_name, option_name, value */
+	gchar *autocomp_accel_string;
 	gboolean load_reference;
 	gboolean show_autocomp_reference;
 	gboolean show_tooltip_reference;
@@ -283,6 +301,7 @@ typedef struct {
 	gchar *autosave_file_prefix;
 	gchar *autosave_file_suffix;
 	gchar *language;
+	GList *templates; /* array of name:URI */
 } Tproperties;
 
 /* the Tglobalsession contains all settings that can change 
@@ -306,18 +325,21 @@ typedef struct {
 	GList *reference_files;		/* all reference files */
 	GList *recent_projects;
 	GList *encodings;			/* all encodings you can choose from */
+#ifdef WITH_MSG_QUEUE
 	gint msg_queue_poll_time;	/* milliseconds, automatically tuned to your system */
+#endif
 } Tglobalsession;
 
 typedef struct {
 	gint wrap_text_default;		/* by default wrap text */
 	gint autoindent;			/* autoindent code */
 	gint editor_tab_width;		/* editor tabwidth */
+	gint editor_indent_wspaces;	/* indent with spaces, not tabs */
 	gint view_line_numbers;		/* view line numbers on the left side by default */
 	gint view_cline;			/* highlight current line by default */
 	gint view_blocks;			/* show blocks on the left side by default */
 	gint autocomplete;			/* whether or not to enable autocomplete by default for each new document */
-	gint view_mbhl;				/* show matching block begin-end by default */
+	gint show_mbhl;				/* show matching block begin-end by default */
 	gint snr_is_expanded;
 	gint sync_delete_deprecated;
 	gint sync_include_hidden;
@@ -339,16 +361,19 @@ typedef struct {
 	gint outputb_scroll_mode;	/* 0=none, 1=first line, 2= last line */
 	gint outputb_show_all_output;
 	gint convertcolumn_horizontally;
-	/* 29 * sizeof(gint) */
+	gint display_right_margin;
+	/* 30 * sizeof(gint) */
 	/* IF YOU EDIT THIS STRUCTURE PLEASE EDIT THE CODE IN PROJECT.C THAT COPIES
 	   A Tsessionvar INTO A NEW Tsessionvar AND ADJUST THE SIZES!!!!!!!!!!!!!!!!!!!!!! */
 #ifdef HAVE_LIBENCHANT
 	gint spell_check_default;
+	gint spell_insert_entities;
 	gchar *spell_lang;
 #endif
 	/* if you add strings or lists to the session, please make sure they are free'ed 
 	in free_session() in project.c */
 	gchar *default_mime_type;
+	gchar *template;
 	gchar *convertcolumn_separator;
 	gchar *convertcolumn_fillempty;
 	gchar *webroot;
@@ -376,7 +401,6 @@ typedef struct {
 	GFile *uri;
 	gchar *name;
 	GList *files;
-	gchar *template;
 	gpointer editor;
 	Tsessionvars *session;
 	gpointer bmarkdata;			/* project bookmarks */
@@ -387,10 +411,10 @@ typedef struct {
 typedef struct {
 	Tsessionvars *session;		/* points to the global session, or to the project session */
 	Tdocument *current_document;	/* one object out of the documentlist, the current visible document */
+	Tdocument *prev_document;
 	gboolean focus_next_new_doc;	/* for documents loading in the background, switch to the first that is finished loading */
 	gint num_docs_not_completed;	/* number of documents that are loading or closing */
 	GList *documentlist;		/* document.c and others: all Tdocument objects */
-	GTimer *idletimer;
 	Tdocument *last_activated_doc;
 	Tproject *project;			/* might be NULL for a default project */
 	GtkWidget *main_window;
@@ -429,10 +453,9 @@ typedef struct {
 	GList *menu_external;
 	GList *menu_encodings;
 	GList *menu_outputbox;
-	GList *menu_windows;
-	GtkWidget *menu_cmenu;
 	GList *menu_cmenu_entries;
 	GList *menu_filetypes;
+	GList *menu_templates;
 #ifdef HAVE_LIBENCHANT
 	gpointer *ed;				/* EnchantDict */
 #endif
@@ -440,17 +463,17 @@ typedef struct {
 	   in the file where it is needed */
 	gpointer outputbox;
 	gpointer bfspell;
-/*	gpointer filebrowser;*/
 	gpointer fb2;				/* filebrowser2 gui */
 	gpointer snr2;
 	GtkTreeView *bmark;
 	GtkTreeModelFilter *bmarkfilter;
 	gchar *bmark_search_prefix;
 	gpointer bmarkdata;			/* a link to the global main_v->bmarkdata, OR project->bmarkdata */
-/*	GtkTreeStore *bookmarkstore; / * this is a link to project->bookmarkstore OR main_v->bookmarkstore
-											  and it is only here for convenience !!!! * /
-	GHashTable *bmark_files;     / * no way, I have to have list of file iters. Other way I 
-	                                cannot properly load bmarks for closed files */
+#ifdef IDENTSTORING
+	GHashTable *identifier_jump;
+	GHashTable *identifier_ac;
+#endif /* IDENTSTORING */
+
 } Tbfwin;
 
 typedef struct {
@@ -469,11 +492,14 @@ typedef struct {
 	gpointer fb2config;			/* filebrowser2config */
 	GList *filefilters;			/* initialized by fb2config functions */
 	Tdocument *bevent_doc;
-	gint bevent_charoffset;
+	gint bevent_charoffset; 	/* for example used in the spellcheck code to find on which 
+											word the user clicked */
+	guint autocomp_accel_key;				 /* by default <ctrl><space> activates autocompletion */
+	GdkModifierType autocomp_accel_mods; /* but this shortcut is also used to switch input languages for example by chinese users */
 	gpointer bmarkdata;
 	gint num_untitled_documents;
-	GtkTooltips *tooltips;
 	gchar *securedir;			/* temporary rwx------ directory for secure file creation */
+	GtkRecentManager *recentm;
 	GSList *plugins;
 	GSList *doc_view_populate_popup_cbs;	/* plugins can register functions here that need to
 											   be called when the right-click menu in the document is populated */
@@ -490,4 +516,27 @@ extern EXPORT Tmain *main_v;
 /* public functions from bluefish.c */
 void bluefish_exit_request(void);
 
+/* backwards compatibility */
+#if !GTK_CHECK_VERSION(2,18,0)
+#define gtk_widget_set_can_focus(arg, arg2) do {if (arg2) GTK_WIDGET_SET_FLAGS(arg, GTK_CAN_FOCUS); else GTK_WIDGET_UNSET_FLAGS(arg, GTK_CAN_FOCUS);} while(0)
+#define gtk_widget_get_allocation(arg1, arg2) (*(arg2) = arg1->allocation)
+#define gtk_dialog_get_content_area(arg) (GTK_DIALOG(arg)->vbox)
+#define gtk_widget_get_window(arg) (GTK_WIDGET(arg)->window)
+#define gtk_widget_get_visible(arg) (GTK_WIDGET_VISIBLE(arg))
+#define gtk_widget_set_can_default(arg, arg2) do {if (arg2) GTK_WIDGET_SET_FLAGS(arg, GTK_CAN_DEFAULT); else GTK_WIDGET_UNSET_FLAGS(arg, GTK_CAN_DEFAULT);} while(0) 
+#define gtk_dialog_get_action_area(arg) (GTK_DIALOG(arg)->action_area)
+#define gtk_selection_data_get_data(arg) (arg->data)
+#define gtk_selection_data_get_length(arg) (arg->length)
+#define gtk_selection_data_get_format(arg) (arg->format)
+#define gtk_selection_data_get_target(arg) (arg->target)
+#define gtk_widget_get_state(arg) GTK_WIDGET_STATE(arg)
+#define gtk_widget_is_sensitive(arg) GTK_WIDGET_IS_SENSITIVE(arg)
+#endif /* GTK_CHECK_VERSION(2,18,0) */
+
+#if !GTK_CHECK_VERSION(2,14,0)
+#define gtk_adjustment_get_upper(arg) (GTK_ADJUSTMENT(arg)->upper)
+#define gtk_adjustment_get_lower(arg) (GTK_ADJUSTMENT(arg)->lower)
+#define gtk_adjustment_get_page_size(arg) (GTK_ADJUSTMENT(arg)->page_size)
+#endif /* GTK_CHECK_VERSION(2,18,0) */
+
 #endif							/* __BLUEFISH_H_ */
