@@ -2,11 +2,11 @@
  * bookmark.c - bookmarks
  *
  * Copyright (C) 2003 Oskar Swida
- * modifications (C) 2004-2009 Olivier Sessink
+ * modifications (C) 2004-2010 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,10 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-#define BMARKSEARCH
 /*#define DEBUG*/
 
 #include <gtk/gtk.h>
@@ -55,14 +53,14 @@ Tbmark. When the user clicks in the Gtktreeview widget, we can get
 immediately a pointer to the Tbmark, and that has the Gtktextmark, so that
 is very easy, and very fast!
 
-But now we have one problem: all normal windows do share the same bookmarks list.
-So it is probably the most logical to have them store the same Gtktreestore as
-well. The best way is to have the project functions create/destroy the
-gtktreestore when they convert a window (Tbfwin) into a project window.
+All normal (non-project) windows do share the same bookmarks list.
+So they store the same Gtktreestore as well. The project functions 
+create a new gtktreestore when they convert a window (Tbfwin)
+into a project window.
 */
 
-#define BMARK_SHOW_NUM_TEXT_CHARS 20
-#define BMARK_STORE_TEXT_NUM_CHARS 15
+#define BMARK_SHOW_NUM_TEXT_CHARS 40
+#define BMARK_STORE_TEXT_NUM_CHARS 25
 enum {
 	NAME_COLUMN,				/* bookmark name */
 	PTR_COLUMN,					/* bookmark pointer */
@@ -416,6 +414,12 @@ static void bmark_unstore(Tbfwin * bfwin, Tbmark * b)
 	b->strarr = NULL;
 }
 
+static Tbmark *get_bmark_at_iter(GtkTreeModel *model, GtkTreeIter *iter) {
+	Tbmark *retval = NULL;
+	gtk_tree_model_get(model,iter,PTR_COLUMN, &retval, -1);
+	return retval;
+}
+
 /* get value from pointer column */
 static Tbmark *get_current_bmark(Tbfwin * bfwin)
 {
@@ -429,7 +433,7 @@ static Tbmark *get_current_bmark(Tbfwin * bfwin)
 				GtkTreeIter iter;
 				GtkTreeModel *model = gtk_tree_view_get_model(bfwin->bmark);
 				gtk_tree_model_get_iter(model, &iter, path);
-				gtk_tree_model_get(model,&iter,PTR_COLUMN, &retval, -1);
+				retval = get_bmark_at_iter(model, &iter);
 			} else {
 				DEBUG_MSG("get_current_bmark, error, depth=%d\n",gtk_tree_path_get_depth(path));
 			}
@@ -452,13 +456,14 @@ void bmark_add_rename_dialog(Tbfwin * bfwin, gchar * dialogtitle)
 		gtk_dialog_new_with_buttons(dialogtitle, GTK_WINDOW(bfwin->main_window), GTK_DIALOG_MODAL,
 									GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
 	button = gtk_button_new_from_stock(GTK_STOCK_OK);
-	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default(button, TRUE);
 	gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_OK);
 
 	table = gtk_table_new(2, 3, FALSE);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 12);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 6);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), table, FALSE, FALSE, 12);
+	gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dlg))),
+			table, FALSE, FALSE, 12);
 
 	name = entry_with_text(m->name, 200);
 	gtk_entry_set_activates_default(GTK_ENTRY(name), TRUE);
@@ -483,7 +488,7 @@ void bmark_add_rename_dialog(Tbfwin * bfwin, gchar * dialogtitle)
 		m->name = g_strdup(gtk_entry_get_text(GTK_ENTRY(name)));
 		g_free(m->description);
 		m->description = g_strdup(gtk_entry_get_text(GTK_ENTRY(desc)));
-		m->is_temp = GTK_TOGGLE_BUTTON(istemp)->active;
+		m->is_temp = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(istemp));
 		tmpstr = bmark_showname(bfwin, m);
 		gtk_tree_store_set(BMARKDATA(bfwin->bmarkdata)->bookmarkstore, &m->iter, NAME_COLUMN,
 						   tmpstr,-1);
@@ -500,28 +505,42 @@ void bmark_add_rename_dialog(Tbfwin * bfwin, gchar * dialogtitle)
 	gtk_widget_destroy(dlg);
 }
 
-static void bmark_popup_menu_goto(Tbfwin *bfwin) {
-	Tbmark *b;
+static void bmark_activate(Tbfwin *bfwin, Tbmark *b, gboolean select_bmark) {
 	GtkTextIter it;
 
-	b = get_current_bmark(bfwin);
-	if (b) {
-		if ( b->doc && b->mark ) {
-			/* recalculate offset */
-			gtk_text_buffer_get_iter_at_mark(b->doc->buffer,&it,b->mark);
-			b->offset = gtk_text_iter_get_offset(&it);
-		}
-		DEBUG_MSG("bmark_popup_menu_goto, bmark at %p, filepath at %p\n",b, b->filepath);
-		DEBUG_MSG("bmark_popup_menu_goto, calling doc_new_from_uri with goto_offset %d\n",b->offset);
-		doc_new_from_uri(bfwin, b->filepath, NULL, FALSE, FALSE, -1, b->offset);
-		/* remove selection */
-		if ( b->doc ) {
-			gtk_text_buffer_get_iter_at_mark(b->doc->buffer,&it,gtk_text_buffer_get_insert(b->doc->buffer));
-			gtk_text_buffer_move_mark_by_name(b->doc->buffer, "selection_bound", &it);
-		}
+	if (!b)
+		return;
+
+	if ( b->doc && b->mark ) {
+		/* recalculate offset */
+		gtk_text_buffer_get_iter_at_mark(b->doc->buffer,&it,b->mark);
+		b->offset = gtk_text_iter_get_offset(&it);
+	}
+	DEBUG_MSG("bmark_activate, bmark at %p, filepath at %p\n",b, b->filepath);
+	DEBUG_MSG("bmark_activate, calling doc_new_from_uri with goto_offset %d\n",b->offset);
+	doc_new_from_uri(bfwin, b->filepath, NULL, FALSE, FALSE, -1, b->offset);
+	/* remove selection */
+	if ( b->doc ) {
+		gtk_text_buffer_get_iter_at_mark(b->doc->buffer,&it,gtk_text_buffer_get_insert(b->doc->buffer));
+		gtk_text_buffer_move_mark_by_name(b->doc->buffer, "selection_bound", &it);
+		gtk_widget_grab_focus(b->doc->view);
+	} else if (bfwin->current_document) {
 		gtk_widget_grab_focus(bfwin->current_document->view);
 	}
+	if (select_bmark) {
+		GtkTreeIter fiter;
+		gtk_tree_model_filter_convert_child_iter_to_iter(bfwin->bmarkfilter,&fiter,&b->iter);
+		gtk_tree_selection_select_iter(gtk_tree_view_get_selection(bfwin->bmark), &fiter);
+	}
+	
 }
+
+/*static void bmark_goto_selected(Tbfwin *bfwin) {
+	Tbmark *b = get_current_bmark(bfwin);
+	if (b) {
+		bmark_activate(bfwin, b, FALSE);
+	}
+}*/
 /*
  * removes the bookmark from the treestore, and if it is the last remaining bookmark
  * for the document, it will remove the parent iter (the filename) from the treestore as well
@@ -611,15 +630,12 @@ static void bmark_popup_menu_deldoc(Tbfwin *bfwin) {
 			g_free(pstr);
 			if (retval == 0)
 				return;
-#ifdef BMARKSEARCH
 			gtk_tree_model_filter_convert_iter_to_child_iter(GTK_TREE_MODEL_FILTER(model),&realiter,&iter);
 			bmark_del_children_backend(bfwin, &realiter);
-#else
-			bmark_del_children_backend(bfwin, &iter);
-#endif
 		}
 	}
-	gtk_widget_grab_focus(bfwin->current_document->view);
+	if (bfwin->current_document)
+		gtk_widget_grab_focus(bfwin->current_document->view);
 }
 
 static void bmark_popup_menu_del(Tbfwin *bfwin) {
@@ -649,15 +665,16 @@ static void bmark_popup_menu_del(Tbfwin *bfwin) {
 		bmark_unstore(bfwin, b);
 		bmark_free(b);
 	}
-	gtk_widget_grab_focus(bfwin->current_document->view);
+	if (bfwin->current_document)
+		gtk_widget_grab_focus(bfwin->current_document->view);
 }
 
 static void bmark_rpopup_action_lcb(gpointer data, guint action, GtkWidget *widget) {
 	Tbfwin *bfwin = BFWIN(data);
 	switch (action) {
-		case 1:
-			bmark_popup_menu_goto(bfwin);
-		break;
+		/*case 1:
+			bmark_goto_selected(bfwin);
+		break;*/
 		case 2: {
 			Tbmark *m = get_current_bmark(bfwin);
 			if (!m) return;
@@ -718,7 +735,7 @@ static void bmark_rpopup_action_lcb(gpointer data, guint action, GtkWidget *widg
 }
 
 static GtkItemFactoryEntry bmark_rpopup_menu_entries[] = {
-	{ N_("/Goto"),		NULL,	bmark_rpopup_action_lcb,		1,	"<Item>" },
+	/*{ N_("/Goto"),		NULL,	bmark_rpopup_action_lcb,		1,	"<Item>" },*/
 	{ N_("/Edit"),NULL,	bmark_rpopup_action_lcb,	2,	"<Item>" },
 	{ N_("/Delete"),NULL,	bmark_rpopup_action_lcb,	3,	"<Item>" },
 	{ "/sep1",						NULL,	NULL,									0,	"<Separator>" },
@@ -749,7 +766,7 @@ static GtkWidget *bmark_popup_menu(Tbfwin * bfwin, gboolean show_bmark_specific,
 	menu = gtk_item_factory_get_widget(menumaker, "<Bookmarks>");
 
 	if (!show_bmark_specific) {
-		gtk_widget_set_sensitive(gtk_item_factory_get_widget(menumaker, "/Goto"), FALSE);
+		/*gtk_widget_set_sensitive(gtk_item_factory_get_widget(menumaker, "/Goto"), FALSE);*/
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(menumaker, "/Edit"), FALSE);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(menumaker, "/Delete"), FALSE);
 	}
@@ -768,24 +785,35 @@ static GtkWidget *bmark_popup_menu(Tbfwin * bfwin, gboolean show_bmark_specific,
 	return menu;
 }
 
-/* right mouse click */
+static void bmark_row_activated(GtkTreeView * tree, GtkTreePath * path,
+									GtkTreeViewColumn * column, Tbfwin *bfwin) {
+	GtkTreeIter iter;
+	gtk_tree_model_get_iter(GTK_TREE_MODEL(bfwin->bmarkfilter), &iter, path);
+	if (gtk_tree_path_get_depth(path)==2) {
+		bmark_activate(bfwin, get_bmark_at_iter(GTK_TREE_MODEL(bfwin->bmarkfilter), &iter), FALSE);
+	}
+}
+
+/* mouse click */
 static gboolean bmark_event_mouseclick(GtkWidget * widget, GdkEventButton * event, Tbfwin * bfwin) {
 	GtkTreePath *path;
 	gboolean show_bmark_specific = FALSE, show_file_specific = FALSE;
 	if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(bfwin->bmark), event->x, event->y, &path, NULL, NULL, NULL)) {
 		if (path) {
 			gint depth = gtk_tree_path_get_depth(path);
-			gtk_tree_path_free(path);
+			
 			if (depth==2) {
 				show_bmark_specific = TRUE;
 				show_file_specific = TRUE;
-/*				if (event->button == 1 && event->type == GDK_2BUTTON_PRESS) {
-					bmark_popup_menu_goto(bfwin);
-					return TRUE;
-				}*/
+				if (event->button == 1) {
+					GtkTreeIter iter;
+					gtk_tree_model_get_iter(GTK_TREE_MODEL(bfwin->bmarkfilter), &iter, path);
+					bmark_activate(bfwin, get_bmark_at_iter(GTK_TREE_MODEL(bfwin->bmarkfilter), &iter), FALSE);
+				}
 			} else if (depth == 1) {
 				show_file_specific = TRUE;
 			}
+			gtk_tree_path_free(path);
 		}
 	}
 	if (event->button == 3 && event->type == GDK_BUTTON_PRESS) {	/* right mouse click */
@@ -795,58 +823,110 @@ static gboolean bmark_event_mouseclick(GtkWidget * widget, GdkEventButton * even
 	return FALSE;
 }
 
-static void bmark_selection_changed_lcb(GtkTreeSelection *treeselection,Tbfwin * bfwin) {
+/*static void bmark_selection_changed_lcb(GtkTreeSelection *treeselection,Tbfwin * bfwin) {
+	/ * this is not the best way to activate bookmarks. according to the gtk documentation:
+	Emitted whenever the selection has (possibly) changed. Please note that this signal is 
+	mostly a hint. It may only be emitted once when a range of rows are selected, and it 
+	may occasionally be emitted when nothing has happened.
+	
+	THUS: we should better use the mouse click event to find the correct bookmark to 
+	activate.
+	* /
 	DEBUG_MSG("bmark_selection_changed_lcb, started\n");
-	bmark_popup_menu_goto(bfwin);
-}
+	bmark_goto_selected(bfwin);
+}*/
 
-void bookmark_menu_cb(Tbfwin *bfwin,guint action,GtkWidget *widget) {
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(bfwin->bmark));
-	if (gtk_tree_selection_get_selected(selection,&model,&iter)) {
-		gboolean success=TRUE;
-		switch (action) {
-			case 1:/* first */ {
-				GtkTreePath *path= gtk_tree_model_get_path(model,&iter);
-				gtk_tree_path_up(path);
-				gtk_tree_path_down(path);
-				success = gtk_tree_model_get_iter(model,&iter,path);
-				gtk_tree_path_free(path);
-			} break;
-			case 2: 	/* previous */ {
-				GtkTreePath *path= gtk_tree_model_get_path(model,&iter);
-				gtk_tree_path_prev(path);
-				success = gtk_tree_model_get_iter(model,&iter,path);
-				gtk_tree_path_free(path);
-			} break;
-			case 3:/* next */
-				success = gtk_tree_model_iter_next(model,&iter);
-			break;
-			case 4: /* last */ {
-				GtkTreeIter parent;
-				gint num;
-				success = gtk_tree_model_iter_parent(model,&parent,&iter);
-				num = gtk_tree_model_iter_n_children(model,&parent);
-				gtk_tree_model_iter_nth_child(model,&iter,&parent,num-1);
-			} break;
-		}
-		if (success) gtk_tree_selection_select_iter(selection,&iter);
-	}
-}
 static void bmark_first_lcb(GtkWidget *widget, Tbfwin *bfwin) {
-	bookmark_menu_cb(bfwin,1,widget);
-}
-static void bmark_previous_lcb(GtkWidget *widget, Tbfwin *bfwin) {
-	bookmark_menu_cb(bfwin,2,widget);
-}
-static void bmark_next_lcb(GtkWidget *widget, Tbfwin *bfwin) {
-	bookmark_menu_cb(bfwin,3,widget);
+	GtkTreeModel *model = GTK_TREE_MODEL(BMARKDATA(bfwin->bmarkdata)->bookmarkstore);
+	GtkTreeIter iter;
+	
+	if (!CURDOC(bfwin) || !CURDOC(bfwin)->bmark_parent)
+		return;
+	DEBUG_MSG("bmark_first_lcb, started\n");
+	if (gtk_tree_model_iter_children(model,&iter,CURDOC(bfwin)->bmark_parent))
+		bmark_activate(bfwin, get_bmark_at_iter(model, &iter), TRUE);
 }
 static void bmark_last_lcb(GtkWidget *widget, Tbfwin *bfwin) {
-	bookmark_menu_cb(bfwin,4,widget);
+	GtkTreeModel *model = GTK_TREE_MODEL(BMARKDATA(bfwin->bmarkdata)->bookmarkstore);
+	GtkTreeIter iter;
+	gint num;
+
+	if (!CURDOC(bfwin) || !CURDOC(bfwin)->bmark_parent)
+		return;
+	DEBUG_MSG("bmark_last_lcb, started\n");
+	num = gtk_tree_model_iter_n_children(model,CURDOC(bfwin)->bmark_parent);
+	if (gtk_tree_model_iter_nth_child(model,&iter,CURDOC(bfwin)->bmark_parent,num-1))
+		bmark_activate(bfwin, get_bmark_at_iter(model, &iter), TRUE);
 }
-#ifdef BMARKSEARCH
+
+static void bmark_previous_lcb(GtkWidget *widget, Tbfwin *bfwin) {
+	GtkTextMark *insert;
+	GtkTextIter titer;
+	Tbmark *bmark=NULL;
+	gint line;
+	
+	if (!CURDOC(bfwin) || !CURDOC(bfwin)->bmark_parent)
+		return;
+	DEBUG_MSG("bmark_previous_lcb, started\n");
+	insert = gtk_text_buffer_get_insert(CURDOC(bfwin)->buffer);
+	gtk_text_buffer_get_iter_at_mark(CURDOC(bfwin)->buffer, &titer, insert);
+	/*gtk_text_iter_set_line_offset(&titer, 0);*/
+	gtk_text_iter_backward_line(&titer);
+	line = bmark_margin_get_initial_bookmark(CURDOC(bfwin), &titer, (gpointer)&bmark);
+	DEBUG_MSG("bmark_previous_lcb, got initial bookmark at line %d, cursor is at line %d\n",line,gtk_text_iter_get_line(&titer));
+	if (-1 == line)
+		return;
+	if (line > gtk_text_iter_get_line(&titer)) {
+		bmark_last_lcb(widget, bfwin);
+		return;
+	}
+	bmark_activate(bfwin, bmark, TRUE);
+}
+
+static void bmark_next_lcb(GtkWidget *widget, Tbfwin *bfwin) {
+	GtkTextMark *insert;
+	GtkTextIter titer;
+	Tbmark *bmark=NULL;
+	gint line;
+	
+	if (!CURDOC(bfwin) || !CURDOC(bfwin)->bmark_parent)
+		return;
+	DEBUG_MSG("bmark_next_lcb, started\n");
+	insert = gtk_text_buffer_get_insert(CURDOC(bfwin)->buffer);
+	gtk_text_buffer_get_iter_at_mark(CURDOC(bfwin)->buffer, &titer, insert);
+	gtk_text_iter_forward_to_line_end(&titer);
+	line = bmark_margin_get_initial_bookmark(CURDOC(bfwin), &titer, (gpointer)&bmark);
+	if (-1 == line)
+		return;
+	if (line <= gtk_text_iter_get_line(&titer)) {
+		/* get the 'next' bookmark */
+		if (-1 == bmark_margin_get_next_bookmark(CURDOC(bfwin), (gpointer)&bmark)) {
+			bmark_first_lcb(widget, bfwin);
+			return;
+		}
+	}
+	bmark_activate(bfwin, bmark, TRUE);
+}
+void bookmark_menu_cb(Tbfwin *bfwin,guint action,GtkWidget *widget) {
+	switch (action) {
+		case 1:
+		bmark_first_lcb(widget, bfwin);
+		break;
+		case 2:
+		bmark_previous_lcb(widget, bfwin);
+		break;
+		case 3:
+		bmark_next_lcb(widget, bfwin);		
+		break;
+		case 4:
+		bmark_last_lcb(widget, bfwin);
+		break;
+		default:
+		g_warning("invalid menu action for bookmark menu, please report as bluefish bug\n ");
+		break;
+	}
+}
+
 
 static gboolean bmark_search_filter_func(GtkTreeModel *model, GtkTreeIter  *iter, gpointer data) {
 	GtkTreeIter piter;
@@ -927,8 +1007,6 @@ static gboolean bmark_search_button_press(GtkWidget *widget, GdkEventButton *eve
 }
 #endif
 
-#endif /* BMARKSEARCH */
-
 /* Initialize bookmarks gui for window */
 GtkWidget *bmark_gui(Tbfwin * bfwin)
 {
@@ -941,7 +1019,6 @@ GtkWidget *bmark_gui(Tbfwin * bfwin)
 	   Tree View is in bfwin->bmark
 	 */
 	vbox = gtk_vbox_new(FALSE, 1);
-#ifdef BMARKSEARCH
 	entry = gtk_entry_new();
 #if GTK_CHECK_VERSION(2,16,0)
 	gtk_entry_set_icon_from_stock(GTK_ENTRY(entry), GTK_ENTRY_ICON_PRIMARY, GTK_STOCK_FIND);
@@ -952,38 +1029,33 @@ GtkWidget *bmark_gui(Tbfwin * bfwin)
 #endif
 	g_signal_connect(G_OBJECT(entry), "changed", G_CALLBACK(bmark_search_changed), bfwin);
 	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, TRUE, 0);
-#endif
 	hbox = gtk_toolbar_new();
 	gtk_toolbar_set_icon_size(GTK_TOOLBAR(hbox),GTK_ICON_SIZE_MENU);
 	gtk_toolbar_set_style(GTK_TOOLBAR(hbox),GTK_TOOLBAR_ICONS);
 
 	but = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_GOTO_TOP,GTK_ICON_SIZE_MENU),"");
 	g_signal_connect(G_OBJECT(but),"clicked",G_CALLBACK(bmark_first_lcb),bfwin);
-	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(but),main_v->tooltips,_("First bookmark"),"");
+	gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(but),_("First bookmark"));
 	gtk_toolbar_insert(GTK_TOOLBAR(hbox),but,-1);
 	but = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_GO_UP,GTK_ICON_SIZE_MENU),"");
 	g_signal_connect(G_OBJECT(but),"clicked",G_CALLBACK(bmark_previous_lcb),bfwin);
-	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(but),main_v->tooltips,_("Previous bookmark"),"");
+	gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(but),_("Previous bookmark"));
 	gtk_toolbar_insert(GTK_TOOLBAR(hbox),but,-1);
 	but = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_GO_DOWN,GTK_ICON_SIZE_MENU),"");
 	g_signal_connect(G_OBJECT(but),"clicked",G_CALLBACK(bmark_next_lcb),bfwin);
-	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(but),main_v->tooltips,_("Next bookmark"),"");
+	gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(but),_("Next bookmark"));
 	gtk_toolbar_insert(GTK_TOOLBAR(hbox),but,-1);
 	but = gtk_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_GOTO_BOTTOM,GTK_ICON_SIZE_MENU),"");
 	g_signal_connect(G_OBJECT(but),"clicked",G_CALLBACK(bmark_last_lcb),bfwin);
-	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(but),main_v->tooltips,_("Last bookmark"),"");
+	gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(but),_("Last bookmark"));
 	gtk_toolbar_insert(GTK_TOOLBAR(hbox),but,-1);
 
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
 
-#ifdef BMARKSEARCH
 	bfwin->bmarkfilter = (GtkTreeModelFilter *)gtk_tree_model_filter_new(GTK_TREE_MODEL(BMARKDATA(bfwin->bmarkdata)->bookmarkstore),NULL);
 	gtk_tree_model_filter_set_visible_func(bfwin->bmarkfilter, bmark_search_filter_func, bfwin, NULL);
 	bfwin->bmark = (GtkTreeView *)gtk_tree_view_new_with_model(GTK_TREE_MODEL(bfwin->bmarkfilter));
 	g_object_unref(bfwin->bmarkfilter);
-#else
-	bfwin->bmark = gtk_tree_view_new_with_model(GTK_TREE_MODEL(BMARKDATA(bfwin->bmarkdata)->bookmarkstore));
-#endif /* BMARKSEARCH */
 	cell = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes("", cell, "text", NAME_COLUMN, NULL);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
@@ -997,12 +1069,13 @@ GtkWidget *bmark_gui(Tbfwin * bfwin)
 	gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
 	g_signal_connect(G_OBJECT(bfwin->bmark), "button-press-event",
 					 G_CALLBACK(bmark_event_mouseclick), bfwin);
+	g_signal_connect(G_OBJECT(bfwin->bmark), "row-activated", G_CALLBACK(bmark_row_activated),bfwin);
 	gtk_tree_view_expand_all(bfwin->bmark);
-	{
+	/*{
 		GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(bfwin->bmark));
 		gtk_tree_selection_set_mode(selection,GTK_SELECTION_BROWSE);
 		g_signal_connect(G_OBJECT(selection), "changed",G_CALLBACK(bmark_selection_changed_lcb), bfwin);
-	}
+	}*/
 
 	return vbox;
 }
@@ -1135,7 +1208,7 @@ void bmark_reload(Tbfwin * bfwin) {
 	tmplist = g_list_first(bfwin->session->bmarks);
 	while (tmplist) {
 		gchar **items = (gchar **) tmplist->data;
-		if (items && count_array(items) == 6) {
+		if (items && g_strv_length(items) == 6) {
 			gchar *ptr;
 			Tbmark *b;
 			b = g_slice_new0(Tbmark);
@@ -1198,14 +1271,10 @@ void bmark_reload(Tbfwin * bfwin) {
 void bmark_set_store(Tbfwin * bfwin) {
 	DEBUG_MSG("bmark_set_store set store %p for bfwin %p\n",BMARKDATA(bfwin->bmarkdata)->bookmarkstore,bfwin);
 	if (BMARKDATA(bfwin->bmarkdata)->bookmarkstore && bfwin->bmark) {
-#ifdef BMARKSEARCH
 		bfwin->bmarkfilter = (GtkTreeModelFilter *)gtk_tree_model_filter_new(GTK_TREE_MODEL(BMARKDATA(bfwin->bmarkdata)->bookmarkstore), NULL);
 		gtk_tree_model_filter_set_visible_func(bfwin->bmarkfilter, bmark_search_filter_func, bfwin, NULL);
 		gtk_tree_view_set_model(bfwin->bmark, GTK_TREE_MODEL(bfwin->bmarkfilter));
 		g_object_unref(bfwin->bmarkfilter);
-#else
-		gtk_tree_view_set_model(bfwin->bmark, GTK_TREE_MODEL(BMARKDATA(bfwin->bmarkdata)->bookmarkstore));
-#endif
 	}
 }
 
@@ -1249,7 +1318,7 @@ void bmark_clean_for_doc(Tdocument * doc) {
 static gboolean bookmark_reposition(Tbmark *mark, gint offset) {
 	gint doclen = gtk_text_buffer_get_char_count(mark->doc->buffer);
 	gint bandwidth = offset>0?2*offset:-2*offset;
-	if (bandwidth < 10*strlen(mark->text)) bandwidth = 10*strlen(mark->text);
+	if (bandwidth < 5*strlen(mark->text)) bandwidth = 5*strlen(mark->text);
 	/* search for the bookmark near the old positions */
 
 	while (TRUE) {
@@ -1388,7 +1457,7 @@ gint bmark_margin_get_initial_bookmark(Tdocument * doc, GtkTextIter *fromit, gpo
  * will use offset if itoffset is NULL
  * will use itoffset if not NULL
  */
-static void bmark_add_backend(Tdocument *doc, GtkTextIter *itoffset, gint offset, const gchar *name, const gchar *text, gboolean is_temp) {
+static Tbmark *bmark_add_backend(Tdocument *doc, GtkTextIter *itoffset, gint offset, const gchar *name, const gchar *text, gboolean is_temp) {
 	Tbmark *m;
 	gchar *displaytext = NULL;
 	GtkTextIter it;
@@ -1423,6 +1492,9 @@ static void bmark_add_backend(Tdocument *doc, GtkTextIter *itoffset, gint offset
 		bmark_store(BFWIN(doc->bfwin), m);
 	}
 	gtk_widget_queue_draw(doc->view);
+	if (doc->slave)
+		gtk_widget_queue_draw(doc->slave);
+	return m;
 }
 
 /**
@@ -1456,6 +1528,9 @@ static gchar *bmark_text_for_offset(Tdocument *doc, GtkTextIter *itoffset, gint
 /* this function will add a bookmark to the current document at current cursor / selection */
 static void bmark_add_current_doc_backend(Tbfwin *bfwin, const gchar *name, gint offset, gboolean is_temp) {
 	GtkTextIter it, eit, sit;
+	Tbmark *m;
+	if (!bfwin->current_document)
+		return;
 	DEBUG_MSG("bmark_add_backend, adding bookmark at offset=%d for bfwin=%p\n",offset,bfwin);
 	/* create bookmark */
 	gtk_text_buffer_get_iter_at_offset(DOCUMENT(bfwin->current_document)->buffer,&it,offset);
@@ -1463,18 +1538,20 @@ static void bmark_add_current_doc_backend(Tbfwin *bfwin, const gchar *name, gint
 	if (gtk_text_buffer_get_selection_bounds(DOCUMENT(bfwin->current_document)->buffer,&sit,&eit)
 				&& gtk_text_iter_in_range(&it,&sit,&eit)) {
 		gchar *text = gtk_text_iter_get_text(&sit, &eit);
-		bmark_add_backend(DOCUMENT(bfwin->current_document), &sit, offset, name, text, is_temp);
+		m = bmark_add_backend(DOCUMENT(bfwin->current_document), &sit, offset, name, text, is_temp);
 		g_free(text);
 
 	} else {
 		gchar *text;
 		text = bmark_text_for_offset(DOCUMENT(bfwin->current_document), &it, offset);
-		bmark_add_backend(DOCUMENT(bfwin->current_document), &it, offset, name, text, is_temp);
+		m = bmark_add_backend(DOCUMENT(bfwin->current_document), &it, offset, name, text, is_temp);
 		g_free(text);
 	}
-	if (bfwin->bmark) {
-		/* only if there is a left panel we should do this */
-		gtk_tree_view_expand_all(bfwin->bmark);
+	if (bfwin->bmark) { /* only if there is a left panel we should do this */
+		GtkTreePath *path;
+		path = gtk_tree_model_get_path(GTK_TREE_MODEL(BMARKDATA(bfwin->bmarkdata)->bookmarkstore),&m->iter);
+		gtk_tree_view_expand_to_path(bfwin->bmark,path);
+		gtk_tree_path_free(path);
 		gtk_widget_grab_focus(bfwin->current_document->view);
 	}
 }
@@ -1529,6 +1606,14 @@ static Tbmark *bmark_get_bmark_at_line(Tdocument *doc, gint offset) {
 	return NULL;
 }
 
+static void bmark_warn_unsaved_file(Tbfwin *bfwin) {
+	message_dialog_new(bfwin->main_window,
+								 GTK_MESSAGE_ERROR,
+								 GTK_BUTTONS_CLOSE,
+								 _("Error adding bookmark"),
+								 _("Cannot add bookmarks to unsaved files."));
+}
+
 /**
  * bmark_add_extern
  * @doc: a #Tdocument* with the document
@@ -1541,6 +1626,12 @@ static Tbmark *bmark_get_bmark_at_line(Tdocument *doc, gint offset) {
  * this function.
  */
 void bmark_add_extern(Tdocument *doc, gint offset, const gchar *name, const gchar *text, gboolean is_temp) {
+	if (!doc)
+		return;
+	if (!doc->uri) {
+		bmark_warn_unsaved_file(doc->bfwin);
+		return;
+	}
 	DEBUG_MSG("adding bookmark at offset %d with name %s\n",offset,name); /* dummy */
 	if (!bmark_get_bmark_at_line(doc, offset)) {
 		if (text) {
@@ -1553,17 +1644,31 @@ void bmark_add_extern(Tdocument *doc, gint offset, const gchar *name, const gcha
 	}
 }
 
+void bmark_toggle(Tdocument *doc, gint offset, const gchar *name, const gchar *text) {
+	Tbmark *bmark;
+	if (!doc)
+		return;
+	if (!doc->uri) {
+		bmark_warn_unsaved_file(doc->bfwin);
+		return;
+	}
+	bmark = bmark_get_bmark_at_line(doc, offset);
+	if (bmark) {
+		bmark_check_remove(BFWIN(doc->bfwin),bmark); /* check  if we should remove a filename too */
+		bmark_unstore(BFWIN(doc->bfwin), bmark);
+		bmark_free(bmark);
+	} else {
+		bmark_add_extern(doc, offset, name, text, !main_v->globses.bookmarks_default_store);
+	}
+}
+
 void bmark_add(Tbfwin * bfwin) {
 	GtkTextIter it, it2;
 	gint offset;
 	gboolean has_mark;
 	/* check for unnamed document */
-	if (!DOCUMENT(bfwin->current_document)->uri) {
-		message_dialog_new(bfwin->main_window,
-								 GTK_MESSAGE_ERROR,
-								 GTK_BUTTONS_CLOSE,
-								 _("Error adding bookmark"),
-								 _("Cannot add bookmarks to unsaved files."));
+	if (!bfwin->current_document || !DOCUMENT(bfwin->current_document)->uri) {
+		bmark_warn_unsaved_file(bfwin);
 		/*Please save the file first. A Save button in this dialog would be cool -- Alastair*/
 		return;
 	}
@@ -1641,7 +1746,7 @@ void bmark_del_all(Tbfwin *bfwin,gboolean ask) {
 	const gchar *buttons[] = {GTK_STOCK_NO, GTK_STOCK_YES, NULL};
 	GtkTreeIter tmpiter;
 
-	if (bfwin==NULL) return;
+	if (bfwin==NULL || !bfwin->current_document) return;
 
 	if (ask)	{
 	  retval = message_dialog_new_multi(bfwin->main_window,
