@@ -1,11 +1,11 @@
 /* Bluefish HTML Editor
  * file_dialogs.c - file dialogs
  *
- * Copyright (C) 2005-2009 Olivier Sessink
+ * Copyright (C) 2005-2010 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* indented with indent -ts4 -kr -l110   */
@@ -38,6 +37,8 @@
 #include "stringlist.h"
 #include "undo_redo.h"
 
+static gchar *modified_on_disk_warning_string(const gchar *filename, GFileInfo *oldfinfo, GFileInfo *newfinfo);
+
 /**************************************************************************/
 /* the start of the callback functions for the menu, acting on a document */
 /**************************************************************************/
@@ -56,8 +57,7 @@ typedef struct {
 
 static void files_advanced_win_findpattern_changed(GtkComboBox * combobox, Tfiles_advanced * tfs)
 {
-	if (strlen(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(tfs->find_pattern)->child)))
-		> 0) {
+	if (strlen(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(tfs->find_pattern))))) > 0) {
 		gtk_dialog_set_response_sensitive(GTK_DIALOG(tfs->dialog), GTK_RESPONSE_ACCEPT, TRUE);
 	} else {
 		gtk_dialog_set_response_sensitive(GTK_DIALOG(tfs->dialog), GTK_RESPONSE_ACCEPT, FALSE);
@@ -70,7 +70,7 @@ static gboolean files_advanced_win_ok_clicked(Tfiles_advanced * tfs)
 	gchar *basedir, *content_filter, *extension_filter;
 	gboolean retval;
 	GError *gerror=NULL;
-	extension_filter = gtk_editable_get_chars(GTK_EDITABLE(GTK_BIN(tfs->find_pattern)->child), 0, -1);
+	extension_filter = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(tfs->find_pattern))), 0, -1);
 	basedir = gtk_editable_get_chars(GTK_EDITABLE(tfs->basedir), 0, -1);
 	baseuri = g_file_new_for_uri(basedir);
 	content_filter = gtk_combo_box_get_active_text(GTK_COMBO_BOX(tfs->grep_pattern));
@@ -79,7 +79,7 @@ static gboolean files_advanced_win_ok_clicked(Tfiles_advanced * tfs)
 
 	retval = open_advanced(tfs->bfwin, baseuri, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tfs->recursive))
 				  , 500
-				  , gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tfs->matchname))
+				  , !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tfs->matchname))
 				  ,
 				  strlen(extension_filter) == 0 ? NULL : extension_filter,
 				  strlen(content_filter) == 0 ? NULL : content_filter,
@@ -96,7 +96,7 @@ static gboolean files_advanced_win_ok_clicked(Tfiles_advanced * tfs)
 	g_object_unref(baseuri);
 
 	tfs->bfwin->session->adv_open_recursive = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tfs->recursive));
-	tfs->bfwin->session->adv_open_matchname = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tfs->matchname));
+	tfs->bfwin->session->adv_open_matchname = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tfs->matchname));
 	return retval;
 }
 
@@ -121,7 +121,7 @@ static void files_advanced_win_select_basedir_lcb(GtkWidget * widget, Tfiles_adv
 
 void files_advanced_win(Tbfwin * bfwin, gchar * basedir)
 {
-	GtkWidget *alignment, *button, *table, *vbox, *vbox2;
+	GtkWidget *alignment, *button, *carea, *table, *vbox, *vbox2;
 	GtkListStore *lstore;
 	GtkTreeIter iter;
 	Tfiles_advanced *tfs;
@@ -149,17 +149,18 @@ void files_advanced_win(Tbfwin * bfwin, gchar * basedir)
 	tfs = g_new(Tfiles_advanced, 1);
 	tfs->bfwin = bfwin;
 
-	tfs->dialog =
-		gtk_dialog_new_with_buttons(_("Advanced open file selector"),
-									GTK_WINDOW(tfs->bfwin->main_window),
-									GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-									GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
+	tfs->dialog = gtk_dialog_new_with_buttons(_("Advanced open file selector"),
+			GTK_WINDOW(tfs->bfwin->main_window),
+			GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 
 	gtk_dialog_set_has_separator(GTK_DIALOG(tfs->dialog), FALSE);
+	carea = gtk_dialog_get_content_area(GTK_DIALOG(tfs->dialog));
 
 	alignment = gtk_alignment_new(0, 0, 1, 1);
 	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 12, 0, 12, 6);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(tfs->dialog)->vbox), alignment, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(carea), alignment, FALSE, FALSE, 0);
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(alignment), vbox);
 
@@ -194,8 +195,8 @@ void files_advanced_win(Tbfwin * bfwin, gchar * basedir)
 
 	table = dialog_table_in_vbox(3, 2, 0, vbox2, FALSE, FALSE, 0);
 
-	tfs->matchname = checkbut_with_value(NULL, tfs->bfwin ? tfs->bfwin->session->adv_open_matchname : TRUE);
-	dialog_mnemonic_label_in_table(_("_Match on file name only:"), tfs->matchname, table, 0, 1, 0, 1);
+	tfs->matchname = checkbut_with_value(NULL, tfs->bfwin ? !tfs->bfwin->session->adv_open_matchname : FALSE);
+	dialog_mnemonic_label_in_table(_("Pattern matches _full path:"), tfs->matchname, table, 0, 1, 0, 1);
 	gtk_table_attach(GTK_TABLE(table), tfs->matchname, 1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 
 	tfs->recursive = checkbut_with_value(NULL, tfs->bfwin ? tfs->bfwin->session->adv_open_recursive : TRUE);
@@ -208,7 +209,7 @@ void files_advanced_win(Tbfwin * bfwin, gchar * basedir)
 
 	alignment = gtk_alignment_new(0, 0, 1, 1);
 	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 12, 18, 12, 6);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(tfs->dialog)->vbox), alignment, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(carea), alignment, FALSE, FALSE, 0);
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(alignment), vbox);
 
@@ -229,7 +230,7 @@ void files_advanced_win(Tbfwin * bfwin, gchar * basedir)
 	gtk_table_attach(GTK_TABLE(table), tfs->regexwarn, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 
 	gtk_dialog_set_response_sensitive(GTK_DIALOG(tfs->dialog), GTK_RESPONSE_ACCEPT, FALSE);
-	gtk_widget_show_all(GTK_DIALOG(tfs->dialog)->vbox);
+	gtk_widget_show_all(carea);
 
 	while (gtk_dialog_run(GTK_DIALOG(tfs->dialog)) == GTK_RESPONSE_ACCEPT) {
 		if (files_advanced_win_ok_clicked(tfs))
@@ -483,18 +484,24 @@ static TcheckNsave_return doc_checkNsave_lcb(TcheckNsave_status status, GError *
 		{
 			/* we have to ask the user what to do */
 			const gchar *buttons[] = { _("_Abort save"), _("_Continue save"), NULL };
+			GFileInfo *newfinfo;
+			GError *gerror=NULL;
 			gint retval;
 			gchar *tmpstr, *utf8uri;
-			utf8uri = g_file_get_uri(doc->uri);
-			if (status == CHECKANDSAVE_ERROR_MODIFIED) {
-				tmpstr = g_strdup_printf(_("File %s has been modified on disk, overwrite?"), utf8uri);
-			} else {
-				/* TODO: this bit of code can never be reached ! it can be removed */
-				tmpstr = g_strdup_printf(_("Failed to check if %s has been modified on disk"), utf8uri);
+			
+			newfinfo = g_file_query_info(doc->uri,G_FILE_ATTRIBUTE_STANDARD_SIZE","G_FILE_ATTRIBUTE_TIME_MODIFIED,0,NULL,&gerror);
+			if (gerror) {
+				g_warning("file was modified on disk, but now an error??");
+				g_error_free(gerror);
+				return CHECKNSAVE_CONT;
 			}
+			utf8uri = g_file_get_uri(doc->uri);
+			tmpstr = modified_on_disk_warning_string(utf8uri, doc->fileinfo, newfinfo);
+			/*g_strdup_printf(_("File %s has been modified on disk, overwrite?"), utf8uri);*/
 			g_free(utf8uri);
+			g_object_unref(newfinfo);
 			retval =
-				message_dialog_new_multi(BFWIN(doc->bfwin)->main_window, GTK_MESSAGE_WARNING, buttons, "",
+				message_dialog_new_multi(BFWIN(doc->bfwin)->main_window, GTK_MESSAGE_WARNING, buttons, _("File changed on disk\n"),
 										 tmpstr);
 			g_free(tmpstr);
 			if (retval == 0) {
@@ -812,7 +819,8 @@ void doc_save_backend(Tdocument * doc, gboolean do_save_as, gboolean do_move, gb
  **/
 void file_save_cb(GtkWidget * widget, Tbfwin * bfwin)
 {
-	doc_save_backend(bfwin->current_document, FALSE, FALSE, FALSE, FALSE);
+	if (bfwin->current_document)
+		doc_save_backend(bfwin->current_document, FALSE, FALSE, FALSE, FALSE);
 }
 
 /**
@@ -826,7 +834,8 @@ void file_save_cb(GtkWidget * widget, Tbfwin * bfwin)
  **/
 void file_save_as_cb(GtkWidget * widget, Tbfwin * bfwin)
 {
-	doc_save_backend(bfwin->current_document, TRUE, FALSE, FALSE, FALSE);
+	if (bfwin->current_document)
+		doc_save_backend(bfwin->current_document, TRUE, FALSE, FALSE, FALSE);
 }
 
 /**
@@ -840,7 +849,8 @@ void file_save_as_cb(GtkWidget * widget, Tbfwin * bfwin)
  **/
 void file_move_to_cb(GtkWidget * widget, Tbfwin * bfwin)
 {
-	doc_save_backend(bfwin->current_document, TRUE, TRUE, FALSE, FALSE);
+	if (bfwin->current_document)
+		doc_save_backend(bfwin->current_document, TRUE, TRUE, FALSE, FALSE);
 }
 
 /**
@@ -999,7 +1009,8 @@ gboolean doc_close_single_backend(Tdocument * doc, gboolean delay_activate, gboo
  **/
 void file_close_cb(GtkWidget * widget, Tbfwin * bfwin)
 {
-	doc_close_single_backend(bfwin->current_document, FALSE, FALSE);
+	if (bfwin->current_document)
+		doc_close_single_backend(bfwin->current_document, FALSE, FALSE);
 }
 
 void doc_close_multiple_backend(Tbfwin * bfwin, gboolean close_window, Tclose_mode close_mode)
@@ -1081,18 +1092,12 @@ void file_close_all_cb(GtkWidget * widget, Tbfwin * bfwin)
 void file_new_cb(GtkWidget * widget, Tbfwin * bfwin)
 {
 	Tdocument *doc;
-	doc = doc_new(bfwin, FALSE);
-	switch_to_document_by_pointer(bfwin, doc);
-	if (bfwin->project && bfwin->project->template && strlen(bfwin->project->template) > 2) {
-		GFile *uri;
-		uri = g_file_new_for_uri(bfwin->project->template);
-		if (uri) {
-			file_into_doc(bfwin->current_document, uri, TRUE, FALSE);
-			g_object_unref(uri);
-		}
-		/*doc_file_to_textbox(doc, bfwin->project->template, FALSE, FALSE);
-		   doc_activate(doc); */
+	GFile *template=NULL;
+	if (bfwin->session->template && bfwin->session->template[0]) {
+		template = g_file_new_for_commandline_arg(bfwin->session->template);
 	}
+	doc = doc_new_with_template(bfwin, template, TRUE);
+	switch_to_document_by_pointer(bfwin, doc);
 }
 
 static void file_reload_all_modified_check_lcb(Tcheckmodified_status status, GError *gerror,
@@ -1188,11 +1193,18 @@ static void sync_dialog_response_lcb(GtkDialog *dialog,gint response_id,gpointer
 
 void sync_dialog(Tbfwin *bfwin) {
 	Tsyncdialog *sd;
-	GtkWidget *hbox;
+	GtkWidget *carea, *hbox;
+
 	sd = g_new0(Tsyncdialog,1);
 	sd->bfwin = bfwin;
-	sd->dialog = gtk_dialog_new_with_buttons(_("Upload / Download"),GTK_WINDOW(bfwin->main_window),GTK_DIALOG_DESTROY_WITH_PARENT
-				,_("Upload"),1,_("Download"),2,GTK_STOCK_CLOSE,GTK_RESPONSE_CLOSE,NULL);
+	sd->dialog = gtk_dialog_new_with_buttons(_("Upload / Download"),
+			GTK_WINDOW(bfwin->main_window),
+			GTK_DIALOG_DESTROY_WITH_PARENT,
+			_("Upload"),1,_("Download"), 2,
+			GTK_STOCK_CLOSE,GTK_RESPONSE_CLOSE,NULL);
+
+	carea = gtk_dialog_get_content_area(GTK_DIALOG(sd->dialog));
+
 #if !GLIB_CHECK_VERSION(2, 18, 0)
 	if (glib_major_version == 2 && glib_minor_version < 18) {
 		gchar *message;
@@ -1201,7 +1213,7 @@ void sync_dialog(Tbfwin *bfwin) {
 		gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
 		gtk_label_set_markup(GTK_LABEL(label), message);
 		g_free(message);
-		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sd->dialog)->vbox), label, FALSE,FALSE,4);
+		gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(sd->dialog))), label, FALSE,FALSE,4);
 	}
 #endif
 	hbox = gtk_hbox_new(FALSE,4);
@@ -1209,23 +1221,23 @@ void sync_dialog(Tbfwin *bfwin) {
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Local directory")), FALSE,FALSE,4);
 	gtk_box_pack_start(GTK_BOX(hbox), sd->entry_local, TRUE,TRUE,4);
 	gtk_box_pack_start(GTK_BOX(hbox), file_but_new2(sd->entry_local, 1, bfwin,GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER), FALSE,FALSE,4);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sd->dialog)->vbox), hbox, FALSE,FALSE,4);
+	gtk_box_pack_start(GTK_BOX(carea), hbox, FALSE,FALSE,4);
 
 	hbox = gtk_hbox_new(FALSE,4);
 	sd->entry_remote = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Remote directory")), FALSE,FALSE,4);
 	gtk_box_pack_start(GTK_BOX(hbox), sd->entry_remote, TRUE,TRUE,4);
 	gtk_box_pack_start(GTK_BOX(hbox), file_but_new2(sd->entry_remote, 1, bfwin,GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER), FALSE,FALSE,4);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sd->dialog)->vbox), hbox, FALSE,FALSE,4);
+	gtk_box_pack_start(GTK_BOX(carea), hbox, FALSE,FALSE,4);
 	
-	sd->delete_deprecated = boxed_checkbut_with_value(_("Delete deprecated files"), bfwin->session->sync_delete_deprecated, GTK_DIALOG(sd->dialog)->vbox);
-	sd->include_hidden = boxed_checkbut_with_value(_("Include hidden files"), bfwin->session->sync_include_hidden, GTK_DIALOG(sd->dialog)->vbox);
+	sd->delete_deprecated = boxed_checkbut_with_value(_("Delete deprecated files"), bfwin->session->sync_delete_deprecated, carea);
+	sd->include_hidden = boxed_checkbut_with_value(_("Include hidden files"), bfwin->session->sync_include_hidden, carea);
 
 	sd->messagelabel = gtk_label_new(NULL);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sd->dialog)->vbox), sd->messagelabel, FALSE,FALSE,4);
+	gtk_box_pack_start(GTK_BOX(carea), sd->messagelabel, FALSE, FALSE, 4);
 
 	sd->progress = gtk_progress_bar_new();
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sd->dialog)->vbox), sd->progress, FALSE,FALSE,4);
+	gtk_box_pack_start(GTK_BOX(carea), sd->progress, FALSE, FALSE, 4);
 	
 	if (bfwin->session->sync_local_uri && bfwin->session->sync_local_uri[0]!='\0') {
 		gtk_entry_set_text(GTK_ENTRY(sd->entry_local), bfwin->session->sync_local_uri);
@@ -1241,6 +1253,119 @@ void sync_dialog(Tbfwin *bfwin) {
 	gtk_widget_show_all(sd->dialog);
 }
 
+static gchar *modified_on_disk_warning_string(const gchar *filename, GFileInfo *oldfinfo, GFileInfo *newfinfo) {
+	gchar *tmpstr, *oldtimestr, *newtimestr;
+	time_t newtime,oldtime;
+	gsize oldsize,newsize;
+
+	newtime = (time_t)g_file_info_get_attribute_uint64(newfinfo,G_FILE_ATTRIBUTE_TIME_MODIFIED);
+	oldtime = (time_t)g_file_info_get_attribute_uint64(oldfinfo,G_FILE_ATTRIBUTE_TIME_MODIFIED);
+	newtimestr = bf_portable_time(&newtime);
+	oldtimestr = bf_portable_time(&oldtime);
+	newsize = g_file_info_get_size(newfinfo);
+	oldsize = g_file_info_get_size(oldfinfo);
+	/*g_print("oldtimestr=%s, newtimestr=%s\n",oldtimestr,newtimestr);*/
+	tmpstr = g_strdup_printf(_("Filename:%s changed on disk.\n\n"
+										"Original modification time was %s\n"
+										"New modification time is %s\n"
+										"Original size was %lu\n"
+										"New size is %lu"), 
+						filename, 
+						oldtimestr, newtimestr,
+						oldsize, newsize);
+	g_free(newtimestr);
+	g_free(oldtimestr);
+	return tmpstr;
+}
+
+static void doc_activate_modified_lcb(Tcheckmodified_status status,GError *gerror,GFileInfo *orig, GFileInfo *new, gpointer callback_data) {
+	Tdocument *doc = callback_data;
+	switch (status) {
+	case CHECKMODIFIED_ERROR:
+		DEBUG_MSG("doc_activate_modified_lcb, CHECKMODIFIED_ERROR ??\n");
+		if (gerror->code == G_IO_ERROR_NOT_FOUND) {
+			gchar *tmpstr;
+			gint retval;
+			const gchar *buttons[] = {_("_Unset file name"),_("_Save"), NULL};
+			/* file is deleted on disk, what do we do now ? */
+			tmpstr = g_strdup_printf(_("File name: %s"), gtk_label_get_text(GTK_LABEL(doc->tab_menu)));
+			retval = message_dialog_new_multi(BFWIN(doc->bfwin)->main_window,
+													 GTK_MESSAGE_WARNING,
+													 buttons,
+													 _("File disappeared from disk\n"),
+													 tmpstr);
+			g_free(tmpstr);
+			if (retval == 1) { /* save */
+				doc_save_backend(doc, FALSE, FALSE, FALSE, FALSE);
+			} else { /* unset */
+				document_unset_filename(doc);
+			}
+		} else {
+			/* TODO: warn the user */	
+		}
+	break;
+	case CHECKMODIFIED_CANCELLED:
+		DEBUG_MSG("doc_activate_modified_lcb, CHECKMODIFIED_CANCELLED\n");
+	break;
+	case CHECKMODIFIED_MODIFIED:
+		{
+		gchar *tmpstr/*, *oldtimestr, *newtimestr*/;
+		gint retval;
+		const gchar *buttons[] = {_("_Ignore"),_("_Reload"),_("Check and reload all documents"), NULL};
+		/*time_t newtime,origtime;
+		
+		newtime = (time_t)g_file_info_get_attribute_uint64(new,G_FILE_ATTRIBUTE_TIME_MODIFIED);
+		origtime = (time_t)g_file_info_get_attribute_uint64(orig,G_FILE_ATTRIBUTE_TIME_MODIFIED);
+		g_print("doc_activate_modified_lcb, newtime=%ld,%d origtime=%ld,%d newsize=%"G_GOFFSET_FORMAT" origsize=%"G_GOFFSET_FORMAT"\n",
+							newtime,g_file_info_get_attribute_uint32(new,G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC),
+							origtime,g_file_info_get_attribute_uint32(orig,G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC),
+							g_file_info_get_size(new),g_file_info_get_size(orig));
+		newtimestr = bf_portable_time(&newtime);
+		oldtimestr = bf_portable_time(&origtime);
+
+		tmpstr = g_strdup_printf(_("Filename: %s\n\nNew modification time is: %s\nOld modification time is: %s"), gtk_label_get_text(GTK_LABEL(doc->tab_menu)), newtimestr, oldtimestr);
+		*/
+		tmpstr = modified_on_disk_warning_string(gtk_label_get_text(GTK_LABEL(doc->tab_menu)), orig, new);
+		retval = message_dialog_new_multi(BFWIN(doc->bfwin)->main_window,
+													 GTK_MESSAGE_WARNING,
+													 buttons,
+													 _("File changed on disk\n"),
+													 tmpstr);
+		g_free(tmpstr);
+		/*g_free(newtimestr);
+		g_free(oldtimestr);*/
+		if (retval == 0) { /* ignore */ 
+			/*if (doc->fileinfo) {
+				g_object_unref(doc->fileinfo);
+			}
+			doc->fileinfo = new;
+			g_object_ref(doc->fileinfo);*/
+			GTimeVal mtime;
+			g_file_info_set_size(doc->fileinfo, g_file_info_get_size(new));
+			g_file_info_get_modification_time(new, &mtime);
+			g_file_info_set_modification_time(doc->fileinfo, &mtime);
+			g_file_info_set_attribute_string(doc->fileinfo, "etag::value",g_file_info_get_attribute_string(new, "etag::value"));
+			doc_set_tooltip(doc);
+		} else if (retval == 1) { /* reload */
+			doc_reload(doc, new, FALSE);
+		} else { /* reload all modified documents */
+			file_reload_all_modified(doc->bfwin);
+		}
+		}
+	break;
+	case CHECKMODIFIED_OK:
+		/* do nothing */
+	break;
+	}
+	doc->action.checkmodified = NULL;
+}
+
+void doc_start_modified_check(Tdocument *doc) {
+	if (doc->uri && doc->fileinfo && !doc->action.checkmodified && !doc->action.save) { /* don't check during another check, or during save */
+		doc->action.checkmodified = file_checkmodified_uri_async(doc->uri, doc->fileinfo, doc_activate_modified_lcb, doc);
+	}
+}
+
 static gboolean modified_on_disk_check_lcb(gpointer data) {
 	GList *tmplist = g_list_first(main_v->bfwinlist);
 	while (tmplist) {
