@@ -1,11 +1,11 @@
 /* Bluefish HTML Editor
  * filebrowser2.c - the filebrowser v2
  *
- * Copyright (C) 2002,2003,2004,2005,2006,2007,2008,2009 Olivier Sessink
+ * Copyright (C) 2002-2010 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*#define DEBUG*/
@@ -53,7 +52,7 @@ alex: g_hash_table_new(gnome_vfs_uri_hash, gnome_vfs_uri_hequal) is what you're
 #include "gtk_easy.h"			/* destroy_disposable_menu_cb() */
 #include "menu.h"				/* menu_translate() */
 #include "project.h"
-#include "stringlist.h"			/* count_array() */
+#include "stringlist.h"			/* add_to_history_stringlist() */
 #include "filefilter.h"
 
 typedef struct {
@@ -246,7 +245,7 @@ static void set_documentroot_dialog(Tbfwin * bfwin, GFile * uri)
 									GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK,
 									GTK_RESPONSE_ACCEPT, NULL);
 	g_signal_connect(G_OBJECT(drd->win), "response", G_CALLBACK(drd_response_lcb), drd);
-	table = dialog_table_in_vbox_defaults(2, 2, 5, GTK_DIALOG(drd->win)->vbox);
+	table = dialog_table_in_vbox_defaults(2, 2, 5, gtk_dialog_get_content_area(GTK_DIALOG(drd->win)));
 	label = gtk_label_new(NULL);
 	gtk_label_set_markup(GTK_LABEL(label),
 						 _
@@ -255,6 +254,7 @@ static void set_documentroot_dialog(Tbfwin * bfwin, GFile * uri)
 
 	tmp = g_file_get_uri(uri);
 	drd->doc_entry = entry_with_text(tmp, 512);
+	gtk_entry_set_editable(GTK_ENTRY(drd->doc_entry), FALSE);
 	g_free(tmp);
 	gtk_table_attach_defaults(GTK_TABLE(table), drd->doc_entry, 1, 2, 1, 2);
 	dialog_mnemonic_label_in_table(_("Documentroot"), drd->doc_entry, table, 0, 1, 1, 2);
@@ -613,6 +613,16 @@ static void fb2_enumerate_children_lcb(GObject * source_object, GAsyncResult * r
 	}
 }
 
+static gboolean fb2_fill_dir_async_low_priority(gpointer data) {
+	Turi_in_refresh *uir=data;
+/*	g_print("start fill dir async low priority\n");*/
+	g_file_enumerate_children_async(uir->uri,
+										"standard::name,standard::display-name,standard::fast-content-type,standard::icon,standard::edit-name,standard::is-backup,standard::is-hidden,standard::type",
+										G_FILE_QUERY_INFO_NONE, G_PRIORITY_LOW, uir->cancel,
+										fb2_enumerate_children_lcb, uir);
+	return FALSE;
+}
+
 static void fb2_fill_dir_async(GtkTreeIter * parent, GFile * uri)
 {
 	if (fb2_get_uri_in_refresh(uri) == NULL) {
@@ -620,24 +630,21 @@ static void fb2_fill_dir_async(GtkTreeIter * parent, GFile * uri)
 
 		fb2_treestore_mark_children_refresh1(FB2CONFIG(main_v->fb2config)->filesystem_tstore,
 											 parent);
-
 		uir = g_slice_new0(Turi_in_refresh);
 		uir->parent = parent;
-		/*uir->p_uri = uri;*/
 		uir->uri = uri;
 		g_object_ref(uir->uri);
-		/*g_object_ref(uir->p_uri);*/
 		DEBUG_MSG("fb2_fill_dir_async, opening ");
 		DEBUG_GFILE(uir->uri, TRUE);
 		uir->cancel = g_cancellable_new();
-		g_file_enumerate_children_async(uir->uri,
+		/*g_file_enumerate_children_async(uir->uri,
 										"standard::name,standard::display-name,standard::fast-content-type,standard::icon,standard::edit-name,standard::is-backup,standard::is-hidden,standard::type",
 										G_FILE_QUERY_INFO_NONE, G_PRIORITY_LOW, uir->cancel,
-										fb2_enumerate_children_lcb, uir);
+										fb2_enumerate_children_lcb, uir);*/
 		FB2CONFIG(main_v->fb2config)->uri_in_refresh =
 		g_list_prepend(FB2CONFIG(main_v->fb2config)->uri_in_refresh, uir);
+		g_idle_add_full(G_PRIORITY_LOW,fb2_fill_dir_async_low_priority,uir, NULL);
 	}
-
 }
 
 /**
@@ -771,8 +778,8 @@ static GtkTreeIter *fb2_build_dir(GFile * uri)
 
 #ifdef DEBUG
 	if (parent_uri == NULL) {
-		g_print("parent_uri should not be NULL\n");
-		exit(123);
+		g_critical("parent_uri should not be NULL\n");
+		g_return_val_if_reached(NULL);
 	}
 #endif
 	if (parent_uri) {							/* now loop to create all directories in the treestore that were not in the treestore yet */
@@ -785,8 +792,8 @@ static GtkTreeIter *fb2_build_dir(GFile * uri)
 			while (!gfile_uri_is_parent(parent_uri, tmp2, FALSE)) {
 				GFile *tmp3 = g_file_get_parent(tmp2);
 				if (!tmp3) {
-					g_warning("tried to get parent for %s, parent_uri=%s, uri=%s\n",g_file_get_uri(tmp2),g_file_get_uri(parent_uri),g_file_get_uri(uri));
-					exit(123);
+					g_critical("uh-oh: fb2_build_dir, tried to get parent for %s, parent_uri=%s, uri=%s\n",g_file_get_uri(tmp2),g_file_get_uri(parent_uri),g_file_get_uri(uri));
+					g_return_val_if_reached(0); /* TODO: must be g_return_val_if_reached() or handle the situation */
 				}
 				g_object_unref(tmp2);
 				tmp2 = tmp3;
@@ -934,8 +941,10 @@ void fb2_focus_document(Tbfwin * bfwin, Tdocument * doc)
 		   could disturb the user... hmmm... */
 		dir_uri = g_file_get_parent(doc->uri);
 #ifdef DEVELOPMENT
-		if (!dir_uri)
-			exit(334);
+		if (!dir_uri) {
+			g_critical("dir_uri should not be NULL\n");
+			g_return_if_reached();
+		}
 #endif
 		fb2_focus_dir(FILEBROWSER2(bfwin->fb2), dir_uri, FALSE);
 		g_object_unref(dir_uri);
@@ -1771,7 +1780,7 @@ static void fb2rpopup_rpopup_action_lcb(Tfilebrowser2 * fb2, guint callback_acti
 			DEBUG_MSG("fb2rpopup_rpopup_action_lcb, fs_path=%p\n", fs_path);
 			refilter_dirlist(fb2, fs_path);
 			gtk_tree_path_free(fs_path);
-			if (fb2->bfwin->session->filebrowser_focus_follow) {
+			if (fb2->bfwin->session->filebrowser_focus_follow && fb2->bfwin->current_document) {
 				fb2_focus_document(fb2->bfwin, fb2->bfwin->current_document);
 			}
 			gtk_tree_model_get_iter_first(fb2->dir_tsort, &iter);
@@ -1784,7 +1793,8 @@ static void fb2rpopup_rpopup_action_lcb(Tfilebrowser2 * fb2, guint callback_acti
 		break;
 	case 9:
 		refilter_dirlist(fb2, NULL);
-		fb2_focus_document(fb2->bfwin, fb2->bfwin->current_document);
+		if (fb2->bfwin->current_document)
+			fb2_focus_document(fb2->bfwin, fb2->bfwin->current_document);
 		break;
 	case 10:
 		{
@@ -1795,7 +1805,7 @@ static void fb2rpopup_rpopup_action_lcb(Tfilebrowser2 * fb2, guint callback_acti
 	case 15:
 		fb2->bfwin->session->filebrowser_focus_follow =
 			gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
-		if (fb2->bfwin->session->filebrowser_focus_follow) {
+		if (fb2->bfwin->session->filebrowser_focus_follow && fb2->bfwin->current_document) {
 			fb2_focus_document(fb2->bfwin, fb2->bfwin->current_document);
 		}
 		break;
@@ -1827,9 +1837,9 @@ static void fb2rpopup_rpopup_action_lcb(Tfilebrowser2 * fb2, guint callback_acti
 
 static void fb2rpopup_filter_toggled_lcb(GtkWidget * widget, Tfilebrowser2 * fb2)
 {
-	if (GTK_CHECK_MENU_ITEM(widget)->active) {
+	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) {
 		/* loop trough the filters for a filter with this name */
-		const gchar *name = gtk_label_get_text(GTK_LABEL(GTK_BIN(widget)->child));
+		const gchar *name = gtk_label_get_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(widget))));
 		Tfilter *filter = find_filter_by_name(name);
 		DEBUG_MSG("fb2rpopup_filter_toggled_lcb, setting curfilter to %p from name %s\n", filter,
 				  name);
@@ -1928,6 +1938,7 @@ static GtkWidget *fb2_rpopup_create_menu(Tfilebrowser2 * fb2, gboolean is_direct
 	if (!is_directory) {
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(menumaker, "/Open Advanced..."),
 								 FALSE);
+		gtk_widget_set_sensitive(gtk_item_factory_get_widget(menumaker, "/Set as documentroot"), FALSE);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(menumaker, "/Set as basedir"), FALSE);
 	}
 	if (!is_file) {
@@ -2372,17 +2383,26 @@ static void fb2_file_v_drag_data_received(GtkWidget * widget, GdkDragContext * c
 {
 
 	gchar *stringdata;
+	const guchar *seldata;
+	gint format;
+	gint length;
 	GFile *destdir = fb2->currentdir;
+
 	g_object_ref(destdir);
 
 	g_signal_stop_emission_by_name(widget, "drag_data_received");
-	if ((data->length == 0) || (data->format != 8)
+
+	format = gtk_selection_data_get_format(data);
+	length = gtk_selection_data_get_length(data);
+	seldata = gtk_selection_data_get_data(data);
+
+	if ((length == 0) || (format != 8)
 		|| ((info != TARGET_STRING) && (info != TARGET_URI_LIST))) {
 		gtk_drag_finish(context, FALSE, TRUE, time);
 		return;
 	}
-	stringdata = g_strndup((gchar *) data->data, data->length);
-	DEBUG_MSG("fb2_file_v_drag_data_received, stringdata='%s', len=%d\n", stringdata, data->length);
+	stringdata = g_strndup((gchar *) seldata, length);
+	DEBUG_MSG("fb2_file_v_drag_data_received, stringdata='%s', len=%d\n", stringdata, length);
 	if (destdir) {
 		if (strchr(stringdata, '\n') == NULL) {	/* no newlines, probably a single file */
 			GSList *list = NULL;
@@ -2408,6 +2428,9 @@ static void fb2_dir_v_drag_data_received(GtkWidget * widget, GdkDragContext * co
 										 Tfilebrowser2 * fb2)
 {
 	gchar *stringdata;
+	const guchar *seldata;
+	gint format;
+	gint length;
 	GFile *destdir = NULL;
 	GtkTreePath *path;
 	/* if we don't do this, we get this text: Gtk-WARNING **: You must override the default 
@@ -2418,13 +2441,17 @@ static void fb2_dir_v_drag_data_received(GtkWidget * widget, GdkDragContext * co
 	   default handler in gtktreeview.c to get an idea what your handler should do.  */
 	g_signal_stop_emission_by_name(widget, "drag_data_received");
 
-	if ((data->length == 0) || (data->format != 8)
+	format = gtk_selection_data_get_format(data);
+	length = gtk_selection_data_get_length(data);
+	seldata = gtk_selection_data_get_data(data);
+
+	if ((length == 0) || (format != 8)
 		|| ((info != TARGET_STRING) && (info != TARGET_URI_LIST))) {
 		gtk_drag_finish(context, FALSE, TRUE, time);
 		return;
 	}
-	stringdata = g_strndup((gchar *) data->data, data->length);
-	DEBUG_MSG("fb2_dir_v_drag_data_received, stringdata='%s', len=%d\n", stringdata, data->length);
+	stringdata = g_strndup((gchar *) seldata, length);
+	DEBUG_MSG("fb2_dir_v_drag_data_received, stringdata='%s', len=%d\n", stringdata, length);
 	/* first find the destination directory */
 	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(fb2->dir_v), x, y, &path, NULL, NULL, NULL);
 	if (path) {
@@ -2700,6 +2727,9 @@ void fb2_update_settings_from_session(Tbfwin * bfwin)
 		} else {
 			fb2_set_basedir(bfwin, NULL);
 		}
+		/* TODO: fb2_set_basedir already calls refilter in most cases (not if the 
+		requested basedir was already the active basedir), so
+		we can optimise this and call refilter only when really needed. */
 		if (need_refilter) {
 			if (fb2->dir_tfilter)
 				gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(fb2->dir_tfilter));
@@ -2714,8 +2744,10 @@ GtkWidget *fb2_init(Tbfwin * bfwin)
 	Tfilebrowser2 *fb2;
 	GtkCellRenderer *renderer;
 #ifdef DEVELOPMENT
-	if (!bfwin->session)
-		exit(321);
+	if (!bfwin->session) {
+		g_critical("bfwin->session should not be NULL\n");
+		g_return_val_if_reached(NULL);
+	}
 #endif
 	fb2 = g_new0(Tfilebrowser2, 1);
 	fb2->filebrowser_viewmode = bfwin->session->filebrowser_viewmode;
@@ -2815,6 +2847,8 @@ void fb2config_init(void)
 	DEBUG_MSG("fb2config_init, finished\n");
 }
 
+#ifdef MEMORY_LEAK_DEBUG
+
 static gboolean treestore_foreach_cleanup(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data) {
 	GFile *uri;
 	GFileInfo *finfo;
@@ -2852,3 +2886,4 @@ void fb2config_cleanup(void)
 	g_object_unref(FB2CONFIG(main_v->fb2config)->filesystem_tstore);
 	g_free(main_v->fb2config);
 }
+#endif
