@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,14 +14,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*#define DEBUG*/
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+
 #include <time.h>      /* nanosleep() */
 #include <string.h>    /* strchr() */
 #include <unistd.h>    /* exit() */
@@ -29,6 +29,12 @@
 
 #include "config.h"
 
+#ifdef MAC_INTEGRATION
+/*#include <ige-mac-integration.h>*/
+#include <gtkosxapplication.h>
+#endif
+
+
 #include "bluefish.h"
 #include "bf_lib.h"         /* get_int_from_string() */
 #include "bookmark.h"
@@ -48,21 +54,13 @@
 #include "undo_redo.h"      /* undo_cb() redo_cb() etc. */
 #include "plugins.h"
 #include "rcfile.h"
+#ifdef IDENTSTORING
+#include "bftextview2_identifier.h"
+#endif /* IDENTSTORING */
 #ifdef HAVE_LIBENCHANT
 #include "bftextview2_spell.h"
 #endif /*HAVE_LIBENCHANT*/
 
-typedef struct {
-	GtkWidget *window;
-	GtkWidget *label;
-} Tsplashscreen;
-
-/******************************/
-/* global vars for this module */
-/******************************/
-#ifndef NOSPLASH
-static Tsplashscreen splashscreen;
-#endif
 /**************************/
 /* start of the functions */
 /**************************/
@@ -93,7 +91,8 @@ void notebook_show(Tbfwin *bfwin) {
 }*/
 
 static gboolean notebook_changed_activate_current_document_lcb(gpointer data) {
-	doc_activate(BFWIN(data)->current_document);
+	if (BFWIN(data)->current_document)
+		doc_activate(BFWIN(data)->current_document);
 	BFWIN(data)->notebook_changed_doc_activate_id = 0;
 	return FALSE;	
 }
@@ -145,6 +144,7 @@ void notebook_changed(Tbfwin *bfwin, gint newpage) {
 		/*g_print("notebook_changed, set enable_scanner to FALSE for doc %p\n",bfwin->current_document);*/
 		BLUEFISH_TEXT_VIEW(bfwin->current_document->view)->enable_scanner = FALSE;
 	}
+	bfwin->prev_document = bfwin->current_document;
 	bfwin->current_document = g_list_nth_data(bfwin->documentlist, cur);
 	if (bfwin->current_document == NULL) {
 		DEBUG_MSG("notebook_changed, WEIRD 2, doclist[%d] == NULL, RETURNING\n",cur);
@@ -157,10 +157,6 @@ void notebook_changed(Tbfwin *bfwin, gint newpage) {
 	if (bfwin->notebook_changed_doc_activate_id ==0) {
 		bfwin->notebook_changed_doc_activate_id = g_idle_add_full(G_PRIORITY_DEFAULT_IDLE+1, notebook_changed_activate_current_document_lcb, bfwin, NULL);
 	}
-	/* BUG: what if bfwin is free'ed in the meantime ?? this can happen during a window close.
-	we either should stop the notebook changed() signal when the window is going to be closed
-	or we should stop this idle callback */
-	DEBUG_MSG("notebook_changed, finished\n");
 }
 
 gboolean bfwin_has_doc(Tbfwin *bfwin, Tdocument *doc) {
@@ -202,6 +198,9 @@ static void notebook_switch_page_lcb(GtkWidget *notebook,GtkNotebookPage *page,g
 void gui_notebook_move(Tbfwin *bfwin, gboolean move_left) {
 	GtkWidget *tmp;
 	
+	if (!bfwin->current_document)
+		return;
+	
 	tmp = gtk_widget_get_parent(bfwin->current_document->view);
 	DEBUG_MSG("gui_notebook_move, found scrolwin %p\n",tmp);
 	if (tmp) {
@@ -335,6 +334,8 @@ void left_panel_rebuild(Tbfwin *bfwin) {
 }
 
 gboolean left_panel_show_hide_toggle(Tbfwin *bfwin,gboolean first_time, gboolean show, gboolean sync_menu) {
+	if (!bfwin->middlebox || !bfwin->notebook_box)
+		return FALSE;
 	DEBUG_MSG("left_panel_show_hide_toggle, bfwin=%p, first_time=%d, show=%d, sync_menu=%d\n",bfwin,first_time,show,sync_menu);
 	if (sync_menu) {
 		DEBUG_MSG("left_panel_show_hide_toggle, trying to sync menu\n");
@@ -352,7 +353,8 @@ gboolean left_panel_show_hide_toggle(Tbfwin *bfwin,gboolean first_time, gboolean
 		} else {
 			gtk_container_remove(GTK_CONTAINER(bfwin->hpane), bfwin->notebook_box);
 			gtk_widget_destroy(bfwin->hpane);
-			left_panel_cleanup(bfwin); 
+			left_panel_cleanup(bfwin);
+			bfwin->hpane = NULL;
 		}
 	}
 	if (show) {
@@ -388,16 +390,44 @@ gboolean left_panel_show_hide_toggle(Tbfwin *bfwin,gboolean first_time, gboolean
 	return TRUE;
 }
 
-void gui_set_title(Tbfwin *bfwin, Tdocument *doc) {
+void gui_set_title(Tbfwin *bfwin, Tdocument *doc, gint num_modified_change) {
 	gchar *title, *prfilepart;
+	gint modified_docs;
 	const gchar *tablabel;
-	tablabel = gtk_label_get_text(GTK_LABEL(doc->tab_label));
-	if (bfwin->project) {
-		prfilepart = g_strconcat(bfwin->project->name," - ",tablabel,NULL);
-	} else {
-		prfilepart = g_strdup(tablabel);
+	
+	if (!doc)
+		return;
+	
+	modified_docs = have_modified_documents(bfwin->documentlist);
+	if (num_modified_change != 0 && (modified_docs > 1 
+			|| (modified_docs==0 && num_modified_change!=-1) 
+			|| (modified_docs==1 && num_modified_change!=1))) {
+		/* no title change */
+		return;
 	}
-	title = g_strconcat(prfilepart, " - Bluefish "VERSION,NULL);
+	tablabel = gtk_label_get_text(GTK_LABEL(doc->tab_label));
+	if (doc->uri) {
+		gchar *curi = g_file_get_uri(doc->uri);
+		if (bfwin->project) {
+			prfilepart = g_strconcat(bfwin->project->name," - ",tablabel, " (", curi, ")",NULL);
+		} else {
+			prfilepart = g_strconcat(tablabel, " (", curi, ")",NULL);
+		}
+		g_free(curi);
+	} else {
+		if (bfwin->project) {
+			prfilepart = g_strconcat(bfwin->project->name," - ",tablabel, NULL);
+		} else {
+			prfilepart = g_strconcat(tablabel, NULL);
+		}
+	}
+	if (modified_docs==0) {
+		title = g_strconcat(prfilepart, " - Bluefish "VERSION,NULL);
+	} else {
+		title = g_strconcat("* ", prfilepart, " - Bluefish "VERSION,NULL);
+	}
+	if (strlen(title)> 120)
+		title[120]='\0';
 	gtk_window_set_title(GTK_WINDOW(bfwin->main_window),title);
 	/*rename_window_entry_in_all_windows(bfwin, title);*/
 	g_free(title);
@@ -459,11 +489,9 @@ void make_main_toolbar(Tbfwin *bfwin) {
 /*
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, _("Print..."), "",
 							new_pixmap(015), G_CALLBACK(file_print_cb), NULL);*/
-#ifndef WIN32
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL,
 							_("View in browser"), "",
 							new_pixmap(102), G_CALLBACK(browser_toolbar_cb), bfwin);
-#endif /* ifndef WIN32 */
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 	gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_PREFERENCES, _("Preferences..."),	
 								"", G_CALLBACK(open_preferences_cb), NULL, -1);	
@@ -477,7 +505,7 @@ void make_main_toolbar(Tbfwin *bfwin) {
 }
 
 void gui_set_undo_redo_widgets(Tbfwin *bfwin, gboolean undo, gboolean redo) {
-	if (GTK_WIDGET_VISIBLE(bfwin->main_toolbar_hb)) {
+	if (gtk_widget_get_visible(bfwin->main_toolbar_hb)) {
 		gtk_widget_set_sensitive(bfwin->toolbar_redo, redo);
 		gtk_widget_set_sensitive(bfwin->toolbar_undo, undo);
 	}
@@ -493,28 +521,36 @@ void gui_set_document_widgets(Tdocument *doc) {
 	setup_toggle_item(tmp1,("/Document/Highlight Syntax"), doc->highlightstate);
 	gui_set_undo_redo_widgets(doc->bfwin, doc_has_undo_list(doc), doc_has_redo_list(doc));
 	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
-			"/Document/Wrap", doc->wrapstate);
+			"/Document/Auto Completion Popup",
+			bluefish_text_view_get_auto_complete(BLUEFISH_TEXT_VIEW(doc->view)));
+	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
+			"/Document/Auto Indent",
+			bluefish_text_view_get_auto_indent(BLUEFISH_TEXT_VIEW(doc->view)));
+	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
+			"/Document/Show Right Margin",
+			bluefish_text_view_get_show_right_margin(BLUEFISH_TEXT_VIEW(doc->view)));
+	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
+			"/Document/Highlight block delimiters",
+			bluefish_text_view_get_show_mbhl(BLUEFISH_TEXT_VIEW(doc->view)));
 	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
 			"/Document/Line Numbers",
 			bluefish_text_view_get_show_line_numbers(BLUEFISH_TEXT_VIEW(doc->view)));
 	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
 			"/Document/Show Blocks",
 			bluefish_text_view_get_show_blocks(BLUEFISH_TEXT_VIEW(doc->view)));
-	/*setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),"/Document/Show symbols", doc->symstate);*/
-	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
-			"/Document/Auto Indent",
-			bluefish_text_view_get_auto_indent(BLUEFISH_TEXT_VIEW(doc->view)));
-	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
-			"/Document/Auto Completion Popup",
-			bluefish_text_view_get_auto_complete(BLUEFISH_TEXT_VIEW(doc->view)));
 	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
 			"/Document/Visible Spacing",
 			bluefish_text_view_get_show_visible_spacing(BLUEFISH_TEXT_VIEW(doc->view)));
+	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
+			"/Document/Wrap", doc->wrapstate);
+	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
+			"/Document/Split view", (doc->slave!=NULL));
 #ifdef HAVE_LIBENCHANT
 	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),
 			"/Document/Spell Check",
 			BLUEFISH_TEXT_VIEW(doc->view)->spell_check);
 #endif
+
 /*#ifndef USE_SCANNER	why did we not set the encoding and filetype with the scanner enabled????*/
 	menu_current_document_set_toggle_wo_activate(BFWIN(doc->bfwin),BLUEFISH_TEXT_VIEW(doc->view)->bflang, doc->encoding);
 
@@ -529,7 +565,6 @@ void gui_set_document_widgets(Tdocument *doc) {
 	menuitem_set_sensitive(BFWIN(doc->bfwin)->menubar, "/Edit/Replace Again", !doc->readonly);
 	menuitem_set_sensitive(BFWIN(doc->bfwin)->menubar, "/Edit/Indent", !doc->readonly);
 	menuitem_set_sensitive(BFWIN(doc->bfwin)->menubar, "/Edit/Unindent", !doc->readonly);
-
 }
 /*
 child : 	the child GtkWidget affected
@@ -538,20 +573,18 @@ page_num : 	the new page number for child
 static void notebook_reordered_lcb(GtkNotebook *notebook,GtkWidget *child,guint page_num,gpointer user_data) {
 	Tbfwin *bfwin = BFWIN(user_data);
 	Tdocument *doc = NULL;
-	GtkWidget *view;
 	GList *tmplist = g_list_first(bfwin->documentlist);
 	DEBUG_MSG("notebook_reordered_lcb, started\n");	
-	/* child is a gtkscrolledwindow which is a gtkbin subclass */
-	view = gtk_bin_get_child(GTK_BIN(child)); 
-	/* look where this child is in the documentlist */
+	/* look where this child (the GtkVPaned) is in the documentlist */
 	while (tmplist) {
-		if (DOCUMENT(tmplist->data)->view == view) {
+		if (DOCUMENT(tmplist->data)->vsplit == child) {
 			doc = DOCUMENT(tmplist->data);
 			break;
 		} 
 		tmplist = g_list_next(tmplist);
 	}
-	
+	if (!doc)
+		return;
 	bfwin->documentlist = g_list_remove(bfwin->documentlist,doc);
 	DEBUG_MSG("notebook_reordered_lcb, moving doc %p to position %d in the documentlist\n",doc,page_num);
 	bfwin->documentlist = g_list_insert(bfwin->documentlist, doc, page_num);
@@ -631,7 +664,7 @@ static gboolean gui_main_window_configure_event_lcb(GtkWidget *widget,GdkEvent *
 				main_v->globses.main_window_h = event->height;
 				DEBUG_MSG("gui_main_window_configure_event_lcb, not maximized, setting width=%d, height=%d\n",main_v->globses.main_window_w,main_v->globses.main_window_h);
 			}
-		} else if (revent->type == GDK_WINDOW_STATE && GTK_WIDGET_VISIBLE(bfwin->main_window) ) {
+		} else if (revent->type == GDK_WINDOW_STATE && gtk_widget_get_visible(bfwin->main_window) ) {
 			GdkEventWindowState *event = (GdkEventWindowState *)revent;
 			DEBUG_MSG("gui_main_window_configure_event_lcb, window state event, state=%d, globses_main_window_w=%d\n",event->new_window_state,main_v->globses.main_window_w);
 			if ((event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED) && main_v->globses.main_window_w > 0) {
@@ -656,12 +689,12 @@ static void main_win_on_drag_data_lcb(GtkWidget * widget, GdkDragContext * conte
 	int mode = 0;
 	gchar *stringdata;
 
-	if ((data->length == 0) || (data->format != 8) || ((info != TARGET_STRING) && (info != TARGET_URI_LIST))) {
+	if ((gtk_selection_data_get_length(data) == 0) || (gtk_selection_data_get_format(data) != 8) || ((info != TARGET_STRING) && (info != TARGET_URI_LIST))) {
 		DEBUG_MSG("on_drag_data_cb, currently unknown DnD object, need to do string comparision\n");
 		gtk_drag_finish(context, FALSE, TRUE, time);
 		return;
 	}
-	stringdata = g_strndup((gchar *)data->data, data->length);
+	stringdata = g_strndup((gchar *)gtk_selection_data_get_data(data), gtk_selection_data_get_length(data));
 	if (strchr(stringdata,'\n')) {
 		gchar **arr, **tmp;
 		tmp = arr = g_strsplit(stringdata, "\n", -1);
@@ -702,7 +735,7 @@ void gui_apply_settings(Tbfwin *bfwin) {
 		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(bfwin->leftpanel_notebook),main_v->props.leftpanel_tabposition);
 
 		/* check if the left panel needs to move over to the right */
-		if (main_v->props.left_panel_left && bfwin->leftpanel_notebook == GTK_PANED(bfwin->hpane)->child1) {
+		if (main_v->props.left_panel_left && bfwin->leftpanel_notebook == gtk_paned_get_child1(GTK_PANED(bfwin->hpane))) {
 			DEBUG_MSG("gui_apply_settings, left panel is on the right location\n");
 		} else {
 			left_panel_show_hide_toggle(bfwin,FALSE, FALSE, FALSE);
@@ -775,6 +808,9 @@ void main_window_destroy_lcb(GtkWidget *widget,Tbfwin *bfwin) {
 	DEBUG_MSG("main_window_destroy_lcb, will destroy the window now\n");
 	gtk_widget_destroy(bfwin->main_window);
 	DEBUG_MSG("main_window_destroy_lcb, going to free bfwin %p\n",bfwin);
+#ifdef IDENTSTORING
+	bftextview2_identifier_hash_destroy(bfwin);
+#endif
 	g_free(bfwin);
 	
 	if (NULL == main_v->bfwinlist) {
@@ -876,6 +912,9 @@ static void gotoline_entry_changed(GtkEditable *editable, Tbfwin *bfwin)
 	guint linecount;
 	GtkTextIter iter;
 
+	if (!bfwin->current_document)
+		return;
+
 	linestr = gtk_editable_get_chars(editable, 0, -1);
 	linenum = get_int_from_string(linestr);
 	g_free(linestr);
@@ -910,13 +949,13 @@ static void gotoline_close_button_clicked(GtkButton *button, Tbfwin* bfwin)
                                      0, 0, NULL,
                                      gotoline_entry_changed,
                                      NULL);
-                                     
-	gtk_widget_grab_focus(bfwin->current_document->view);
+	if (bfwin->current_document)                          
+		gtk_widget_grab_focus(bfwin->current_document->view);
 }
 
 void gui_gotoline_frame_show(Tbfwin *bfwin, guint callback_action, GtkWidget *widget)
 {
-	if (!GTK_WIDGET_VISIBLE(bfwin->gotoline_frame))
+	if (!gtk_widget_get_visible(bfwin->gotoline_frame))
 		gtk_widget_show(bfwin->gotoline_frame);
 
 	gtk_widget_grab_focus(bfwin->gotoline_entry);
@@ -949,6 +988,11 @@ static void gui_create_gotoline_frame(Tbfwin *bfwin) {
 void gui_create_main(Tbfwin *bfwin) {
 	GtkWidget *vbox;
 	DEBUG_MSG("gui_create_main, bfwin=%p, main_window_w=%d\n",bfwin,main_v->globses.main_window_w);
+
+#ifdef IDENTSTORING
+	bftextview2_identifier_hash_init(bfwin);
+#endif /* IDENTSTORING */
+
 	bfwin->main_window = window_full2(_("New Bluefish Window"), GTK_WIN_POS_CENTER, 0, G_CALLBACK(main_window_destroy_lcb), bfwin, FALSE, NULL);
 	gtk_window_set_role(GTK_WINDOW(bfwin->main_window), "bluefish");
 	gtk_widget_realize(bfwin->main_window);
@@ -1070,7 +1114,6 @@ void gui_create_main(Tbfwin *bfwin) {
 		gui_notebook_set_tab_accels(bfwin);
 		gui_notebook_bind_tab_signals(bfwin);
 	}
-	
 	/* everything is ready - we can start loading documents */
 	/* start to open an empty doc */
 	file_new_cb(NULL, bfwin);
@@ -1095,11 +1138,71 @@ void gui_create_main(Tbfwin *bfwin) {
 	}
 }
 
-void gui_show_main(Tbfwin *bfwin) {
-	DEBUG_MSG("gui_show_main, before show\n");
-	/* don't use show_all since some widgets are and should be hidden */
-	gtk_widget_show(bfwin->main_window);
+#ifdef MAC_INTEGRATION
+/* first move all accelerators away from <control> to <command>, and then in a second run
+move the <alt> accelerators to <control> (alt doesn't work on osx) */
+static void osx_accel_map_foreach_controltometa_lcb(gpointer data,const gchar *accel_path,guint accel_key, GdkModifierType accel_mods, gboolean changed) {
+	if (accel_mods & GDK_CONTROL_MASK) {
+		accel_mods &= (accel_mods & GDK_MOD1_MASK) ? ~GDK_MOD1_MASK : ~GDK_CONTROL_MASK;
+		accel_mods |= GDK_META_MASK;
+		if (!gtk_accel_map_change_entry(accel_path,accel_key,accel_mods,FALSE)) {
+			g_print("controltometa, could not change accelerator %s\n",accel_path);
+		}
+	}
+}
+static void osx_accel_map_foreach_mod1tocontrol_lcb(gpointer data,const gchar *accel_path,guint accel_key, GdkModifierType accel_mods, gboolean changed) {
+	if (accel_mods & GDK_MOD1_MASK) {
+		accel_mods &= ~GDK_MOD1_MASK;
+		accel_mods |= GDK_CONTROL_MASK;
+		if (!gtk_accel_map_change_entry(accel_path,accel_key,accel_mods,FALSE)) {
+			g_print("mod1tocontrol, could not change accelerator %s\n",accel_path);
+		}
+	}
+}
+#endif
 
+void gui_show_main(Tbfwin *bfwin) {
+#ifdef MAC_INTEGRATION
+	GtkWidget *menuitem;
+	GtkItemFactory *ifactory;
+
+	GtkOSXApplicationMenuGroup *group;
+	GtkOSXApplication *theApp = g_object_new(GTK_TYPE_OSX_APPLICATION, NULL);
+	gtk_widget_hide(bfwin->menubar);
+	gtk_osxapplication_set_menu_bar(theApp, GTK_MENU_SHELL(bfwin->menubar));
+
+	ifactory = gtk_item_factory_from_widget(bfwin->menubar);
+	
+	menuitem = gtk_item_factory_get_widget(ifactory, _("/Edit/Preferences"));
+	group = gtk_osxapplication_add_app_menu_group(theApp);
+	gtk_osxapplication_add_app_menu_item (theApp, group, GTK_MENU_ITEM(menuitem));
+	
+	
+	gtk_accel_map_foreach_unfiltered(NULL,osx_accel_map_foreach_controltometa_lcb);
+	gtk_accel_map_foreach_unfiltered(NULL,osx_accel_map_foreach_mod1tocontrol_lcb);
+/*	IgeMacMenuGroup *group;
+	gtk_widget_hide(bfwin->menubar);
+	
+	ige_mac_menu_set_menu_bar(GTK_MENU_SHELL(bfwin->menubar));
+
+	ifactory = gtk_item_factory_from_widget(bfwin->menubar);
+	menuitem = gtk_item_factory_get_widget(ifactory, _("/File/Quit"));
+	ige_mac_menu_set_quit_menu_item(GTK_MENU_ITEM(menuitem));
+
+	group = ige_mac_menu_add_app_menu_group ();
+	ige_mac_menu_add_app_menu_item(group,GTK_MENU_ITEM(menuitem),NULL);
+	
+	menuitem = gtk_item_factory_get_widget(ifactory, _("/Edit/Preferences"));
+	group = ige_mac_menu_add_app_menu_group();
+	ige_mac_menu_add_app_menu_item(group,GTK_MENU_ITEM(menuitem),NULL);
+	ige_mac_menu_set_global_key_handler_enabled(TRUE);*/
+
+	/* MACTODO: add focus in and focus out event so we can sync the menu
+	when we switch to a different bluefish window */
+#endif
+	DEBUG_MSG("gui_show_main, before show\n");
+	gtk_widget_show(bfwin->main_window);
+	/* don't use show_all since some widgets are and should be hidden */
 }
 /***********************/
 /* statusbar functions */
@@ -1140,6 +1243,9 @@ void go_to_line_from_selection_cb(Tbfwin *bfwin,guint callback_action, GtkWidget
 	GtkClipboard* cb;
 	gint linenum;
 
+	if (!bfwin->current_document)
+		return;
+
 	cb = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
 	string = gtk_clipboard_wait_for_text(cb);
 	if (string) {
@@ -1150,56 +1256,9 @@ void go_to_line_from_selection_cb(Tbfwin *bfwin,guint callback_action, GtkWidget
 	}
 }
 
-#ifndef NOSPLASH
-
-void splash_screen_set_label(gchar *label) {
-#ifdef DEBUG
-	g_print("Setting splash label to %s\n", label);
-#endif
-	gtk_label_set(GTK_LABEL(splashscreen.label),label);
-}
-
-GtkWidget *start_splash_screen() {
-	GtkWidget *image, *vbox;
-	GdkColor color;
-
-	splashscreen.window = window_with_title(PACKAGE_STRING, GTK_WIN_POS_CENTER_ALWAYS, 0);
-	gtk_window_set_decorated(GTK_WINDOW(splashscreen.window), FALSE);
-	gtk_window_set_role(GTK_WINDOW(splashscreen.window), "splash");
-	gtk_window_set_resizable(GTK_WINDOW(splashscreen.window),FALSE);
-	color.red = 65535;
-	color.blue = 65535;
-	color.green = 65535;
-	gtk_widget_modify_bg(splashscreen.window, GTK_STATE_NORMAL,&color);
-
-	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(splashscreen.window), vbox);
-	gtk_widget_show(vbox);
-	splashscreen.label = gtk_label_new(_("starting bluefish"));
-	gtk_box_pack_end(GTK_BOX(vbox),splashscreen.label , FALSE, FALSE, 0);
-	gtk_widget_show(splashscreen.label);
-	/*{
-		GError *error=NULL;
-		GdkPixbuf* pixbuf= gdk_pixbuf_new_from_file(BLUEFISH_SPLASH_FILENAME,&error);
-		if (error) {
-			g_print("ERROR while loading splash screen: %s\n", error->message);
-			g_error_free(error);
-		} else if (pixbuf) {
-			image = gtk_image_new_from_pixbuf(pixbuf);
-			gtk_box_pack_end(GTK_BOX(vbox), image, FALSE, FALSE, 0);
-			g_object_unref(pixbuf);
-			gtk_widget_show(image);
-		}
-	}*/
-	image = gtk_image_new_from_file(BLUEFISH_SPLASH_FILENAME);
-	gtk_box_pack_end(GTK_BOX(vbox), image, FALSE, FALSE, 0);
-	gtk_widget_show(image);
-	gtk_widget_show(splashscreen.window);
-	DEBUG_MSG("start_splash_screen, should be visible\n");
-	return splashscreen.window;
-}
-#endif /* #ifndef NOSPLASH */
 void gui_set_main_toolbar_visible(Tbfwin *bfwin, gboolean visible, gboolean sync_menu) {
+	if (!bfwin->main_toolbar_hb)
+		return;
 	if (sync_menu) {
 		DEBUG_MSG("gui_set_main_toolbar_visible, trying to sync menu\n");
 		setup_toggle_item_from_widget(bfwin->menubar, "/View/Main Toolbar", visible);
@@ -1215,11 +1274,11 @@ void tb_fullscreen_cb(GtkWidget *widget, Tbfwin *bfwin) {
 	cmi = GTK_CHECK_MENU_ITEM(gtk_item_factory_get_item(gtk_item_factory_from_widget(bfwin->menubar), "/View/Full Screen"));
 	if (!cmi)
 		return;
-	gtk_check_menu_item_set_active(cmi, !cmi->active); 
+	gtk_check_menu_item_set_active(cmi, !gtk_check_menu_item_get_active(cmi)); 
 }
 
 void gui_fullscreen_cb(Tbfwin *bfwin,guint action,GtkWidget *widget) {
-	if (GTK_CHECK_MENU_ITEM(widget)->active) {
+	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) {
 		gtk_window_fullscreen(GTK_WINDOW(bfwin->main_window));
 		gtk_widget_hide(bfwin->toolbar_fullscreen);
 		gtk_widget_show(bfwin->toolbar_normalscreen);
@@ -1231,6 +1290,8 @@ void gui_fullscreen_cb(Tbfwin *bfwin,guint action,GtkWidget *widget) {
 }
 
 void gui_statusbar_show_hide_toggle(Tbfwin *bfwin, gboolean visible, gboolean sync_menu) {
+	if (!bfwin->statusbar)
+		return;
 	if (sync_menu) {
 		setup_toggle_item_from_widget(bfwin->menubar, "/View/Statusbar", visible);
 	}
@@ -1241,7 +1302,7 @@ void gui_statusbar_show_hide_toggle(Tbfwin *bfwin, gboolean visible, gboolean sy
 }
 
 void gui_toggle_hidewidget_cb(Tbfwin *bfwin,guint action,GtkWidget *widget) {
-	gboolean active = GTK_CHECK_MENU_ITEM(widget)->active;
+	gboolean active = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
 	DEBUG_MSG("gui_toggle_hidewidget_cb, action=%d, active=%d\n",action,active);
 	switch (action) {
 	case 1:
@@ -1271,6 +1332,8 @@ void gui_toggle_hidewidget_cb(Tbfwin *bfwin,guint action,GtkWidget *widget) {
 
 Tbfwin *gui_new_window(Tproject *project) {
 	Tbfwin *bfwin = g_new0(Tbfwin,1);
+	g_print("gui_new_window, start bfwin %p\n",bfwin);
+
 	if (project) {
 		bfwin->project = project;
 		bfwin->session = project->session;
