@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*#define DEBUG*/ 
@@ -52,8 +51,6 @@ typedef struct {
 	gint output_subpat;
 	gboolean scrolled_once;
 	GRegex *reg;
-	/*pcre *pcre_c;
-	pcre_extra *pcre_s;*/
 } Toutput_def;
 
 typedef struct {
@@ -79,7 +76,8 @@ static void ob_lview_row_activated_lcb(GtkTreeView *tree, GtkTreePath *path,GtkT
 	if (file && strlen(file)) {
 		doc_new_from_input(ob->bfwin, file, FALSE,FALSE, lineval);
 	} else if (lineval > 0){
-		doc_select_line(ob->bfwin->current_document, lineval, TRUE);
+		if (ob->bfwin->current_document)
+			doc_select_line(ob->bfwin->current_document, lineval, TRUE);
 	}
 	g_free(line);
 	g_free(file);
@@ -205,6 +203,7 @@ static Toutputbox *init_output_box(Tbfwin *bfwin) {
 	GtkCellRenderer *renderer;
 	GtkWidget *vbox2, *but, *image;
 	Toutputbox *ob;
+	GtkAllocation allocation;
 	
 	ob = g_new0(Toutputbox,1);
 	DEBUG_MSG("init_output_box, created %p\n",ob);
@@ -214,7 +213,8 @@ static Toutputbox *init_output_box(Tbfwin *bfwin) {
 
 	ob->hbox = gtk_hbox_new(FALSE,0);
 	gtk_paned_add2(GTK_PANED(bfwin->vpane), ob->hbox);
-	gtk_paned_set_position(GTK_PANED(bfwin->vpane),(gint)(bfwin->vpane->allocation.height * 0.8));
+	gtk_widget_get_allocation(bfwin->vpane, &allocation);
+	gtk_paned_set_position(GTK_PANED(bfwin->vpane),(gint)(allocation.height * 0.8));
 
 	ob->lstore = gtk_list_store_new (3,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING);
 	ob->lfilter = GTK_TREE_MODEL_FILTER(gtk_tree_model_filter_new(GTK_TREE_MODEL(ob->lstore),NULL));
@@ -286,7 +286,6 @@ void fill_output_box(gpointer data, gchar *string) {
 	
 	match = g_regex_match(ob->def->reg,string,0,&match_info);
 	nummatches = g_match_info_get_match_count(match_info);
-	/*nummatches = pcre_exec(ob->def->pcre_c, ob->def->pcre_s,string, strlen(string), 0,0, ovector, 30);*/
 	if (match && nummatches > 0) {
 		/* we have a valid line */
 		gchar *filename=NULL,*line=NULL,*output=NULL;
@@ -301,15 +300,6 @@ void fill_output_box(gpointer data, gchar *string) {
 		if (ob->def->output_subpat >= 0 && nummatches >= ob->def->output_subpat) {
 			output = g_match_info_fetch(match_info, ob->def->output_subpat);
 		}
-/*		if (ob->def->file_subpat >= 0 && ovector[ob->def->file_subpat*2] >=0) {
-			pcre_get_substring(string,ovector,nummatches,ob->def->file_subpat,&filename);
-		}
-		if (ob->def->line_subpat >= 0&& ovector[ob->def->line_subpat*2] >=0) {
-			pcre_get_substring(string,ovector,nummatches,ob->def->line_subpat,&line);
-		}
-		if (ob->def->output_subpat >= 0&& ovector[ob->def->output_subpat*2] >=0) {
-			pcre_get_substring(string,ovector,nummatches,ob->def->output_subpat,&output);
-		}*/
 		DEBUG_MSG("fill_output_box, filename=%s, line=%s, output=%s\n",filename,line,output);
 		if (filename) {
 			GFile *addtolist;
@@ -353,7 +343,7 @@ void fill_output_box(gpointer data, gchar *string) {
 		GtkAdjustment* vadj;
 		DEBUG_MSG("fill_output_box, scroll to end..\n");
 		vadj = gtk_tree_view_get_vadjustment(GTK_TREE_VIEW(ob->lview));
-		gtk_adjustment_set_value(vadj, vadj->upper);		
+		gtk_adjustment_set_value(vadj, gtk_adjustment_get_upper(vadj));		
 		
 	}
 }
@@ -364,8 +354,6 @@ static void outputbox_def_cleanup(Toutputbox *ob, gboolean do_shutdown) {
 	if (ob->def->reg)
 		g_regex_unref(ob->def->reg);
 	g_free(ob->def->pattern);
-	/*pcre_free(ob->def->pcre_c);
-	pcre_free(ob->def->pcre_s);*/
 	if (ob->def->docuri) 
 		g_object_unref(ob->def->docuri);
 	g_free(ob->def);
@@ -409,7 +397,7 @@ void outputbox(Tbfwin *bfwin,gchar *pattern, gint file_subpat, gint line_subpat,
 	
 	ob->def = g_new0(Toutput_def,1);
 	ob->def->pattern = g_strdup(pattern);
-	if (bfwin->current_document->uri) {
+	if (bfwin->current_document && bfwin->current_document->uri) {
 		ob->def->docuri = bfwin->current_document->uri;
 		g_object_ref(ob->def->docuri);
 	}
@@ -419,14 +407,6 @@ void outputbox(Tbfwin *bfwin,gchar *pattern, gint file_subpat, gint line_subpat,
 	
 /*	ob->def->show_all_output = show_all_output;*/
 	
-	/*ob->def->pcre_c = pcre_compile(ob->def->pattern, PCRE_UTF8,&errptr,&erroffset,NULL);
-	if (ob->def->pcre_c == NULL) {
-		gchar *tmpstr = g_strdup_printf(_("Failed to compile outputbox pattern %s"),ob->def->pattern);
-		statusbar_message(bfwin,tmpstr,4);
-		g_free(tmpstr);
-		return;
-	}
-	ob->def->pcre_s = pcre_study(ob->def->pcre_c, 0,&errptr);*/
 	ob->def->reg = g_regex_new(ob->def->pattern, G_REGEX_OPTIMIZE, 0, &gerror);
 	if (gerror) {
 		gchar *tmpstr = g_strdup_printf(_("Failed to compile outputbox pattern %s: %s"),ob->def->pattern, gerror->message);
