@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*#define DEBUG*/
@@ -346,7 +345,7 @@ static void insert_time_callback(GtkWidget * widget, TimeInsert * timeinsert)
   insert_string = g_malloc0(32);
   final_string = g_malloc0(255);
   for (count = 1; count < 7; count++) {
-    if (GTK_TOGGLE_BUTTON(timeinsert->check[count])->active) {
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(timeinsert->check[count]))) {
       gtk_label_get(GTK_LABEL(timeinsert->label[count]), &temp_string);
       insert_string = extract_time_string(temp_string);
       strncat(final_string, insert_string, 31);
@@ -563,31 +562,31 @@ static void quickanchorok_lcb(GtkWidget * widget, Thtml_diag * dg)
   Tbfwin *bfwin = dg->bfwin;
 
   thestring = g_strdup(cap("<A"));
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[2])->child), cap("HREF"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[1])->child), cap("TARGET"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[2]), cap("NAME"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[4]), NULL, thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[5]), cap("ONCLICK"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[6]), cap("ONDBLCLICK"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[7]), cap("ONMOUSEOVER"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[8]), cap("ONMOUSEDOWN"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[9]), cap("ONMOUSEMOVE"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[10]), cap("ONMOUSEOUT"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[11]), cap("ONMOUSEUP"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[12]), cap("ONKEYDOWN"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[13]), cap("ONKEYPRESS"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[14]), cap("ONKEYUP"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->attrwidget[0])->child), cap("CLASS"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[1]), cap("ID"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[2]), cap("STYLE"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[17]), cap("LANG"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[18]), cap("TITLE"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(dg->combo[2]))), cap("HREF"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[1]), cap("TARGET"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[2]), cap("NAME"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[4]), NULL, thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[5]), cap("ONCLICK"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[6]), cap("ONDBLCLICK"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[7]), cap("ONMOUSEOVER"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[8]), cap("ONMOUSEDOWN"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[9]), cap("ONMOUSEMOVE"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[10]), cap("ONMOUSEOUT"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[11]), cap("ONMOUSEUP"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[12]), cap("ONKEYDOWN"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[13]), cap("ONKEYPRESS"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[14]), cap("ONKEYUP"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(dg->attrwidget[0]))), cap("CLASS"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->attrwidget[1]), cap("ID"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->attrwidget[2]), cap("STYLE"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[17]), cap("LANG"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[18]), cap("TITLE"), thestring, NULL);
   finalstring = g_strdup_printf("%s>", thestring);
   g_free(thestring);
 
-  bfwin->session->urllist = add_entry_to_stringlist(bfwin->session->urllist, GTK_WIDGET(GTK_BIN(dg->combo[2])->child));
-  bfwin->session->targetlist = add_entry_to_stringlist(bfwin->session->targetlist, GTK_WIDGET(GTK_BIN(dg->combo[1])->child));
-  bfwin->session->classlist = add_entry_to_stringlist(bfwin->session->classlist, GTK_WIDGET(GTK_BIN(dg->attrwidget[0])->child));
+  bfwin->session->urllist = add_entry_to_stringlist(bfwin->session->urllist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[2]))));
+  bfwin->session->targetlist = add_entry_to_stringlist(bfwin->session->targetlist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[1]))));
+  bfwin->session->classlist = add_entry_to_stringlist(bfwin->session->classlist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->attrwidget[0]))));
 
   if (dg->range.end == -1) {
     doc_insert_two_strings(dg->doc, finalstring, cap("</A>"));
@@ -624,11 +623,13 @@ void quickanchor_dialog(Tbfwin *bfwin, Ttagpopup *data) {
 
     free_stringlist(rel_link_list);
   }
-  file_but = file_but_new(GTK_WIDGET(GTK_BIN(dg->combo[2])->child), 0, bfwin);
+  file_but = file_but_new(GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[2]))), 0, bfwin);
   gtk_table_attach(GTK_TABLE(dgtable), GTK_WIDGET(file_but), 2, 3, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
   bf_mnemonic_label_tad_with_alignment(_("_HREF:"), dg->combo[2], 0, 0.5, dgtable, 0, 1, 0, 1);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[2])), 1, 2, 0, 1);
-
+	if (bfwin->session->targetlist==NULL) {
+		bfwin->session->targetlist = list_from_arglist(TRUE, "_top", "_blank", "_parent", "_selfs", NULL);
+	}
   dg->combo[1] = combobox_with_popdown(avalues[1], bfwin->session->targetlist, 1);
   bf_mnemonic_label_tad_with_alignment(_("_Target:"), dg->combo[1], 0, 0.5, dgtable, 0, 1, 1, 2);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[1])), 1, 3, 1, 2);
@@ -741,16 +742,16 @@ static void block_tag_editok_lcb(gint type, Thtml_diag * dg) {
   break;
   }
 
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[1])->child), cap("ALIGN"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[2])->child), cap("CLASS"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[1]), cap("STYLE"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[2]), cap("NAME"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[3]), cap("ID"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->entry[4]), NULL, thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[1]), cap("ALIGN"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[2]), cap("CLASS"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[1]), cap("STYLE"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[2]), cap("NAME"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[3]), cap("ID"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[4]), NULL, thestring, NULL);
   finalstring = g_strdup_printf("%s>", thestring);
   g_free(thestring);
 
-  bfwin->session->classlist = add_entry_to_stringlist(bfwin->session->classlist, GTK_WIDGET(GTK_BIN(dg->combo[2])->child));
+  bfwin->session->classlist = add_entry_to_stringlist(bfwin->session->classlist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[2]))));
 
   if (dg->range.end == -1) {
     doc_insert_two_strings(dg->doc, finalstring, endstring);
@@ -935,11 +936,11 @@ static void quickruleok_lcb(GtkWidget * widget, Thtml_diag * dg) {
   gchar *thestring, *finalstring;
 
   thestring = g_strdup(cap("<HR"));
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[1])->child), cap("ALIGN"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[1]), cap("ALIGN"), thestring, NULL);
   thestring = insert_integer_if_spin(dg->spin[1], cap("SIZE"), thestring, FALSE, 1);
-  thestring = insert_integer_if_spin(dg->spin[2], cap("WIDTH"), thestring, GTK_TOGGLE_BUTTON(dg->check[1])->active,GTK_TOGGLE_BUTTON(dg->check[1])->active ? 100 : 0);
+  thestring = insert_integer_if_spin(dg->spin[2], cap("WIDTH"), thestring, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[1])), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[1])) ? 100 : 0);
   thestring = insert_attr_if_checkbox(dg->check[2], main_v->props.xhtml == 1 ? cap("NOSHADE=\"noshade\"") : cap("NOSHADE"), thestring);
-  thestring = insert_string_if_entry(dg->entry[1], NULL, thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[1]), NULL, thestring, NULL);
   finalstring = g_strdup_printf(main_v->props.xhtml == 1 ? "%s />" : "%s>", thestring);
   g_free(thestring);
 
@@ -1014,32 +1015,32 @@ static void bodyok_lcb(GtkWidget * widget, Thtml_diag *dg) {
 
   thestring = g_strdup(cap("<BODY"));
   if (dg->entry[1]) {
-    thestring = insert_string_if_entry(dg->entry[1], cap("BACKGROUND"), thestring, NULL);
-    thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[1])->child), cap("BGCOLOR"), thestring, NULL);
-    thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[2])->child), cap("TEXT"), thestring, NULL);
-    thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[3])->child), cap("LINK"), thestring, NULL);
-    thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[4])->child), cap("VLINK"), thestring, NULL);
-    thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[5])->child), cap("ALINK"), thestring, NULL);
+    thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[1]), cap("BACKGROUND"), thestring, NULL);
+    thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[1]), cap("BGCOLOR"), thestring, NULL);
+    thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[2]), cap("TEXT"), thestring, NULL);
+    thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[3]), cap("LINK"), thestring, NULL);
+    thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[4]), cap("VLINK"), thestring, NULL);
+    thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[5]), cap("ALINK"), thestring, NULL);
   }
-  thestring = insert_string_if_entry(dg->entry[3], cap("STYLE"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[6])->child), cap("CLASS"), thestring, NULL);
-  thestring = insert_string_if_entry(dg->entry[4], cap("ID"), thestring, NULL);
-  thestring = insert_string_if_entry(dg->entry[5], cap("LANG"), thestring, NULL);
-  thestring = insert_string_if_entry(dg->entry[6], cap("ONLOAD"), thestring, NULL);
-  thestring = insert_string_if_entry(dg->entry[7], cap("ONUNLOAD"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[3]), cap("STYLE"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[6]), cap("CLASS"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[4]), cap("ID"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[5]), cap("LANG"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[6]), cap("ONLOAD"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[7]), cap("ONUNLOAD"), thestring, NULL);
 
-  thestring = insert_string_if_entry(dg->entry[2], NULL, thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[2]), NULL, thestring, NULL);
   finalstring = g_strconcat(thestring, ">\n", NULL);
   g_free(thestring);
 
   if (dg->entry[1]) {
-    bfwin->session->colorlist = add_entry_to_stringlist(bfwin->session->colorlist, GTK_WIDGET(GTK_BIN(dg->combo[1])->child));
-    bfwin->session->colorlist = add_entry_to_stringlist(bfwin->session->colorlist, GTK_WIDGET(GTK_BIN(dg->combo[2])->child));
-    bfwin->session->colorlist = add_entry_to_stringlist(bfwin->session->colorlist, GTK_WIDGET(GTK_BIN(dg->combo[3])->child));
-    bfwin->session->colorlist = add_entry_to_stringlist(bfwin->session->colorlist, GTK_WIDGET(GTK_BIN(dg->combo[4])->child));
-    bfwin->session->colorlist = add_entry_to_stringlist(bfwin->session->colorlist, GTK_WIDGET(GTK_BIN(dg->combo[5])->child));
+    bfwin->session->colorlist = add_entry_to_stringlist(bfwin->session->colorlist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[1]))));
+    bfwin->session->colorlist = add_entry_to_stringlist(bfwin->session->colorlist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[2]))));
+    bfwin->session->colorlist = add_entry_to_stringlist(bfwin->session->colorlist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[3]))));
+    bfwin->session->colorlist = add_entry_to_stringlist(bfwin->session->colorlist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[4]))));
+    bfwin->session->colorlist = add_entry_to_stringlist(bfwin->session->colorlist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[5]))));
   }
-  bfwin->session->classlist = add_entry_to_stringlist(bfwin->session->classlist, GTK_WIDGET(GTK_BIN(dg->combo[6])->child));
+  bfwin->session->classlist = add_entry_to_stringlist(bfwin->session->classlist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[6]))));
 
   if (dg->range.end == -1) {
     doc_insert_two_strings(dg->doc, finalstring, cap("</BODY>"));
@@ -1137,31 +1138,31 @@ void body_dialog(Tbfwin *bfwin, Ttagpopup *data) {
     gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->entry[1], 1, 2, 0, 1);
 
     dg->combo[1] = combobox_with_popdown(bodyvalues[1], bfwin->session->colorlist, 1);
-    color_but = color_but_new(GTK_BIN(dg->combo[1])->child, dg->dialog);
+    color_but = color_but_new(gtk_bin_get_child(GTK_BIN(dg->combo[1])), dg->dialog);
     gtk_table_attach(GTK_TABLE(dgtable[0]), color_but, 2, 3, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
     gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->combo[1], 1, 2, 1, 2);
     bf_mnemonic_label_tad_with_alignment(_("Back_ground Color:"), dg->combo[1], 0, 0.5, dgtable[0], 0, 1, 1, 2);
 
     dg->combo[2] = combobox_with_popdown(bodyvalues[2], bfwin->session->colorlist, 1);
-    color_but = color_but_new(GTK_BIN(dg->combo[2])->child, dg->dialog);
+    color_but = color_but_new(gtk_bin_get_child(GTK_BIN(dg->combo[2])), dg->dialog);
     gtk_table_attach(GTK_TABLE(dgtable[0]), color_but, 2, 3, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
     gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->combo[2], 1, 2, 2, 3);
     bf_mnemonic_label_tad_with_alignment(_("_Text Color:"), dg->combo[2], 0, 0.5, dgtable[0], 0, 1, 2, 3);
 
     dg->combo[3] = combobox_with_popdown(bodyvalues[3], bfwin->session->colorlist, 1);
-    color_but = color_but_new(GTK_BIN(dg->combo[3])->child, dg->dialog);
+    color_but = color_but_new(gtk_bin_get_child(GTK_BIN(dg->combo[3])), dg->dialog);
     gtk_table_attach(GTK_TABLE(dgtable[0]), color_but, 2, 3, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
     gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->combo[3], 1, 2, 3, 4);
     bf_mnemonic_label_tad_with_alignment(_("_Link Color:"), dg->combo[3], 0, 0.5, dgtable[0], 0, 1, 3, 4);
 
     dg->combo[4] = combobox_with_popdown(bodyvalues[4], bfwin->session->colorlist, 1);
-    color_but = color_but_new(GTK_BIN(dg->combo[4])->child, dg->dialog);
+    color_but = color_but_new(gtk_bin_get_child(GTK_BIN(dg->combo[4])), dg->dialog);
     gtk_table_attach(GTK_TABLE(dgtable[0]), color_but, 2, 3, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
     gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->combo[4], 1, 2, 4, 5);
     bf_mnemonic_label_tad_with_alignment(_("_Visited Link Color:"), dg->combo[4], 0, 0.5, dgtable[0], 0, 1, 4, 5);
 
     dg->combo[5] = combobox_with_popdown(bodyvalues[5], bfwin->session->colorlist, 1);
-    color_but = color_but_new(GTK_BIN(dg->combo[5])->child, dg->dialog);
+    color_but = color_but_new(gtk_bin_get_child(GTK_BIN(dg->combo[5])), dg->dialog);
     gtk_table_attach(GTK_TABLE(dgtable[0]), color_but, 2, 3, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
     gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->combo[5], 1, 2, 5, 6);
     bf_mnemonic_label_tad_with_alignment(_("_Active Link Color:"), dg->combo[5], 0, 0.5, dgtable[0], 0, 1, 5, 6);
@@ -1185,15 +1186,15 @@ static void metaok_lcb(GtkWidget * widget, Thtml_diag *dg)
   gchar *thestring, *finalstring;
 
   thestring = g_strdup(cap("<META"));
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[1])->child), cap("HTTP-EQUIV"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[2])->child), cap("NAME"), thestring, NULL);
-  thestring = insert_string_if_entry(dg->entry[1], cap("CONTENT"), thestring, NULL);
-  thestring = insert_string_if_entry(dg->entry[2], cap("LANG"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[1]), cap("HTTP-EQUIV"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[2]), cap("NAME"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[1]), cap("CONTENT"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[2]), cap("LANG"), thestring, NULL);
   if (main_v->props.xhtml == 1) {
-    thestring = insert_string_if_entry(dg->entry[2], cap("XML:LANG"), thestring, NULL);
+    thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[2]), cap("XML:LANG"), thestring, NULL);
   }
-  thestring = insert_string_if_entry(dg->entry[3], cap("SCHEME"), thestring, NULL);
-  thestring = insert_string_if_entry(dg->entry[4], NULL, thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[3]), cap("SCHEME"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[4]), NULL, thestring, NULL);
   finalstring = g_strconcat(thestring,main_v->props.xhtml == 1 ? " />" : ">", NULL);
   g_free(thestring);
 
@@ -1289,9 +1290,9 @@ static void generalfontdialog_lcb(gint type, GtkWidget * widget, Thtml_diag *dg)
     thestring = tmpstr;
     g_free(sizecombo);
   }
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[2])->child), cap("COLOR"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[3])->child), cap("FACE"), thestring, NULL);
-  thestring = insert_string_if_entry(dg->entry[1], NULL, thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[2]), cap("COLOR"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[3]), cap("FACE"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[1]), NULL, thestring, NULL);
   finalstring = g_strconcat(thestring, ">", NULL);
   g_free(thestring);
 
@@ -1345,10 +1346,10 @@ static void generalfontdialog_cb(gint type, Tbfwin *bfwin, Ttagpopup *data) {
   gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[1])), 1, 2, 0, 1);
   dg->spin[1] = spinbut_with_value(NULL, 0, 100, 1.0, 5.0);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[1], 2, 3, 0, 1);
-  parse_integer_for_dialog(tagvalues[0], dg->spin[1], GTK_WIDGET(GTK_BIN(dg->combo[1])->child), NULL);
+  parse_integer_for_dialog(tagvalues[0], dg->spin[1], GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[1]))), NULL);
 
   dg->combo[2] = combobox_with_popdown(tagvalues[2], bfwin->session->colorlist, 1);
-  color_but = color_but_new(GTK_BIN(dg->combo[2])->child, dg->dialog);
+  color_but = color_but_new(gtk_bin_get_child(GTK_BIN(dg->combo[2])), dg->dialog);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Color")), 6, 7, 0, 1);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[2])), 7, 9, 0, 1);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(color_but), 9, 10, 0, 1);
@@ -1480,7 +1481,7 @@ static void quicklistok_lcb(GtkWidget * widget, Thtml_diag *dg)
     temp = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dg->spin[1]));
     finalstring = g_malloc((8+temp*12)*sizeof(gchar));
 
-    active = GTK_TOGGLE_BUTTON(dg->radio[1])->active;
+    active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->radio[1]));
 
     if (active) {
       strcpy(finalstring, cap("<OL>"));
@@ -1533,12 +1534,12 @@ static void framesetdialogok_lcb(GtkWidget * widget, Thtml_diag *dg) {
   gchar *thestring, *finalstring;
 
   thestring = g_strdup(cap("<FRAMESET"));
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->entry[1])), cap("COLS"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->entry[2])), cap("ROWS"), thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(dg->entry[1])), cap("COLS"), thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(dg->entry[2])), cap("ROWS"), thestring, NULL);
   finalstring = g_strconcat(thestring, ">", NULL);
   g_free(thestring);
 
-  if ((dg->check[1]) && (GTK_TOGGLE_BUTTON(dg->check[1])->active)) {
+  if ((dg->check[1]) && (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[1])))) {
     gint rowcount=1, colcount=1, framecount=0;
     gchar *rowstr, *colstr, *tmpstr, *framestring, *tmp;
     rowstr = gtk_editable_get_chars(GTK_EDITABLE(dg->entry[1]), 0, -1);
@@ -1631,19 +1632,19 @@ static void framedialogok_lcb(GtkWidget * widget, Thtml_diag *dg) {
   gchar *thestring, *finalstring, *tmp;
 
   thestring = g_strdup(cap("<FRAME"));
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[1])->child), cap("SRC"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->combo[2])->child), cap("NAME"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->spin[0])), cap("FRAMEBORDER"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[1]), cap("SRC"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[2]), cap("NAME"), thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(dg->spin[0])), cap("FRAMEBORDER"), thestring, NULL);
   tmp = gtk_combo_box_get_active_text(GTK_COMBO_BOX(dg->combo[3]));
   thestring = insert_string_if_string(tmp, cap("SCROLLING"), thestring, NULL);
   g_free(tmp);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->spin[1])), cap("MARGINWIDTH"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->spin[2])), cap("MARGINHEIGHT"), thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(dg->spin[1])), cap("MARGINWIDTH"), thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(dg->spin[2])), cap("MARGINHEIGHT"), thestring, NULL);
   thestring = insert_attr_if_checkbox(dg->check[1], main_v->props.xhtml == 1 ? cap("NORESIZE=\"noresize\"") : cap("NORESIZE"), thestring);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->entry[1])), NULL, thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(dg->entry[1])), NULL, thestring, NULL);
 
-  bfwin->session->targetlist = add_entry_to_stringlist(bfwin->session->targetlist, GTK_WIDGET(GTK_BIN(dg->combo[2])->child));
-  bfwin->session->urllist = add_entry_to_stringlist(bfwin->session->urllist, GTK_WIDGET(GTK_BIN(dg->combo[1])->child));
+  bfwin->session->targetlist = add_entry_to_stringlist(bfwin->session->targetlist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[2]))));
+  bfwin->session->urllist = add_entry_to_stringlist(bfwin->session->urllist, GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[1]))));
 
   if (main_v->props.xhtml == 1) {
     finalstring = g_strconcat(thestring," />", NULL);
@@ -1678,7 +1679,7 @@ void frame_dialog(Tbfwin *bfwin, Ttagpopup *data) {
   dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
   dg->combo[1] = combobox_with_popdown(tagvalues[0], bfwin->session->urllist, 1);
-  file_but = file_but_new(GTK_WIDGET(GTK_BIN(dg->combo[1])->child), 0, bfwin);
+  file_but = file_but_new(GTK_WIDGET(gtk_bin_get_child(GTK_BIN(dg->combo[1]))), 0, bfwin);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(file_but), 9, 10, 0, 1);
   bf_mnemonic_label_tad_with_alignment(_("_Source:"), dg->combo[1], 0, 0.5, dgtable, 0, 1, 0, 1);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[1])), 1, 9, 0, 1);
@@ -1736,11 +1737,11 @@ static void embedok_lcb(GtkWidget * widget,Thtml_diag *dg )
   gchar *thestring, *finalstring;
 
   thestring = g_strdup(cap("<EMBED"));
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->entry[1])), cap("SRC"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->spin[1])), cap("WIDTH"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->spin[2])), cap("HEIGHT"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->spin[3])), cap("BORDER"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_BIN(dg->combo[1])->child)), cap("ALIGN"), thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(dg->entry[1])), cap("SRC"), thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(dg->spin[1])), cap("WIDTH"), thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(dg->spin[2])), cap("HEIGHT"), thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(dg->spin[3])), cap("BORDER"), thestring, NULL);
+  thestring = insert_string_if_entry((GTK_ENTRY(gtk_bin_get_child(GTK_BIN(dg->combo[1])))), cap("ALIGN"), thestring, NULL);
 
   if (main_v->props.xhtml == 1) {
     finalstring = g_strconcat(thestring, " />", NULL);
@@ -1791,10 +1792,10 @@ static void scriptok_lcb(GtkWidget * widget,Thtml_diag *dg ) {
   gchar *thestring, *finalstring, *endstring;
 
   thestring = g_strdup(cap("<SCRIPT"));
-  thestring = insert_string_if_entry(dg->entry[0], cap("SRC"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_BIN(dg->combo[0])->child, cap("LANGUAGE"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_BIN(dg->combo[1])->child, cap("TYPE"), thestring, NULL);
-  thestring = insert_string_if_entry(dg->entry[1], NULL, thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[0]), cap("SRC"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[0]), cap("LANGUAGE"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[1]), cap("TYPE"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[1]), NULL, thestring, NULL);
   finalstring = g_strconcat(thestring, ">", NULL);
   endstring = cap("</SCRIPT>");
   g_free(thestring);
@@ -1840,8 +1841,8 @@ void script_dialog(Tbfwin *bfwin, Ttagpopup *data) {
 
   dg->combo[0] = combobox_with_popdown(tagvalues[1], tmplist, 1);
   gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-  gtk_label_set_mnemonic_widget(GTK_LABEL(label), (GTK_BIN(dg->combo[0])->child));
-  gtk_entry_set_activates_default(GTK_ENTRY(GTK_BIN(dg->combo[0])->child), TRUE);
+  gtk_label_set_mnemonic_widget(GTK_LABEL(label), (gtk_bin_get_child(GTK_BIN(dg->combo[0]))));
+  gtk_entry_set_activates_default(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(dg->combo[0]))), TRUE);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), label, 0, 2, 1, 2);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[0], 2, 6, 1, 2);
 
@@ -1862,15 +1863,15 @@ static void linkdialogok_lcb(GtkWidget * widget, Thtml_diag *dg) {
   gchar *thestring, *finalstring;
 
   thestring = g_strdup(cap("<LINK"));
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->attrwidget[0])->child), cap("HREF"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[1]), cap("HREFLANG"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[2]), cap("TITLE"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->attrwidget[3])->child), cap("TYPE"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->attrwidget[4])->child), cap("REL"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->attrwidget[5])->child), cap("REV"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(GTK_BIN(dg->attrwidget[6])->child), cap("MEDIA"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[7]), cap("LANG"), thestring, NULL);
-  thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[8]), NULL, thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->attrwidget[0]), cap("HREF"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->attrwidget[1]), cap("HREFLANG"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->attrwidget[2]), cap("TITLE"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->attrwidget[3]), cap("TYPE"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->attrwidget[4]), cap("REL"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->attrwidget[5]), cap("REV"), thestring, NULL);
+  thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->attrwidget[6]), cap("MEDIA"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->attrwidget[7]), cap("LANG"), thestring, NULL);
+  thestring = insert_string_if_entry(GTK_ENTRY(dg->attrwidget[8]), NULL, thestring, NULL);
   finalstring = g_strconcat(thestring, (main_v->props.xhtml == 1) ? " />" : ">", NULL);
   g_free(thestring);
 
@@ -1918,7 +1919,7 @@ void linkdialog_dialog(Tbfwin *bfwin, Ttagpopup *data, Tlinkdialog_mode mode) {
   }
   bf_mnemonic_label_tad_with_alignment(_("_HREF:"), dg->attrwidget[0], 0, 0.5, dgtable, 0, 1, 0, 1);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[0], 1, 2, 0, 1);
-  but = file_but_new(GTK_BIN(dg->attrwidget[0])->child, 0, bfwin);
+  but = file_but_new(gtk_bin_get_child(GTK_BIN(dg->attrwidget[0])), 0, bfwin);
   gtk_table_attach_defaults(GTK_TABLE(dgtable), but, 2, 3, 0, 1);
 
   dg->attrwidget[1] = entry_with_text(tagvalues[1], 1024);
