@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <string.h>
@@ -61,9 +60,7 @@ static void htmlbar_doc_view_populate_popup(GtkTextView *textview,GtkMenu *menu,
 static void htmlbar_doc_view_button_press(GtkWidget *widget,GdkEventButton *bevent, Tdocument *doc) {
 	DEBUG_MSG("htmlbar_doc_view_button_press, called\n");
 	if (bevent->button == 3) {
-		GtkTextIter iter;
-		doc_get_iter_at_bevent(doc, bevent, &iter);
-		rpopup_bevent_in_html_code(doc, &iter); 	 
+		rpopup_bevent_in_html_code(doc);
 	}
 }
 
