@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
  
 #include <string.h>
@@ -183,11 +182,11 @@ static void htmlbar_view_lcb(Thtmlbarwin *hbw,guint action,GtkWidget *widget) {
 	Thtmlbarsession *hbs;
 	hbs = g_hash_table_lookup(htmlbar_v.lookup,hbw->bfwin->session);
 	if (hbs) {
-		hbs->view_htmlbar = GTK_CHECK_MENU_ITEM(widget)->active;
+		hbs->view_htmlbar = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
 	} else {
 		DEBUG_MSG("htmlbar_view_lcb, ERROR, no htmlbarsession ???\n");
 	}
-	htmlbar_view_toolbar(hbw, GTK_CHECK_MENU_ITEM(widget)->active);
+	htmlbar_view_toolbar(hbw, gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget)));
 }
 
 #ifdef ENABLE_NLS
@@ -473,7 +472,7 @@ void htmlbar_build_menu(Thtmlbarwin *hbw) {
 	} else {
 		DEBUG_MSG("htmlbar_build_menu, ERROR, no htmlbarsession in hasht %p for session %p!!?!?!?!?\n",htmlbar_v.lookup,bfwin->session);
 	}
-	
+	DEBUG_MSG("htmlbar_build_menu, finished\n");
 	gtk_widget_show_all(bfwin->menubar);
 }
 
@@ -754,7 +753,7 @@ static Ttoolbaritem tbi[] = {
 	{"breakclear", breakclear_clicked_lcb, pixmap_breakall, N_("Break and clear")},
 	{"nbsp", nbsp_clicked_lcb, pixmap_nbsp, N_("Non-breaking space")},
 	{"", NULL, 0, NULL}, /* spacing */
-	{"anchor...", quickanchor_clicked_lcb,pixmap_anchor , N_("Anchor...")},
+	{"anchor...", quickanchor_clicked_lcb,pixmap_anchor , N_("Anchor / Hyperlink...")},
 	{"rule...", quickrule_clicked_lcb, pixmap_hrule, N_("Rule...")},
 	{"center", center_clicked_lcb,  pixmap_center, N_("Center")},
 	{"rightjustify", rightjustify_clicked_lcb, pixmap_right, N_("Right Justify")},
