@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*#define DEBUG*/
@@ -25,6 +24,12 @@
 #include <stdlib.h>
 
 #include "bluefish.h"
+
+#ifdef MAC_INTEGRATION
+/*#include <ige-mac-integration.h>*/
+#include <gtkosxapplication.h>
+#endif
+
 #include "project.h"
 #include "stringlist.h"
 #include "gui.h"
@@ -39,6 +44,7 @@
 #include "bookmark.h"
 #include "plugins.h"
 #include "preferences.h"
+#include "bftextview2_spell.h"
 
 static void free_session(Tsessionvars *session) {
 	/* call free_session for any of the plugins that have free_session defined */
@@ -59,6 +65,7 @@ static void free_session(Tsessionvars *session) {
 	g_free(session->encoding);
 	g_free(session->last_filefilter);
 	g_free(session->default_mime_type);
+	g_free(session->template);
 	g_free(session->webroot);
 	g_free(session->documentroot);
 	g_free(session->sync_local_uri);
@@ -82,6 +89,7 @@ static void project_setup_initial_session(Tsessionvars *session, gboolean before
 		session->savedir = g_strdup(main_v->session->savedir);
 		session->last_filefilter = g_strdup(main_v->session->last_filefilter);
 		session->default_mime_type = g_strdup(main_v->session->default_mime_type);
+		session->template = g_strdup(main_v->session->template);
 #ifdef HAVE_LIBENCHANT
 		session->spell_lang = g_strdup(main_v->session->spell_lang);
 		session->spell_check_default = main_v->session->spell_check_default;
@@ -90,7 +98,7 @@ static void project_setup_initial_session(Tsessionvars *session, gboolean before
 
 	/* THE NUBER OF BYTES THAT IS COPIED HERE MUST BE EQUAL TO THE NUMBER OF INTEGERS
 	IN THE STRUCT IN bluefish.h ELSE ALL WILL FAIL */
-	memcpy(session, main_v->session, 29 * sizeof(gint));
+	memcpy(session, main_v->session, 30 * sizeof(gint));
 }
 
 Tbfwin *project_is_open(GFile *uri) {
@@ -131,9 +139,16 @@ static void setup_bfwin_for_project(Tbfwin *bfwin) {
 	bfwin->bmarkdata = bfwin->project->bmarkdata;
 	bmark_set_store(bfwin);
 	bmark_reload(bfwin);
-
+#ifdef HAVE_LIBENCHANT
+	reload_spell_dictionary(bfwin);
+#endif
 	gui_apply_session(bfwin);
 	set_project_menu_widgets(bfwin, TRUE);
+#ifdef MAC_INTEGRATION
+/*	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(doc->bfwin)->menubar));*/
+	gtk_osxapplication_sync_menubar(g_object_new(GTK_TYPE_OSX_APPLICATION, NULL));
+#endif
+
 }
 
 static void setup_bfwin_for_nonproject(Tbfwin *bfwin) {
@@ -144,10 +159,28 @@ static void setup_bfwin_for_nonproject(Tbfwin *bfwin) {
 	bmark_set_store(bfwin);
 	/* normally there is always a current_document, but this function might be called in the transition
 	after all documents are just closed */
-	if (bfwin->current_document) gui_set_title(bfwin, bfwin->current_document);
+	if (bfwin->current_document) gui_set_title(bfwin, bfwin->current_document, 0);
 
 	gui_apply_session(bfwin);
 	set_project_menu_widgets(bfwin, FALSE);
+#ifdef MAC_INTEGRATION
+/*	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));*/
+	gtk_osxapplication_sync_menubar(g_object_new(GTK_TYPE_OSX_APPLICATION, NULL));
+#endif
+}
+
+static void project_destroy(Tproject *project) {
+	DEBUG_MSG("project_destroy, project=%p, project->session=%p\n",project,project->session);
+	bookmark_data_cleanup(project->bmarkdata);
+	if (project->files) 
+		free_stringlist(project->files);
+	if (project->session)
+		free_session(project->session);
+	if (project->uri)
+		g_object_unref(project->uri);
+	if (project->name)
+		g_free(project->name);
+	g_free(project);
 }
 
 /* bfwin is allowed to be NULL for an empty project */
@@ -178,7 +211,7 @@ static Tproject *create_new_project(Tbfwin *bfwin) {
 		tmplist = g_list_first(bfwin->session->bmarks);
 		while (tmplist) {
 			gchar **entry = (gchar**)tmplist->data;
-			if (count_array(entry) > 2) {
+			if (g_strv_length(entry) > 2) {
 				GList *tmplist2 = g_list_first(prj->files);
 				while (tmplist2) {
 					if (strcmp(tmplist2->data, entry[2])==0) {
@@ -194,7 +227,6 @@ static Tproject *create_new_project(Tbfwin *bfwin) {
 			tmplist = g_list_next(tmplist);
 		}
 	}
-	prj->template = g_strdup("");
 	if (bfwin) {
 		bfwin->session = prj->session;
 		setup_bfwin_for_project(bfwin);
@@ -215,8 +247,8 @@ gboolean project_save(Tbfwin *bfwin, gboolean save_as) {
 	update_project_filelist(bfwin, bfwin->project);
 /*	bfwin->project->recentfiles = limit_stringlist(bfwin->project->recentfiles, main_v->props.max_recent_files, TRUE);*/
 
-	bfwin->project->session->searchlist = limit_stringlist(bfwin->project->session->searchlist, 10, FALSE);
-	bfwin->project->session->replacelist = limit_stringlist(bfwin->project->session->replacelist, 10, FALSE);
+	bfwin->project->session->searchlist = limit_stringlist(bfwin->project->session->searchlist, 10, TRUE);
+	bfwin->project->session->replacelist = limit_stringlist(bfwin->project->session->replacelist, 10, TRUE);
 	bfwin->project->session->urllist = limit_stringlist(bfwin->project->session->urllist, 10, TRUE);
 	bmark_store_all(bfwin);
 
@@ -298,10 +330,11 @@ void project_open_from_file(Tbfwin *bfwin, GFile *fromuri) {
 	retval = rcfile_parse_project(prj, fromuri);
 	if (!retval) {
 		DEBUG_MSG("project_open_from_file, failed parsing the project\n");
-		/* TODO: use project_destroy() */
+		project_destroy(prj);
+		/* TODO: use project_destroy() 
 		bookmark_data_cleanup(prj->bmarkdata);
 		g_free(prj->session);
-		g_free(prj);
+		g_free(prj);*/
 		return;
 	}
 	add_to_recent_list(bfwin,fromuri, FALSE, TRUE);
@@ -314,7 +347,8 @@ void project_open_from_file(Tbfwin *bfwin, GFile *fromuri) {
 		prwin->session = prj->session;
 		DEBUG_MSG("project_open_from_file, project %p will be in existing prwin=%p\n",prj,bfwin);
 		/* destroy the current empty document, it should use settings from the new session */
-		doc_destroy(bfwin->current_document, TRUE);
+		if (bfwin->current_document)
+			doc_destroy(bfwin->current_document, TRUE);
 	} else {
 		/* we will open a new Bluefish window for this project */
 		DEBUG_MSG("project_open_from_file, we need a new window\n");
@@ -357,22 +391,13 @@ static void project_open(Tbfwin *bfwin) {
 	gtk_widget_show_all(dialog);
 }
 
-static void project_destroy(Tproject *project) {
-	DEBUG_MSG("project_destroy, project=%p, project->session=%p\n",project,project->session);
-	bookmark_data_cleanup(project->bmarkdata);
-	free_stringlist(project->files);
-	free_session(project->session);
-	if (project->uri)
-		g_object_unref(project->uri);
-	g_free(project->name);
-	g_free(project->template);
-	g_free(project);
-}
-
 void project_save_and_mark_closed(Tbfwin *bfwin) {
 	if (bfwin->project) {
 		project_save(bfwin, FALSE);
-		add_to_recent_list(bfwin,bfwin->project->uri, TRUE, TRUE);
+
+		if (bfwin->project->uri)
+			add_to_recent_list(bfwin,bfwin->project->uri, TRUE, TRUE);
+
 		bfwin->project->close=TRUE;
 	}
 }
@@ -421,7 +446,6 @@ gboolean project_final_close(Tbfwin *bfwin, gboolean close_win) {
 
 typedef enum {
 	name,
-	template,
 	projecteditor_entries_num
 } Tprojecteditor_entries;
 
@@ -438,10 +462,11 @@ static void project_edit_destroy_lcb(GtkWidget *widget, Tprojecteditor *pred) {
 	DEBUG_MSG("project_edit_destroy_lcb, called for pred=%p\n",pred);
 /*	gtk_widget_destroy(pred->win);*/
 	if (pred->destroy_project_on_close) {
-		project_destroy(pred->project);
 		if (pred->bfwin) {
 			setup_bfwin_for_nonproject(pred->bfwin);
 		}
+		project_destroy(pred->project);
+		pred->project=NULL;
 	}
 	if (pred->project) {
 		pred->project->editor = NULL;
@@ -462,14 +487,13 @@ static void project_edit_ok_clicked_lcb(GtkWidget *widget, Tprojecteditor *pred)
 	gtk_widget_hide(pred->win);
 	DEBUG_MSG("project_edit_ok_clicked_lcb, Tproject at %p, bfwin at %p\n",prj,pred->bfwin);
 	string_apply(&prj->name, pred->entries[name]);
-	string_apply(&prj->template, pred->entries[template]);
 	sessionprefs_apply(&pred->sprefs, pred->project->session);
 	if (pred->bfwin == NULL) {
 		pred->bfwin = gui_new_window(pred->project);
 		pred->bfwin->session = pred->project->session;
 		setup_bfwin_for_project(pred->bfwin);
 	} else {
-		gui_set_title(pred->bfwin, pred->bfwin->current_document);
+		gui_set_title(pred->bfwin, pred->bfwin->current_document, 0);
 	}
 /* set_project_menu_widgets(pred->bfwin, TRUE);*/
 	project_save(pred->bfwin,FALSE);
@@ -541,13 +565,13 @@ void project_edit(Tbfwin *bfwin) {
 	bf_mnemonic_label_tad_with_alignment(_("Project _Name:"), pred->entries[name], 1, 0.5, table, 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(table), pred->entries[name], 2, 3, 0, 1);
 
-	pred->entries[template] = entry_with_text(pred->project->template, 255);
+/*	pred->entries[template] = entry_with_text(pred->project->template, 255);
 	bf_mnemonic_label_tad_with_alignment(_("_Template:"), pred->entries[template], 1, 0.5, table, 0, 1, 3, 4);
 	but = file_but_new(pred->entries[template], 1, NULL);
 	gtk_table_attach_defaults(GTK_TABLE(table), pred->entries[template], 2, 3, 3, 4);
-	gtk_table_attach_defaults(GTK_TABLE(table), but, 3, 4, 3, 4);
+	gtk_table_attach_defaults(GTK_TABLE(table), but, 3, 4, 3, 4);*/
 
-	sessionprefs(_("Initial document settings"),&pred->sprefs, pred->project->session);
+	sessionprefs(_("<b>Project Defaults</b>"),&pred->sprefs, pred->project->session);
 	gtk_box_pack_start(GTK_BOX(vbox), pred->sprefs.frame, FALSE, FALSE, 6);
 
 	hbox = gtk_hbutton_box_new();
@@ -674,4 +698,3 @@ void project_menu_cb(Tbfwin *bfwin,guint callback_action, GtkWidget *widget) {
 	break;
 	}
 }
-
